@@ -54,9 +54,10 @@ void FGAIThermal::readFromScenario(SGPropertyNode* scFileNode) {
   setHeight(scFileNode->getDoubleValue("height-msl", 5000.0));
 }
 
-bool FGAIThermal::init() {
+bool FGAIThermal::init(bool search_in_AI_path) {
    factor = 8.0 * max_strength / (diameter * diameter * diameter);
-   return FGAIBase::init();
+   setAltitude( height );
+   return FGAIBase::init(search_in_AI_path);
 }
 
 void FGAIThermal::bind() {
@@ -87,8 +88,8 @@ void FGAIThermal::Run(double dt) {
    double user_altitude  = manager->get_user_altitude();
 
    // calculate range to target in feet and nautical miles
-   double lat_range = fabs(pos.lat() - user_latitude) * ft_per_deg_lat;
-   double lon_range = fabs(pos.lon() - user_longitude) * ft_per_deg_lon;
+   double lat_range = fabs(pos.getLatitudeDeg() - user_latitude) * ft_per_deg_lat;
+   double lon_range = fabs(pos.getLongitudeDeg() - user_longitude) * ft_per_deg_lon;
    double range_ft = sqrt(lat_range*lat_range + lon_range*lon_range);
    range = range_ft / 6076.11549;
 
