@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: controls.cxx,v 1.1 2005/11/01 13:41:49 ehofman Exp $
+// $Id: controls.cxx,v 1.3 2006-02-21 01:16:05 mfranz Exp $
 
 
 #include <simgear/compiler.h>
@@ -198,6 +198,7 @@ FGControls::init ()
 	fuel_pump[engine] = false;
 	prop_advance[engine] = 1.0;
 	magnetos[engine] = 0;
+	feed_tank[engine] = -1; // set to -1 to turn off all tanks 0 feeds all engines from center body tank
 	starter[engine] = false;
         ignition[engine] = false;
         fire_switch[engine] = false;
@@ -342,6 +343,13 @@ FGControls::bind ()
     fgTie(name, this, index,
 	 &FGControls::get_magnetos, &FGControls::set_magnetos);
     fgSetArchivable(name);
+    
+   snprintf(name, MAX_NAME_LEN,
+       "/controls/engines/engine[%d]/feed_tank", index);
+    fgTie(name, this, index,
+	 &FGControls::get_feed_tank, &FGControls::set_feed_tank);
+    fgSetArchivable(name);
+
 
     snprintf(name, MAX_NAME_LEN, "/controls/engines/engine[%d]/WEP", index);
     fgTie(name, this, index,
@@ -835,6 +843,9 @@ void FGControls::unbind ()
     snprintf(name, MAX_NAME_LEN,
              "/controls/engines/engine[%d]/throttle", index);
     fgUntie(name);
+    snprintf(name, MAX_NAME_LEN,
+             "/controls/engines/engine[%d]/feed_tank", index);
+    fgUntie(name);
     snprintf(name, MAX_NAME_LEN,
              "/controls/engines/engine[%d]/starter", index);
     fgUntie(name);
@@ -1330,6 +1341,23 @@ FGControls::set_cutoff( int engine, bool val )
     }
 }
 
+void
+FGControls::set_feed_tank( int engine, int tank )
+{ 
+    if ( engine == ALL_ENGINES ) {
+	for ( int i = 0; i < MAX_ENGINES; i++ ) {
+	    feed_tank[i] = tank;
+	    CLAMP( &feed_tank[i], -1, 4 );
+	}
+    } else {
+	if ( (engine >= 0) && (engine < MAX_ENGINES) ) {
+	    feed_tank[engine] = tank;
+	    CLAMP( &feed_tank[engine], -1, 4 );
+	}
+    } 
+ //   feed_tank[engine] = engine;
+}
+
 
 void
 FGControls::set_mixture( int engine, double pos )
