@@ -16,7 +16,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: dynamics.cxx,v 1.7 2006-03-31 16:56:14 mfranz Exp $
+// $Id: dynamics.cxx,v 1.12.2.1 2006-11-15 11:05:01 fredb Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -39,7 +39,6 @@
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 #include <Airports/runways.hxx>
-#include <simgear/xml/easyxml.hxx>
 
 #include STL_STRING
 #include <vector>
@@ -49,9 +48,6 @@ SG_USING_STD(vector);
 SG_USING_STD(sort);
 SG_USING_STD(random_shuffle);
 
-#include "parking.hxx"
-#include "groundnetwork.hxx"
-#include "runwayprefs.hxx"
 #include "dynamics.hxx"
 
 /********** FGAirport Dynamics *********************************************/
@@ -95,7 +91,6 @@ FGAirportDynamics::FGAirportDynamics(const FGAirportDynamics& other)
 // Destructor
 FGAirportDynamics::~FGAirportDynamics()
 {
-  
 }
 
 
@@ -113,6 +108,7 @@ void FGAirportDynamics::init()
   // add the gate positions to the ground network. 
   groundNetwork.addNodes(&parkings);
   groundNetwork.init();
+  groundNetwork .setTowerController(&towerController);
 }
 
 bool FGAirportDynamics::getAvailableParking(double *lat, double *lon, double *heading, int *gateId, double rad, const string &flType, const string &acType, const string &airline)
@@ -165,11 +161,19 @@ bool FGAirportDynamics::getAvailableParking(double *lat, double *lon, double *he
 	      continue;
 	    }
 	  else // Airline code doesn't match
-	    if (i->getCodes().find(airline, 0) == string::npos)
-	      {
-		available = false;
-		continue;
-	      }
+	    {
+	      //cerr << "Code = " << airline << ": Codes " << i->getCodes();
+	      if (i->getCodes().find(airline, 0) == string::npos)
+		{
+		  available = false;
+		  //cerr << "Unavailable" << endl;
+		  continue;
+		}
+	      else
+		{
+		  //cerr << "Available" << endl;
+		}
+	    }
 	  // Type doesn't match
 	  if (i->getType() != flType)
 	    {
@@ -469,11 +473,11 @@ void FGAirportDynamics::getActiveRunway(const string &trafficType, int action, s
       RunwayGroup *currRunwayGroup = 0;
       int nrActiveRunways = 0;
       time_t dayStart = fgGetLong("/sim/time/utc/day-seconds");
-      if (((dayStart - lastUpdate) > 600) || trafficType != prevTrafficType)
+      if ((abs((long)(dayStart - lastUpdate)) > 600) || trafficType != prevTrafficType)
 	{
 	  landing.clear();
 	  takeoff.clear();
-	  //lastUpdate = dayStart;
+	  lastUpdate = dayStart;
 	  prevTrafficType = trafficType;
 
 	  FGEnvironment 
