@@ -19,13 +19,15 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: runwayprefs.cxx,v 1.7 2006-03-31 16:56:14 mfranz Exp $
+// $Id: runwayprefs.cxx,v 1.7.2.3 2007-11-27 19:28:27 durk Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
 #include <math.h>
+#include <stdlib.h>
+#include <cstring>
 //#include <algorithm>
 
 #include <simgear/compiler.h>
@@ -255,12 +257,12 @@ void RunwayGroup::setActive(const string &aptId,
 		  //cerr << "Crosswnd : " << crossWind << endl;
 		  if ((tailWind > maxTail) || (crossWind > maxCross))
 		    {
-		      //cerr << "Invalid : ";
+		      //cerr << "Invalid : " << endl;
 		      validSelection = false;
 		   }
 		  else 
 		    {
-		      //cerr << "Valid   : ";
+		      //cerr << "Valid   : " << endl;;
 		  }
 		}else {
 		SG_LOG( SG_GENERAL, SG_INFO, "Failed to find runway " << name << " at " << aptId );
@@ -269,18 +271,18 @@ void RunwayGroup::setActive(const string &aptId,
 	    }
 	  if (validSelection) 
 	    {
-	      //cerr << "Valid   : ";
+	      //cerr << "Valid selection  : " << i << endl;;
 	      foundValidSelection = true;
 	      for (stringVecIterator it = currentlyActive->begin(); 
 		   it != currentlyActive->end(); it++)
 		{
 		  if ((*it) == name)
 		    match++;
-		  if (match >= bestMatch) {
-		    bestMatch = match;
-		    bestChoice = i;
-		  }
 		}
+	      if (match >= bestMatch) {
+		bestMatch = match;
+		bestChoice = i;
+	      }
 	    } 
 	  //cerr << "Preference " << i << " bestMatch " << bestMatch << " choice " << bestChoice << endl;
 	}
@@ -440,7 +442,7 @@ void  FGRunwayPreference::startXML () {
 }
 
 void  FGRunwayPreference::endXML () {
-  cout << "End XML" << endl;
+  //  cout << "End XML" << endl;
 }
 
 void  FGRunwayPreference::startElement (const char * name, const XMLAttributes &atts) {
