@@ -18,14 +18,14 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: cockpit.cxx,v 1.17 2006-02-21 01:16:07 mfranz Exp $
+// $Id: cockpit.cxx,v 1.22.2.1 2007-10-13 14:47:34 durk Exp $
 
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#ifdef HAVE_WINDOWS_H          
+#ifdef HAVE_WINDOWS_H
 #  include <windows.h>
 #endif
 
@@ -45,7 +45,7 @@
 
 #include <Aircraft/aircraft.hxx>
 #include <Include/general.hxx>
-#ifdef ENABLE_SP_FMDS
+#ifdef ENABLE_SP_FDM
 #include <FDM/SP/ADA.hxx>
 #endif
 #include <Main/globals.hxx>
@@ -77,7 +77,7 @@ float get_lat_min( void )
 {
     double a, d;
 
-    a = current_aircraft.fdm_state->get_Latitude() * SGD_RADIANS_TO_DEGREES;    
+    a = current_aircraft.fdm_state->get_Latitude() * SGD_RADIANS_TO_DEGREES;
     if (a < 0.0) {
         a = -a;
     }
@@ -99,9 +99,11 @@ get_formated_gmt_time( void )
 {
     static char buf[32];
     const struct tm *p = globals->get_time_params()->getGmt();
-    sprintf( buf, "%d/%d/%4d %d:%02d:%02d", 
+    if (snprintf( buf, 32, "%d/%d/%4d %d:%02d:%02d",
          p->tm_mon+1, p->tm_mday, 1900 + p->tm_year,
-         p->tm_hour, p->tm_min, p->tm_sec);
+         p->tm_hour, p->tm_min, p->tm_sec) >= 32) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
+    }
 
     return buf;
 }
@@ -110,12 +112,12 @@ get_formated_gmt_time( void )
 float get_long_min( void )
 {
     double  a, d;
-    a = current_aircraft.fdm_state->get_Longitude() * SGD_RADIANS_TO_DEGREES;   
+    a = current_aircraft.fdm_state->get_Longitude() * SGD_RADIANS_TO_DEGREES;
     if (a < 0.0) {
         a = -a;
     }
     d = (double) ( (int) a);
-    float lon_min = (a - d) * 60.0; 
+    float lon_min = (a - d) * 60.0;
 
     return lon_min;
 }
@@ -151,7 +153,7 @@ float get_speed( void )
     static const SGPropertyNode * speedup_node = fgGetNode("/sim/speed-up");
 
     float speed = current_aircraft.fdm_state->get_V_calibrated_kts()
-	* speedup_node->getIntValue();
+        * speedup_node->getIntValue();
 
     return speed;
 }
@@ -159,7 +161,7 @@ float get_speed( void )
 float get_mach(void)
 {
     return current_aircraft.fdm_state->get_Mach_number();
-}	
+}
 
 float get_aoa( void )
 {
@@ -228,7 +230,7 @@ float get_frame_rate( void )
 
 float get_fov( void )
 {
-    return globals->get_current_view()->get_fov(); 
+    return globals->get_current_view()->get_fov();
 }
 
 float get_vfc_ratio( void )
@@ -272,13 +274,13 @@ float get_view_direction( void )
 {
     double view_off = SGD_2PI - globals->get_current_view()->getHeadingOffset_deg() * SGD_DEGREES_TO_RADIANS;
     double view = ( current_aircraft.fdm_state->get_Psi() + view_off)
-	* SGD_RADIANS_TO_DEGREES;
-    
-    if(view > 360.)
+        * SGD_RADIANS_TO_DEGREES;
+
+    if (view > 360.)
         view -= 360.;
-    else if(view<0.)
+    else if (view<0.)
         view += 360.;
-    
+
     return view;
 }
 
@@ -341,7 +343,7 @@ float get_anzg   ( void )
     return anzg;
 }
 
-#ifdef ENABLE_SP_FMDS
+#ifdef ENABLE_SP_FDM
 int get_iaux1 (void)
 {
     FGADA *fdm = (FGADA *)current_aircraft.fdm_state;
@@ -522,158 +524,6 @@ float get_aux18 (void)
     return fdm->get_faux(10);
 }
 #endif
-// $$$ end - added, VS Renganathan 13 Oct 2K
-
-
-#ifdef NOT_USED
-/****************************************************************************/
-/* Convert degrees to dd mm'ss.s" (DMS-Format)                              */
-/****************************************************************************/
-char *dmshh_format(double degrees)
-{
-    static char buf[16];    
-    int deg_part;
-    int min_part;
-    double sec_part;
-    
-    if (degrees < 0)
-      degrees = -degrees;
-
-    deg_part = degrees;
-    min_part = 60.0 * (degrees - deg_part);
-    sec_part = 3600.0 * (degrees - deg_part - min_part / 60.0);
-
-    /* Round off hundredths */
-    if (sec_part + 0.005 >= 60.0)
-      sec_part -= 60.0, min_part += 1;
-    if (min_part >= 60)
-      min_part -= 60, deg_part += 1;
-
-    sprintf(buf,"%02d*%02d %05.2f",deg_part,min_part,sec_part);
-
-    return buf;
-}
-#endif // 0
-
-
-/************************************************************************
- Convert degrees to dd mm.mmm' (DMM-Format)
- Description: Converts using a round-off factor tailored to the required
- precision of the minutes field (three decimal places).  Round-off
- prevents function from returning a minutes value of 60.
-
- Input arguments: Coordinate value in decimal degrees
-
-************************************************************************/
-static char *toDM(float dd)
-{
-    static char  dm[16];
-    double tempdd;
-    double mn;
-    double sign = 1;
-    int deg;
-
-    if (dd < 0) {
-	sign = -1;
-    }
-    /* round for minutes expressed to three decimal places */
-    tempdd = fabs(dd) + (5.0E-4 / 60.0);
-    deg = (int)tempdd;
-    mn = fabs( (tempdd - (double)(deg)) * 60.0 - 4.999E-4 );
-    deg *= (int)sign;
-    sprintf(dm, "%d*%06.3f", deg, mn);
-    return dm;
-}
-
-
-/************************************************************************
- Convert degrees to dd mm'ss.s'' (DMS-Format)
- Description: Converts using a round-off factor tailored to the required
- precision of the seconds field (one decimal place).  Round-off
- prevents function from returning a seconds value of 60.
-
- Input arguments: Coordinate value in decimal degrees
-
-************************************************************************/
-static char *toDMS(float dd)
-{
-    static char  dms[16];
-    double tempdd, tempmin;
-    int deg;
-    int mn;
-    double sec;
-    double sign = 1;
-
-    if(dd < 0) {
-	sign = -1;
-    }
-    /* round up for seconds expressed to one decimal place */
-    tempdd = fabs(dd) + (0.05 / 3600.0);
-    deg = (int)tempdd;
-    tempmin =  (tempdd - (double)(deg)) * 60.0;
-    mn = (int)tempmin;
-    sec = fabs( (tempmin - (double)(mn)) * 60.0 - 0.049 );
-    deg *= (int)sign;
-    sprintf(dms, "%d*%02d %04.1f", deg, mn, sec);
-    return dms;
-}
-
-
-// Have to set the LatLon display type
-//static char *(*fgLatLonFormat)(float) = toDM;
-static char *(*fgLatLonFormat)(float);
-
-char *coord_format_lat(float latitude)
-{
-    static char buf[16];
-
-    sprintf(buf,"%s%c",
-//      dmshh_format(latitude),
-//      toDMS(latitude),
-//      toDM(latitude),
-        fgLatLonFormat(latitude),           
-        latitude > 0 ? 'N' : 'S');
-    return buf;
-}
-
-char *coord_format_lon(float longitude)
-{
-    static char buf[80];
-
-    sprintf(buf,"%s%c",
-//      dmshh_format(longitude),
-//      toDMS(longitude),
-//      toDM(longitude),
-        fgLatLonFormat(longitude),
-        longitude > 0 ? 'E' : 'W');
-    return buf;
-}
-
-void fgLatLonFormatToggle( puObject *)
-{
-    static int toggle = 0;
-
-    if ( toggle ) 
-        fgLatLonFormat = toDM;
-    else
-        fgLatLonFormat = toDMS;
-    
-    toggle = ~toggle;
-}
-
-#ifdef NOT_USED
-char *coord_format_latlon(double latitude, double longitude)
-{
-    static char buf[1024];
-
-    sprintf(buf,"%s%c %s%c",
-        dmshh_format(latitude),
-        latitude > 0 ? 'N' : 'S',
-        dmshh_format(longitude),
-        longitude > 0 ? 'E' : 'W');
-    return buf;
-}
-#endif
 
 
 bool fgCockpitInit( fgAIRCRAFT *cur_aircraft )
@@ -685,30 +535,27 @@ bool fgCockpitInit( fgAIRCRAFT *cur_aircraft )
 
     // If aircraft has HUD specified we will get the specs from its def
     // file. For now we will depend upon hard coding in hud?
-    
+
     // We must insure that the existing instrument link is purged.
     // This is done by deleting the links in the list.
-    
+
     // HI_Head is now a null pointer so we can generate a new list from the
     // current aircraft.
 
     fgHUDInit( cur_aircraft );
     ac_cockpit = new fg_Cockpit();
-    
-    // Have to set the LatLon display type
-    fgLatLonFormat = toDM;
-    
+
     SG_LOG( SG_COCKPIT, SG_INFO,
-        "  Code " << ac_cockpit->code() << " Status " 
+        "  Code " << ac_cockpit->code() << " Status "
         << ac_cockpit->status() );
 
-	return true;
+    return true;
 }
 
 void fgCockpitUpdate( void ) {
 
     SG_LOG( SG_COCKPIT, SG_DEBUG,
-            "Cockpit: code " << ac_cockpit->code() << " status " 
+            "Cockpit: code " << ac_cockpit->code() << " status "
             << ac_cockpit->status() );
 
     static const SGPropertyNode * xsize_node = fgGetNode("/sim/startup/xsize");
@@ -719,7 +566,7 @@ void fgCockpitUpdate( void ) {
     int iwidth   = xsize_node->getIntValue();
     int iheight  = ysize_node->getIntValue();
 
-				// FIXME: inefficient
+                                // FIXME: inefficient
     if ( hud_visibility_node->getBoolValue() ) {
         // This will check the global hud linked list pointer.
         // If there is anything to draw it will.
@@ -729,3 +576,66 @@ void fgCockpitUpdate( void ) {
     glViewport( 0, 0, iwidth, iheight );
 }
 
+
+
+
+
+struct FuncTable {
+    char *name;
+    FLTFNPTR func;
+} fn_table[] = {
+    { "agl", get_agl },
+    { "aileronval", get_aileronval },
+    { "altitude", get_altitude },
+    { "anzg", get_anzg },
+    { "aoa", get_aoa },
+    { "ax", get_Ax },
+    { "climb", get_climb_rate },
+    { "elevatortrimval", get_elev_trimval },
+    { "elevatorval", get_elevatorval },
+    { "fov", get_fov },
+    { "framerate", get_frame_rate },
+    { "heading", get_heading },
+    { "latitude", get_latitude },
+    { "longitude", get_longitude },
+    { "mach", get_mach },
+    { "rudderval", get_rudderval },
+    { "speed", get_speed },
+    { "throttleval", get_throttleval },
+    { "view_direction", get_view_direction },
+    { "vfc_tris_culled", get_vfc_tris_culled },
+    { "vfc_tris_drawn", get_vfc_tris_drawn },
+#ifdef ENABLE_SP_FDM
+    { "aux1", get_aux1 },
+    { "aux2", get_aux2 },
+    { "aux3", get_aux3 },
+    { "aux4", get_aux4 },
+    { "aux5", get_aux5 },
+    { "aux6", get_aux6 },
+    { "aux7", get_aux7 },
+    { "aux8", get_aux8 },
+    { "aux9", get_aux9 },
+    { "aux10", get_aux10 },
+    { "aux11", get_aux11 },
+    { "aux12", get_aux12 },
+    { "aux13", get_aux13 },
+    { "aux14", get_aux14 },
+    { "aux15", get_aux15 },
+    { "aux16", get_aux16 },
+    { "aux17", get_aux17 },
+    { "aux18", get_aux18 },
+#endif
+    { 0, 0 },
+};
+
+
+FLTFNPTR get_func(const char *name)
+{
+    for (int i = 0; fn_table[i].name; i++)
+        if (!strcmp(fn_table[i].name, name))
+            return fn_table[i].func;
+
+    return 0;
+}
+
+
