@@ -16,7 +16,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-//  $Id: panel.cxx,v 1.40 2006-02-21 01:16:08 mfranz Exp $
+//  $Id: panel.cxx,v 1.44 2006-09-05 20:28:48 curt Exp $
 
 //JVK
 // On 2D panels all instruments include light sources were in night displayed
@@ -52,6 +52,7 @@
 #include <Main/fg_props.hxx>
 #include <Main/viewmgr.hxx>
 #include <Time/light.hxx>
+#include <GUI/new_gui.hxx>	// FGFontCache
 
 #include "hud.hxx"
 #include "panel.hxx"
@@ -64,7 +65,7 @@
 // The number of polygon-offset "units" to place between layers.  In
 // principle, one is supposed to be enough.  In practice, I find that
 // my hardware/driver requires many more.
-#define POFF_UNITS 4
+#define POFF_UNITS 8
 
 ////////////////////////////////////////////////////////////////////////
 // Local functions.
@@ -91,13 +92,13 @@ get_aspect_adjust (int xsize, int ysize)
 bool
 fgPanelVisible ()
 {
-     if(globals->get_current_panel() == 0)
+     if (globals->get_current_panel() == 0)
 	return false;
-     if(globals->get_current_panel()->getVisibility() == 0)
+     if (globals->get_current_panel()->getVisibility() == 0)
 	return false;
-     if(globals->get_viewmgr()->get_current() != 0)
+     if (globals->get_viewmgr()->get_current() != 0)
 	return false;
-     if(globals->get_current_view()->getHeadingOffset_deg() * SGD_DEGREES_TO_RADIANS != 0)
+     if (globals->get_current_view()->getHeadingOffset_deg() * SGD_DEGREES_TO_RADIANS != 0)
 	return false;
      return true;
 }
@@ -175,8 +176,6 @@ FGCroppedTexture::getTexture ()
 ////////////////////////////////////////////////////////////////////////
 
 static fntRenderer text_renderer;
-static fntTexFont *default_font = 0;
-static fntTexFont *led_font = 0;
 static sgVec4 panel_color;
 static sgVec4 emissive_panel_color = {1,1,1,1};
 
@@ -230,28 +229,6 @@ FGPanel::addInstrument (FGPanelInstrument * instrument)
 void
 FGPanel::init ()
 {
-    SGPath base_path;
-    char* envp = ::getenv( "FG_FONTS" );
-    if ( envp != NULL ) {
-        base_path.set( envp );
-    } else {
-        base_path.set( globals->get_fg_root() );
-	base_path.append( "Fonts" );
-    }
-
-    SGPath fntpath;
-
-    // Install the default font
-    fntpath = base_path;
-    fntpath.append( "typewriter.txf" );
-    default_font = new fntTexFont ;
-    default_font -> load ( (char *)fntpath.c_str() ) ;
-
-    // Install the LED font
-    fntpath = base_path;
-    fntpath.append( "led.txf" );
-    led_font = new fntTexFont ;
-    led_font -> load ( (char *)fntpath.c_str() ) ;
 }
 
 
@@ -394,7 +371,7 @@ FGPanel::draw()
   glEnable(GL_COLOR_MATERIAL);
   glEnable(GL_CULL_FACE);
   glCullFace(GL_BACK);
-  if( _enable_depth_test )
+  if ( _enable_depth_test )
       glDepthFunc(GL_ALWAYS);
   else
     glDisable(GL_DEPTH_TEST);
@@ -442,17 +419,46 @@ FGPanel::draw()
   }
 
   // Draw the instruments.
+  // Syd Adams: added instrument clipping
   instrument_list_type::const_iterator current = _instruments.begin();
   instrument_list_type::const_iterator end = _instruments.end();
 
+  GLdouble blx[4]={1.0,0.0,0.0,0.0};
+  GLdouble bly[4]={0.0,1.0,0.0,0.0};
+  GLdouble urx[4]={-1.0,0.0,0.0,0.0};
+  GLdouble ury[4]={0.0,-1.0,0.0,0.0};
+
   for ( ; current != end; current++) {
     FGPanelInstrument * instr = *current;
     glPushMatrix();
     glTranslated(instr->getXPos(), instr->getYPos(), 0);
+
+    int ix= instr->getWidth();
+    int iy= instr->getHeight();
+    glPushMatrix();
+    glTranslated(-ix/2,-iy/2,0);
+    glClipPlane(GL_CLIP_PLANE0,blx);
+    glClipPlane(GL_CLIP_PLANE1,bly);
+    glEnable(GL_CLIP_PLANE0);
+    glEnable(GL_CLIP_PLANE1);
+
+    glTranslated(ix,iy,0);
+    glClipPlane(GL_CLIP_PLANE2,urx);
+    glClipPlane(GL_CLIP_PLANE3,ury);
+    glEnable(GL_CLIP_PLANE2);
+    glEnable(GL_CLIP_PLANE3);
+    glPopMatrix();
     instr->draw();
+
     glPopMatrix();
   }
 
+  glDisable(GL_CLIP_PLANE0);
+  glDisable(GL_CLIP_PLANE1);
+  glDisable(GL_CLIP_PLANE2);
+  glDisable(GL_CLIP_PLANE3);
+
+
   // Draw yellow "hotspots" if directed to.  This is a panel authoring
   // feature; not intended to be high performance or to look good.
   if ( fgGetBool("/sim/panel-hotspots") ) {
@@ -465,7 +471,7 @@ FGPanel::draw()
 
 
   // restore some original state
-  if( _enable_depth_test )
+  if ( _enable_depth_test )
     glDepthFunc(GL_LESS);
   glPopAttrib();
   glPolygonOffset(0, 0);
@@ -896,7 +902,7 @@ FGInstrumentLayer::transform () const
 	val = t->max;
       }
 
-      if(t->table==0) {
+      if (t->table==0) {
 	val = val * t->factor + t->offset;
       } else {
 	val = t->table->interpolate(val) * t->factor + t->offset;
@@ -1011,7 +1017,7 @@ FGTexturedLayer::draw ()
 ////////////////////////////////////////////////////////////////////////
 
 FGTextLayer::FGTextLayer (int w, int h)
-  : FGInstrumentLayer(w, h), _pointSize(14.0), _font_name("default")
+  : FGInstrumentLayer(w, h), _pointSize(14.0), _font_name("Helvetica.txf")
 {
   _then.stamp();
   _color[0] = _color[1] = _color[2] = 0.0;
@@ -1033,11 +1039,10 @@ FGTextLayer::draw ()
   if (test()) {
     glColor4fv(_color);
     transform();
-    if ( _font_name == "led" && led_font != 0) {
-	text_renderer.setFont(led_font);
-    } else {
-	text_renderer.setFont(guiFntHandle);
-    }
+
+    FGFontCache *fc = globals->get_fontcache();
+    text_renderer.setFont(fc->getTexFont(_font_name.c_str()));
+
     text_renderer.setPointSize(_pointSize);
     text_renderer.begin();
     text_renderer.start3f(0, 0, 0);
@@ -1096,7 +1101,7 @@ FGTextLayer::setPointSize (float size)
 void
 FGTextLayer::setFontName(const string &name)
 {
-  _font_name = name;
+  _font_name = name + ".txf";
 }
 
 
@@ -1198,6 +1203,3 @@ FGSwitchLayer::draw ()
 
 
 // end of panel.cxx
-
-
-
