@@ -16,13 +16,13 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-//  $Id: panel_io.cxx,v 1.22 2006-02-21 01:16:09 mfranz Exp $
+//  $Id: panel_io.cxx,v 1.26 2006-08-10 11:12:39 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#ifdef HAVE_WINDOWS_H          
+#ifdef HAVE_WINDOWS_H
 #  include <windows.h>
 #endif
 
@@ -287,9 +287,9 @@ readTransformation (const SGPropertyNode * node, float w_scale, float h_scale)
   const SGPropertyNode * trans_table = node->getNode("interpolation");
   if (trans_table != 0) {
     SG_LOG( SG_COCKPIT, SG_INFO, "Found interpolation table with "
-            << trans_table->nChildren() << "children" );
+            << trans_table->nChildren() << " children" );
     t->table = new SGInterpTable();
-    for(int i = 0; i < trans_table->nChildren(); i++) {
+    for (int i = 0; i < trans_table->nChildren(); i++) {
       const SGPropertyNode * node = trans_table->getChild(i);
       if (!strcmp(node->getName(), "entry")) {
 	double ind = node->getDoubleValue("ind", 0.0);
@@ -491,7 +491,7 @@ readLayer (const SGPropertyNode * node, float w_scale, float h_scale)
     tlayer->setPointSize(pointSize);
 
 				// Set the font.
-    string fontName = node->getStringValue("font", "default");
+    string fontName = node->getStringValue("font", "Helvetica");
     tlayer->setFontName(fontName);
 
     const SGPropertyNode * chunk_group = node->getNode("chunks");
@@ -769,10 +769,10 @@ readPanel (const SGPropertyNode * root)
         FGPanelInstrument * instrument = readInstrument(node);
         if (instrument != 0)
           panel->addInstrument(instrument);
-      } else if(!strcmp(node->getName(), "special-instrument")) {
+      } else if (!strcmp(node->getName(), "special-instrument")) {
         //cout << "Special instrument found in instruments section!\n";
         const string name = node->getStringValue("name");
-        if(name == "KLN89 GPS") {
+        if (name == "KLN89 GPS") {
           //cout << "Special instrument is KLN89\n";
           
           int x = node->getIntValue("x", -1);
@@ -804,7 +804,7 @@ readPanel (const SGPropertyNode * root)
           // Warning - hardwired size!!!
           RenderArea2D* instrument = new RenderArea2D(158, 40, 158, 40, x, y);
           KLN89* gps = (KLN89*)globals->get_subsystem("kln89");
-		  if(gps == NULL) {
+		  if (gps == NULL) {
 			  gps = new KLN89(instrument);
 			  globals->add_subsystem("kln89", gps);
 		  }
