@@ -8,20 +8,20 @@
  ------------- Copyright (C) 2001  Jon S. Berndt (jsb@hal-pc.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU General Public License as published by the Free Software
+ the terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU General Public License along with
+ You should have received a copy of the GNU Lesser General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Further information about the GNU General Public License can also be found on
+ Further information about the GNU Lesser General Public License can also be found on
  the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
@@ -39,7 +39,7 @@ INCLUDES
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGJSBBase.cpp,v 1.14 2006-01-12 15:04:22 ehofman Exp $";
+static const char *IdSrc = "$Id: FGJSBBase.cpp,v 1.14.2.3 2007-05-27 13:24:32 ehofman Exp $";
 static const char *IdHdr = ID_JSBBASE;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -94,94 +94,85 @@ const double FGJSBBase::slugtolb = 32.174049;
 const double FGJSBBase::lbtoslug = 1.0/slugtolb;
 
 const string FGJSBBase::needed_cfg_version = "2.0";
-const string FGJSBBase::JSBSim_version = "0.9.10.111805";
+const string FGJSBBase::JSBSim_version = "Pre-1.0 "__DATE__" "__TIME__;
 
-std::queue <FGJSBBase::Message*> FGJSBBase::Messages;
+std::queue <FGJSBBase::Message> FGJSBBase::Messages;
 FGJSBBase::Message FGJSBBase::localMsg;
 unsigned int FGJSBBase::messageId = 0;
-unsigned int FGJSBBase::frame = 0;
 
 short FGJSBBase::debug_lvl  = 1;
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-FGJSBBase::Message* FGJSBBase::PutMessage(Message* msg)
+void FGJSBBase::PutMessage(const Message& msg)
 {
   Messages.push(msg);
-  return msg;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-FGJSBBase::Message* FGJSBBase::PutMessage(const string& text)
+void FGJSBBase::PutMessage(const string& text)
 {
-  Message *msg = new Message();
-  msg->text = text;
-  msg->messageId = messageId++;
-  msg->subsystem = "FDM";
-  msg->type = Message::eText;
+  Message msg;
+  msg.text = text;
+  msg.messageId = messageId++;
+  msg.subsystem = "FDM";
+  msg.type = Message::eText;
   Messages.push(msg);
-  return msg;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-FGJSBBase::Message* FGJSBBase::PutMessage(const string& text, bool bVal)
+void FGJSBBase::PutMessage(const string& text, bool bVal)
 {
-  Message *msg = new Message();
-  msg->text = text;
-  msg->messageId = messageId++;
-  msg->subsystem = "FDM";
-  msg->type = Message::eBool;
-  msg->bVal = bVal;
+  Message msg;
+  msg.text = text;
+  msg.messageId = messageId++;
+  msg.subsystem = "FDM";
+  msg.type = Message::eBool;
+  msg.bVal = bVal;
   Messages.push(msg);
-  return msg;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-FGJSBBase::Message* FGJSBBase::PutMessage(const string& text, int iVal)
+void FGJSBBase::PutMessage(const string& text, int iVal)
 {
-  Message *msg = new Message();
-  msg->text = text;
-  msg->messageId = messageId++;
-  msg->subsystem = "FDM";
-  msg->type = Message::eInteger;
-  msg->bVal = (iVal != 0);
+  Message msg;
+  msg.text = text;
+  msg.messageId = messageId++;
+  msg.subsystem = "FDM";
+  msg.type = Message::eInteger;
+  msg.bVal = (iVal != 0);
   Messages.push(msg);
-  return msg;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-FGJSBBase::Message* FGJSBBase::PutMessage(const string& text, double dVal)
+void FGJSBBase::PutMessage(const string& text, double dVal)
 {
-  Message *msg = new Message();
-  msg->text = text;
-  msg->messageId = messageId++;
-  msg->subsystem = "FDM";
-  msg->type = Message::eDouble;
-  msg->bVal = (dVal != 0.0);
+  Message msg;
+  msg.text = text;
+  msg.messageId = messageId++;
+  msg.subsystem = "FDM";
+  msg.type = Message::eDouble;
+  msg.bVal = (dVal != 0.0);
   Messages.push(msg);
-  return msg;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-FGJSBBase::Message* FGJSBBase::ReadMessage(void)
+int FGJSBBase::SomeMessages(void)
 {
-  if (!Messages.empty()) return Messages.front();
-  else                   return NULL;
+  return !Messages.empty();
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 FGJSBBase::Message* FGJSBBase::ProcessMessage(void)
 {
-  if (!Messages.empty())
-    localMsg = *(Messages.front());
-  else
-    return NULL;
+  if (Messages.empty()) return NULL;
+  localMsg = Messages.front();
   Messages.pop();
   return &localMsg;
 }
