@@ -7,20 +7,20 @@
  ------------- Copyright (C) 2001  Jon S. Berndt (jsb@hal-pc.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU General Public License as published by the Free Software
+ the terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU General Public License along with
+ You should have received a copy of the GNU Lesser General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Further information about the GNU General Public License can also be found on
+ Further information about the GNU Lesser General Public License can also be found on
  the world wide web at http://www.gnu.org.
 
 HISTORY
@@ -93,7 +93,7 @@ using std::max;
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_JSBBASE "$Id: FGJSBBase.h,v 1.20 2006-02-17 21:53:22 fredb Exp $"
+#define ID_JSBBASE "$Id: FGJSBBase.h,v 1.20.2.2 2007-05-27 09:33:09 ehofman Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -109,7 +109,7 @@ CLASS DOCUMENTATION
 *   This class provides universal constants, utility functions, messaging
 *   functions, and enumerated constants to JSBSim.
     @author Jon S. Berndt
-    @version $Id: FGJSBBase.h,v 1.20 2006-02-17 21:53:22 fredb Exp $
+    @version $Id: FGJSBBase.h,v 1.20.2.2 2007-05-27 09:33:09 ehofman Exp $
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -125,7 +125,8 @@ public:
   ~FGJSBBase() {};
 
   /// JSBSim Message structure
-  typedef struct Msg {
+  class Message {
+  public:
     unsigned int fdmId;
     unsigned int messageId;
     string text;
@@ -134,7 +135,7 @@ public:
     bool bVal;
     int  iVal;
     double dVal;
-  } Message;
+  };
 
   ///@name JSBSim console output highlighting terms.
   //@{
@@ -167,29 +168,29 @@ public:
   /** Places a Message structure on the Message queue.
       @param msg pointer to a Message structure
       @return pointer to a Message structure */
-  Message* PutMessage(Message* msg);
+void PutMessage(const Message& msg);
   /** Creates a message with the given text and places it on the queue.
       @param text message text
       @return pointer to a Message structure */
-  Message* PutMessage(const string& text);
+  void PutMessage(const string& text);
   /** Creates a message with the given text and boolean value and places it on the queue.
       @param text message text
       @param bVal boolean value associated with the message
       @return pointer to a Message structure */
-  Message* PutMessage(const string& text, bool bVal);
+void PutMessage(const string& text, bool bVal);
   /** Creates a message with the given text and integer value and places it on the queue.
       @param text message text
       @param iVal integer value associated with the message
       @return pointer to a Message structure */
-  Message* PutMessage(const string& text, int iVal);
+void PutMessage(const string& text, int iVal);
   /** Creates a message with the given text and double value and places it on the queue.
       @param text message text
       @param dVal double value associated with the message
       @return pointer to a Message structure */
-  Message* PutMessage(const string& text, double dVal);
+void PutMessage(const string& text, double dVal);
   /** Reads the message on the queue (but does not delete it).
-      @return pointer to a Message structure (or NULL if no mesage) */
-  Message* ReadMessage(void);
+      @return 1 if some messages */
+  int SomeMessages(void);
   /** Reads the message on the queue and removes it from the queue.
       @return pointer to a Message structure (or NULL if no mesage) */
   Message* ProcessMessage(void);
@@ -218,6 +219,13 @@ public:
     return (rankine - 491.67)/1.8;
   }
 
+  /** Converts from degrees Rankine to degrees Kelvin.
+  *   @param rankine The temperature in degrees Rankine.
+  *   @return The temperature in Kelvin. */
+  static double RankineToKelvin (double rankine) {
+    return rankine/1.8;
+  }
+
   /** Converts from degrees Fahrenheit to degrees Celsius.
   *   @param fahrenheit The temperature in degrees Fahrenheit.
   *   @return The temperature in Celsius. */
@@ -269,11 +277,10 @@ public:
 protected:
   static Message localMsg;
 
-  static std::queue <Message*> Messages;
+  static std::queue <Message> Messages;
 
   void Debug(int from) {};
 
-  static unsigned int frame;
   static unsigned int messageId;
 
   static const double radtodeg;
