@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: JSBSim.cxx,v 1.39 2006-02-02 09:35:25 ehofman Exp $
+// $Id: JSBSim.cxx,v 1.39.2.5 2007-05-27 13:24:32 ehofman Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -262,6 +262,8 @@ FGJSBsim::FGJSBsim( double dt )
     wind_from_east = fgGetNode("/environment/wind-from-east-fps" ,true);
     wind_from_down = fgGetNode("/environment/wind-from-down-fps" ,true);
 
+    slaved = fgGetNode("/sim/slaved/enabled", true);
+
     for (unsigned int i = 0; i < Propulsion->GetNumEngines(); i++) {
       SGPropertyNode * node = fgGetNode("engines/engine", i, true);
       Propulsion->GetEngine(i)->GetThruster()->SetRPM(node->getDoubleValue("rpm") /
@@ -423,10 +425,14 @@ void FGJSBsim::update( double dt )
     if (!cache_ok) {
       SG_LOG(SG_FLIGHT, SG_WARN,
              "FGInterface is being called without scenery below the aircraft!");
-      cout << "altitude         = " << alt << endl;
-      cout << "sea level radius = " << slr << endl;
-      cout << "latitude         = " << lat << endl;
-      cout << "longitude        = " << lon << endl;
+      SG_LOG(SG_FLIGHT, SG_WARN,
+             "altitude         = " << alt);
+      SG_LOG(SG_FLIGHT, SG_WARN,
+            "sea level radius = " << slr);
+      SG_LOG(SG_FLIGHT, SG_WARN,
+            "latitude         = " << lat);
+      SG_LOG(SG_FLIGHT, SG_WARN,
+            "longitude        = " << lon);
       //return;
     }
 
@@ -460,7 +466,7 @@ void FGJSBsim::update( double dt )
     }
 
     FGJSBBase::Message* msg;
-    while (fdmex->ReadMessage()) {
+    while (fdmex->SomeMessages()) {
       msg = fdmex->ProcessMessage();
       switch (msg->type) {
       case FGJSBBase::Message::eText:
@@ -601,6 +607,7 @@ bool FGJSBsim::copy_to_JSBsim()
     SGPropertyNode* node = fgGetNode("/systems/refuel", true);
     Propulsion->SetRefuel(node->getDoubleValue("contact"));
     Propulsion->SetFuelFreeze((fgGetNode("/sim/freeze/fuel",true))->getBoolValue());
+    fdmex->SetSlave(slaved->getBoolValue());
 
     return true;
 }
@@ -883,7 +890,7 @@ bool FGJSBsim::ToggleDataLogging(bool state)
 //Positions
 void FGJSBsim::set_Latitude(double lat)
 {
-    static const SGPropertyNode *altitude = fgGetNode("/position/altitude-ft");
+    static SGConstPropertyNode_ptr altitude = fgGetNode("/position/altitude-ft");
     double alt;
     double sea_level_radius_meters, lat_geoc;
 
@@ -923,7 +930,7 @@ void FGJSBsim::set_Longitude(double lon)
 
 void FGJSBsim::set_Altitude(double alt)
 {
-    static const SGPropertyNode *latitude = fgGetNode("/position/latitude-deg");
+    static SGConstPropertyNode_ptr latitude = fgGetNode("/position/latitude-deg");
 
     double sea_level_radius_meters,lat_geoc;
 
@@ -1098,8 +1105,10 @@ void FGJSBsim::do_trim(void)
   } else {
     trimmed->setBoolValue(true);
   }
+#if 0
   if (FGJSBBase::debug_lvl > 0)
-      State->ReportState();
+      State->ReportState();	/* FIXME: Function not implemented */
+#endif
 
   delete fgtrim;
 
