@@ -7,20 +7,20 @@
  ------------- Copyright (C) 1999  Anthony K. Peden (apeden@earthlink.net) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU General Public License as published by the Free Software
+ the terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU General Public License along with
+ You should have received a copy of the GNU Lesser General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Further information about the GNU General Public License can also be found on
+ Further information about the GNU Lesser General Public License can also be found on
  the world wide web at http://www.gnu.org.
 
  HISTORY
@@ -50,12 +50,13 @@ INCLUDES
 #include <FGFDMExec.h>
 #include <FGJSBBase.h>
 #include <math/FGColumnVector3.h>
+#include <input_output/FGXMLFileRead.h>
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_INITIALCONDITION "$Id: FGInitialCondition.h,v 1.2 2006-02-02 09:35:25 ehofman Exp $"
+#define ID_INITIALCONDITION "$Id: FGInitialCondition.h,v 1.2.2.2 2007-05-27 09:33:09 ehofman Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -70,7 +71,8 @@ typedef enum { setwned, setwmd, setwhc } windset;
 CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-/** Takes a set of initial conditions and provide a kinematically consistent set
+/** Initializes the simulation run.
+    Takes a set of initial conditions (IC) and provide a kinematically consistent set
     of body axis velocity components, euler angles, and altitude.  This class
     does not attempt to trim the model i.e. the sim will most likely start in a
     very dynamic state (unless, of course, you have chosen your IC's wisely, or
@@ -191,14 +193,14 @@ CLASS DOCUMENTATION
    @property ic/r-rad_sec (read/write) Yaw rate initial condition in radians/second
 
    @author Tony Peden
-   @version "$Id: FGInitialCondition.h,v 1.2 2006-02-02 09:35:25 ehofman Exp $"
+   @version "$Id: FGInitialCondition.h,v 1.2.2.2 2007-05-27 09:33:09 ehofman Exp $"
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DECLARATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-class FGInitialCondition : public FGJSBBase
+class FGInitialCondition : public FGJSBBase, public FGXMLFileRead
 {
 public:
   /// Constructor
@@ -449,11 +451,11 @@ public:
   inline double GetWindNFpsIC(void) const { return wnorth; }
 
   /** Gets the initial wind velocity in local frame.
-      @return Initial wind velocity toward north in feet/second */
+      @return Initial wind velocity eastwards in feet/second */
   inline double GetWindEFpsIC(void) const { return weast; }
 
   /** Gets the initial wind velocity in local frame.
-      @return Initial wind velocity toward north in feet/second */
+      @return Initial wind velocity downwards in feet/second */
   inline double GetWindDFpsIC(void) const { return wdown; }
 
   /** Gets the initial total wind velocity in feet/sec.
