@@ -8,20 +8,20 @@
  ------------- Copyright (C) 2002 -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU General Public License as published by the Free Software
+ the terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU General Public License along with
+ You should have received a copy of the GNU Lesser General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Further information about the GNU General Public License can also be found on
+ Further information about the GNU Lesser General Public License can also be found on
  the world wide web at http://www.gnu.org.
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -48,7 +48,7 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_PROPERTYMANAGER "$Id: FGPropertyManager.h,v 1.1 2006-01-12 15:04:22 ehofman Exp $"
+#define ID_PROPERTYMANAGER "$Id: FGPropertyManager.h,v 1.1.2.3 2007-05-27 09:33:09 ehofman Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -73,12 +73,12 @@ CLASS DECLARATION
 class FGPropertyManager : public SGPropertyNode, public FGJSBBase
 {
   private:
-    bool suppress_warning;
+    static bool suppress_warning;
   public:
     /// Constructor
     FGPropertyManager(void) {suppress_warning = false;}
     /// Destructor
-    ~FGPropertyManager(void) {}
+    virtual ~FGPropertyManager(void) {}
 
     /** Property-ify a name
      *  replaces spaces with '-' and, optionally, makes name all lower case
@@ -115,6 +115,11 @@ class FGPropertyManager : public SGPropertyNode, public FGJSBBase
      */
     string GetName( void );
 
+    /**
+     * Get the name of a node without underscores, etc.
+     */
+    string GetPrintableName( void );
+
     /**
      * Get the fully qualified name of a node
      * This function is very slow, so is probably useful for debugging only.
