@@ -8,20 +8,20 @@
  ------------- Copyright (C) 2001  Jon S. Berndt (jsb@hal-pc.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU General Public License as published by the Free Software
+ the terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU General Public License along with
+ You should have received a copy of the GNU Lesser General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Further information about the GNU General Public License can also be found on
+ Further information about the GNU Lesser General Public License can also be found on
  the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
@@ -48,7 +48,7 @@ using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGTable.cpp,v 1.5 2006-03-20 20:19:16 ehofman Exp $";
+static const char *IdSrc = "$Id: FGTable.cpp,v 1.5.2.2 2007-05-27 09:33:09 ehofman Exp $";
 static const char *IdHdr = ID_TABLE;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -80,11 +80,15 @@ FGTable::FGTable(const FGTable& t) : PropertyManager(t.PropertyManager)
   nTables = t.nTables;
   dimension = t.dimension;
   internal = t.internal;
+  Name = t.Name;
+  lookupProperty[0] = t.lookupProperty[0];
+  lookupProperty[1] = t.lookupProperty[1];
+  lookupProperty[2] = t.lookupProperty[2];
 
   Tables = t.Tables;
   Data = Allocate();
-  for (int r=0; r<=nRows; r++) {
-    for (int c=0; c<=nCols; c++) {
+  for (unsigned int r=0; r<=nRows; r++) {
+    for (unsigned int c=0; c<=nCols; c++) {
       Data[r][c] = t.Data[r][c];
     }
   }
@@ -97,7 +101,7 @@ FGTable::FGTable(const FGTable& t) : PropertyManager(t.PropertyManager)
 
 FGTable::FGTable(FGPropertyManager* propMan, Element* el) : PropertyManager(propMan)
 {
-  int i;
+  unsigned int i;
 
   stringstream buf;
   string property_string;
@@ -117,6 +121,7 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el) : PropertyManager(prop
   // Is this an internal lookup table?
 
   internal = false;
+  Name = el->GetAttributeValue("name"); // Allow this table to be named with a property
   call_type = el->GetAttributeValue("type");
   if (call_type == string("internal")) {
     parent_element = el->GetParent();
@@ -271,6 +276,9 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el) : PropertyManager(prop
     cout << "No dimension given" << endl;
     break;
   }
+
+  bind();
+
   if (debug_lvl & 1) Print();
 }
 
@@ -279,9 +287,9 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el) : PropertyManager(prop
 double** FGTable::Allocate(void)
 {
   Data = new double*[nRows+1];
-  for (int r=0; r<=nRows; r++) {
+  for (unsigned int r=0; r<=nRows; r++) {
     Data[r] = new double[nCols+1];
-    for (int c=0; c<=nCols; c++) {
+    for (unsigned int c=0; c<=nCols; c++) {
       Data[r][c] = 0.0;
     }
   }
@@ -292,23 +300,28 @@ double** FGTable::Allocate(void)
 
 FGTable::~FGTable()
 {
+  if (!Name.empty() && !internal) {
+    string tmp = PropertyManager->mkPropertyName(Name, false); // Allow upper case
+    PropertyManager->Untie(tmp);
+  }
+
   if (nTables > 0) {
-cout << "nTables = " << nTables << endl;
-    for (int i=0; i<nTables; i++) delete Tables[i];
+    for (unsigned int i=0; i<nTables; i++) delete Tables[i];
     Tables.clear();
   }
-  for (int r=0; r<=nRows; r++) if (Data[r]) delete[] Data[r];
-  if (Data) delete[] Data;
+  for (unsigned int r=0; r<=nRows; r++) delete[] Data[r];
+  delete[] Data;
+
   Debug(1);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-int FGTable::FindNumColumns(string test_line)
+unsigned int FGTable::FindNumColumns(string test_line)
 {
   // determine number of data columns in table (first column is row lookup - don't count)
-  int position=0;
-  int nCols=0;
+  size_t position=0;
+  unsigned int nCols=0;
   while ((position = test_line.find_first_not_of(" \t", position)) != string::npos) {
     nCols++;
     position = test_line.find_first_of(" \t", position);
@@ -346,7 +359,7 @@ double FGTable::GetValue(void) const
 double FGTable::GetValue(double key) const
 {
   double Factor, Value, Span;
-  int r=lastRowIndex;
+  unsigned int r = lastRowIndex;
 
   //if the key is off the end of the table, just return the
   //end-of-table value, do not extrapolate
@@ -361,14 +374,12 @@ double FGTable::GetValue(double key) const
   }
 
   // the key is somewhere in the middle, search for the right breakpoint
-  // assume the correct breakpoint has not changed since last frame or
+  // The search is particularly efficient if 
+  // the correct breakpoint has not changed since last frame or
   // has only changed very little
 
-  if ( r > 2 && Data[r-1][0] > key ) {
-    while( Data[r-1][0] > key && r > 2) { r--; }
-  } else if ( Data[r][0] < key ) {
-    while( Data[r][0] <= key && r <= nRows) { r++; }
-  }
+  while (r > 2     && Data[r-1][0] > key) { r--; }
+  while (r < nRows && Data[r][0]   < key) { r++; }
 
   lastRowIndex=r;
   // make sure denominator below does not go to zero.
@@ -391,22 +402,14 @@ double FGTable::GetValue(double key) const
 double FGTable::GetValue(double rowKey, double colKey) const
 {
   double rFactor, cFactor, col1temp, col2temp, Value;
-  int r=lastRowIndex;
-  int c=lastColumnIndex;
+  unsigned int r = lastRowIndex;
+  unsigned int c = lastColumnIndex;
 
-  if ( r > 2 && Data[r-1][0] > rowKey ) {
-    while ( Data[r-1][0] > rowKey && r > 2) { r--; }
-  } else if ( Data[r][0] < rowKey ) {
-    while ( r <= nRows && Data[r][0] <= rowKey ) { r++; }
-    if ( r > nRows ) r = nRows;
-  }
+  while(r > 2     && Data[r-1][0] > rowKey) { r--; }
+  while(r < nRows && Data[r]  [0] < rowKey) { r++; }
 
-  if ( c > 2 && Data[0][c-1] > colKey ) {
-    while( Data[0][c-1] > colKey && c > 2) { c--; }
-  } else if ( Data[0][c] < colKey ) {
-    while( Data[0][c] <= colKey && c <= nCols) { c++; }
-    if ( c > nCols ) c = nCols;
-  }
+  while(c > 2     && Data[0][c-1] > colKey) { c--; }
+  while(c < nCols && Data[0][c]   < colKey) { c++; }
 
   lastRowIndex=r;
   lastColumnIndex=c;
@@ -433,7 +436,7 @@ double FGTable::GetValue(double rowKey, double colKey) const
 double FGTable::GetValue(double rowKey, double colKey, double tableKey) const
 {
   double Factor, Value, Span;
-  int r=lastRowIndex;
+  unsigned int r = lastRowIndex;
 
   //if the key is off the end  (or before the beginning) of the table,
   // just return the boundary-table value, do not extrapolate
@@ -447,14 +450,12 @@ double FGTable::GetValue(double rowKey, double colKey, double tableKey) const
   }
 
   // the key is somewhere in the middle, search for the right breakpoint
-  // assume the correct breakpoint has not changed since last frame or
+  // The search is particularly efficient if 
+  // the correct breakpoint has not changed since last frame or
   // has only changed very little
 
-  if ( r > 2 && Data[r-1][1] > tableKey ) {
-    while( Data[r-1][1] > tableKey && r > 2) { r--; }
-  } else if ( Data[r][1] < tableKey ) {
-    while( Data[r][1] <= tableKey && r <= nRows) { r++; }
-  }
+  while(r > 2     && Data[r-1][1] > tableKey) { r--; }
+  while(r < nRows && Data[r]  [1] < tableKey) { r++; }
 
   lastRowIndex=r;
   // make sure denominator below does not go to zero.
@@ -480,11 +481,11 @@ void FGTable::operator<<(stringstream& in_stream)
   int startRow=0;
   int startCol=0;
 
-  if (Type == tt1D || Type == tt3D) startRow = 1;
-  if (Type == tt3D) startCol = 1;
+// In 1D table, no pseudo-row of column-headers (i.e. keys):
+  if (Type == tt1D) startRow = 1;
 
-  for (int r=startRow; r<=nRows; r++) {
-    for (int c=startCol; c<=nCols; c++) {
+  for (unsigned int r=startRow; r<=nRows; r++) {
+    for (unsigned int c=startCol; c<=nCols; c++) {
       if (r != 0 || c != 0) {
         in_stream >> Data[r][c];
       }
@@ -544,9 +545,9 @@ void FGTable::Print(void)
       break;
   }
   cout.precision(4);
-  for (int r=startRow; r<=nRows; r++) {
+  for (unsigned int r=startRow; r<=nRows; r++) {
     cout << "	";
-    for (int c=startCol; c<=nCols; c++) {
+    for (unsigned int c=startCol; c<=nCols; c++) {
       if (r == 0 && c == 0) {
         cout << "	";
       } else {
@@ -562,6 +563,16 @@ void FGTable::Print(void)
   cout.setf(flags); // reset
 }
 
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void FGTable::bind(void)
+{
+  typedef double (FGTable::*PMF)(void) const;
+  if ( !Name.empty() && !internal) {
+    string tmp = PropertyManager->mkPropertyName(Name, false); // Allow upper
+    PropertyManager->Tie( tmp, this, (PMF)&FGTable::GetValue);
+  }
+}
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 //    The bitmasked value choices are as follows:
 //    unset: In this case (the default) JSBSim would only print
