@@ -9,20 +9,20 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jsb@hal-pc.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU General Public License as published by the Free Software
+ the terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU General Public License along with
+ You should have received a copy of the GNU Lesser General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Further information about the GNU General Public License can also be found on
+ Further information about the GNU Lesser General Public License can also be found on
  the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
@@ -45,16 +45,18 @@ INCLUDES
 #include <models/flight_control/FGFilter.h>
 #include <models/flight_control/FGDeadBand.h>
 #include <models/flight_control/FGGain.h>
+#include <models/flight_control/FGPID.h>
 #include <models/flight_control/FGGradient.h>
 #include <models/flight_control/FGSwitch.h>
 #include <models/flight_control/FGSummer.h>
 #include <models/flight_control/FGKinemat.h>
 #include <models/flight_control/FGFCSFunction.h>
 #include <models/flight_control/FGSensor.h>
+#include <models/flight_control/FGActuator.h>
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGFCS.cpp,v 1.1 2006-01-12 15:04:23 ehofman Exp $";
+static const char *IdSrc = "$Id: FGFCS.cpp,v 1.1.2.3 2007-05-27 09:33:10 ehofman Exp $";
 static const char *IdHdr = ID_FCS;
 
 #if defined(WIN32) && !defined(__CYGWIN__)
@@ -76,7 +78,7 @@ FGFCS::FGFCS(FGFDMExec* fdmex) : FGModel(fdmex)
   LeftBrake = RightBrake = CenterBrake = 0.0;
 
   bind();
-  for (i=0;i<=NForms;i++) {
+  for (i=0;i<NForms;i++) {
     DePos[i] = DaLPos[i] = DaRPos[i] = DrPos[i] = 0.0;
     DfPos[i] = DsbPos[i] = DspPos[i] = 0.0;
   }
@@ -146,7 +148,9 @@ bool FGFCS::Run(void)
 
   // Cycle through the sensor, autopilot, and flight control components
   for (i=0; i<sensors.size(); i++) sensors[i]->Run();
-  for (i=0; i<APComponents.size(); i++) APComponents[i]->Run();
+  for (i=0; i<APComponents.size(); i++) {
+    APComponents[i]->Run();
+  }
   for (i=0; i<FCSComponents.size(); i++) FCSComponents[i]->Run();
 
   return false;
@@ -453,13 +457,10 @@ void FGFCS::SetPropFeather(int engineNum, bool setting)
 
 bool FGFCS::Load(Element* el)
 {
-  string name, file, fname, comp_name, interface_property_string;
-  unsigned i;
+  string name, file, fname, interface_property_string;
   vector <FGFCSComponent*> *Components;
-  Element *FCS_cfg, *document, *component_element, *property_element, *sensor_element;
+  Element *component_element, *property_element, *sensor_element;
   Element *channel_element;
-  ifstream* controls_file = new ifstream();
-  FGXMLParse controls_file_parser;
 
   Components=0;
   // Determine if the FCS/Autopilot is defined inline in the aircraft configuration
@@ -474,15 +475,12 @@ bool FGFCS::Load(Element* el)
 
   if (name.empty()) {
     fname = el->GetAttributeValue("file");
-    file = FDMExec->GetAircraftPath() + separator + FDMExec->GetModelName() + separator + fname + ".xml";
+    file = FDMExec->GetFullAircraftPath() + separator + fname + ".xml";
     if (fname.empty()) {
       cerr << "FCS/Autopilot does not appear to be defined inline nor in a file" << endl;
       return false;
     } else {
-      controls_file->open(file.c_str());
-      readXML(*controls_file, controls_file_parser);
-      delete controls_file;
-      document = controls_file_parser.GetDocument();
+      document = LoadXMLDocument(file);
     }
   } else {
     document = el;
@@ -527,60 +525,44 @@ bool FGFCS::Load(Element* el)
 
   channel_element = document->FindElement("channel");
   while (channel_element) {
-    component_element = channel_element->FindElement("component");
-    if (component_element) {
-      cout << "This form of the component specification is being deprecated" << endl;
-    } else {
-      component_element = channel_element->GetElement();
-    }
+    component_element = channel_element->GetElement();
     while (component_element) {
-      comp_name = component_element->GetAttributeValue("type");
       try {
-        if ((comp_name == "LAG_FILTER") ||
-            (comp_name == "LEAD_LAG_FILTER") ||
-            (comp_name == "SECOND_ORDER_FILTER") ||
-            (comp_name == "WASHOUT_FILTER") ||
-            (comp_name == "INTEGRATOR") ||
-            (component_element->GetName() == string("lag_filter")) ||
+        if ((component_element->GetName() == string("lag_filter")) ||
             (component_element->GetName() == string("lead_lag_filter")) ||
             (component_element->GetName() == string("washout_filter")) ||
             (component_element->GetName() == string("second_order_filter")) ||
             (component_element->GetName() == string("integrator")) )
         {
           Components->push_back(new FGFilter(this, component_element));
-        } else if ((comp_name == "PURE_GAIN") ||
-                   (comp_name == "SCHEDULED_GAIN") ||
-                   (comp_name == "AEROSURFACE_SCALE") ||
-                   (component_element->GetName() == string("pure_gain")) ||
+        } else if ((component_element->GetName() == string("pure_gain")) ||
                    (component_element->GetName() == string("scheduled_gain")) ||
                    (component_element->GetName() == string("aerosurface_scale")))
         {
           Components->push_back(new FGGain(this, component_element));
-        } else if ((comp_name == "SUMMER") || (component_element->GetName() == string("summer"))) {
+        } else if (component_element->GetName() == string("summer")) {
           Components->push_back(new FGSummer(this, component_element));
-        } else if ((comp_name == "DEADBAND") || (component_element->GetName() == string("deadband"))) {
+        } else if (component_element->GetName() == string("deadband")) {
           Components->push_back(new FGDeadBand(this, component_element));
-        } else if (comp_name == "GRADIENT") {
-          Components->push_back(new FGGradient(this, component_element));
-        } else if ((comp_name == "SWITCH") || (component_element->GetName() == string("switch"))) {
+        } else if (component_element->GetName() == string("switch")) {
           Components->push_back(new FGSwitch(this, component_element));
-        } else if ((comp_name == "KINEMAT") || (component_element->GetName() == string("kinematic"))) {
+        } else if (component_element->GetName() == string("kinematic")) {
           Components->push_back(new FGKinemat(this, component_element));
-        } else if ((comp_name == "FUNCTION") || (component_element->GetName() == string("fcs_function"))) {
+        } else if (component_element->GetName() == string("fcs_function")) {
           Components->push_back(new FGFCSFunction(this, component_element));
+        } else if (component_element->GetName() == string("pid")) {
+          Components->push_back(new FGPID(this, component_element));
+        } else if (component_element->GetName() == string("actuator")) {
+          Components->push_back(new FGActuator(this, component_element));
         } else {
-          cerr << "Unknown FCS component: " << comp_name << endl;
+          cerr << "Unknown FCS component: " << component_element->GetName() << endl;
         }
       } catch(string s) {
         cerr << highint << fgred << endl << "  " << s << endl;
         cerr << reset << endl;
         return false;
       }
-      if (comp_name.empty()) { // comp_name will be empty if using new format
-        component_element = channel_element->GetNextElement();
-      } else {
-        component_element = channel_element->FindNextElement("component");
-      }
+      component_element = channel_element->GetNextElement();
     }
     channel_element = document->FindNextElement("channel");
   }
@@ -622,7 +604,9 @@ string FGFCS::GetComponentStrings(string delimeter)
 
   for (comp = 0; comp < APComponents.size(); comp++)
   {
-    CompStrings += delimeter;
+    if (firstime) firstime = false;
+    else          CompStrings += delimeter;
+
     CompStrings += APComponents[comp]->GetName();
   }
 
@@ -635,7 +619,7 @@ string FGFCS::GetComponentValues(string delimeter)
 {
   unsigned int comp;
   string CompValues = "";
-  char buffer[12];
+  char buffer[17];
   bool firstime = true;
 
   for (comp = 0; comp < FCSComponents.size(); comp++) {
@@ -647,10 +631,14 @@ string FGFCS::GetComponentValues(string delimeter)
   }
 
   for (comp = 0; comp < APComponents.size(); comp++) {
-    sprintf(buffer, "%s%9.6f", delimeter.c_str(), APComponents[comp]->GetOutput());
+    if (firstime) firstime = false;
+    else          CompValues += delimeter;
+
+    sprintf(buffer, "%9.6f", APComponents[comp]->GetOutput());
     CompValues += string(buffer);
   }
 
+  CompValues += "\0";
   return CompValues;
 }
 
@@ -667,7 +655,7 @@ void FGFCS::AddThrottle(void)
   PropFeatherCmd.push_back(false);
   PropFeather.push_back(false);
 
-  unsigned int num = ThrottleCmd.size()-1;
+  unsigned int num = (unsigned int)ThrottleCmd.size()-1;
   bindThrottle(num);
 }
 
@@ -680,19 +668,24 @@ void FGFCS::AddGear(void)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+double FGFCS::GetDt(void)
+{
+  return FDMExec->GetDeltaT()*rate;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 void FGFCS::bind(void)
 {
   PropertyManager->Tie("fcs/aileron-cmd-norm", this, &FGFCS::GetDaCmd, &FGFCS::SetDaCmd);
   PropertyManager->Tie("fcs/elevator-cmd-norm", this, &FGFCS::GetDeCmd, &FGFCS::SetDeCmd);
   PropertyManager->Tie("fcs/rudder-cmd-norm", this, &FGFCS::GetDrCmd, &FGFCS::SetDrCmd);
-  PropertyManager->Tie("fcs/steer-cmd-norm", this, &FGFCS::GetDsCmd, &FGFCS::SetDsCmd);
   PropertyManager->Tie("fcs/flap-cmd-norm", this, &FGFCS::GetDfCmd, &FGFCS::SetDfCmd);
   PropertyManager->Tie("fcs/speedbrake-cmd-norm", this, &FGFCS::GetDsbCmd, &FGFCS::SetDsbCmd);
   PropertyManager->Tie("fcs/spoiler-cmd-norm", this, &FGFCS::GetDspCmd, &FGFCS::SetDspCmd);
   PropertyManager->Tie("fcs/pitch-trim-cmd-norm", this, &FGFCS::GetPitchTrimCmd, &FGFCS::SetPitchTrimCmd);
   PropertyManager->Tie("fcs/roll-trim-cmd-norm", this, &FGFCS::GetRollTrimCmd, &FGFCS::SetRollTrimCmd);
   PropertyManager->Tie("fcs/yaw-trim-cmd-norm", this, &FGFCS::GetYawTrimCmd, &FGFCS::SetYawTrimCmd);
-  PropertyManager->Tie("gear/gear-cmd-norm", this, &FGFCS::GetGearCmd, &FGFCS::SetGearCmd);
 
   PropertyManager->Tie("fcs/left-aileron-pos-rad", this, ofRad, &FGFCS::GetDaLPos, &FGFCS::SetDaLPos);
   PropertyManager->Tie("fcs/left-aileron-pos-deg", this, ofDeg, &FGFCS::GetDaLPos, &FGFCS::SetDaLPos);
@@ -729,6 +722,11 @@ void FGFCS::bind(void)
   PropertyManager->Tie("fcs/mag-spoiler-pos-rad", this, ofMag, &FGFCS::GetDspPos);
 
   PropertyManager->Tie("gear/gear-pos-norm", this, &FGFCS::GetGearPos, &FGFCS::SetGearPos);
+  PropertyManager->Tie("gear/gear-cmd-norm", this, &FGFCS::GetGearCmd, &FGFCS::SetGearCmd);
+  PropertyManager->Tie("fcs/left-brake-cmd-norm", this, &FGFCS::GetLBrake, &FGFCS::SetLBrake);
+  PropertyManager->Tie("fcs/right-brake-cmd-norm", this, &FGFCS::GetRBrake, &FGFCS::SetRBrake);
+  PropertyManager->Tie("fcs/center-brake-cmd-norm", this, &FGFCS::GetCBrake, &FGFCS::SetCBrake);
+  PropertyManager->Tie("fcs/steer-cmd-norm", this, &FGFCS::GetDsCmd, &FGFCS::SetDsCmd);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
