@@ -9,20 +9,20 @@
  ------------- Copyright (C) 2000  Jon S. Berndt (jsb@hal-pc.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU General Public License as published by the Free Software
+ the terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU General Public License along with
+ You should have received a copy of the GNU Lesser General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Further information about the GNU General Public License can also be found on
+ Further information about the GNU Lesser General Public License can also be found on
  the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
@@ -57,7 +57,7 @@ INCLUDES
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGPropulsion.cpp,v 1.1 2006-01-12 15:04:23 ehofman Exp $";
+static const char *IdSrc = "$Id: FGPropulsion.cpp,v 1.1.2.2 2007-05-27 09:33:10 ehofman Exp $";
 static const char *IdHdr = ID_PROPULSION;
 
 extern short debug_lvl;
@@ -124,7 +124,7 @@ bool FGPropulsion::Run(void)
     Tanks[i]->Calculate( dt * rate );
     if (Tanks[i]->GetType() == FGTank::ttFUEL) {
       TotalFuelQuantity += Tanks[i]->GetContents();
-    } 
+    }
   }
 
   if (refuel) DoRefuel( dt * rate );
@@ -199,11 +199,7 @@ bool FGPropulsion::ICEngineStart(void)
 bool FGPropulsion::Load(Element* el)
 {
   string type, engine_filename;
-  int Feed;
   bool ThrottleAdded = false;
-  Element* document;
-  FGXMLParse engine_file_parser;
-  ifstream* engine_file;
 
   Debug(2);
 
@@ -217,8 +213,7 @@ bool FGPropulsion::Load(Element* el)
     }
 
     engine_filename = FindEngineFullPathname(engine_filename);
-    readXML(engine_filename, engine_file_parser);
-    document = engine_file_parser.GetDocument(); // document holds the engine description
+    document = LoadXMLDocument(engine_filename);
     document->SetParent(engine_element);
 
     type = document->GetName();
@@ -253,7 +248,7 @@ bool FGPropulsion::Load(Element* el)
     numEngines++;
 
     engine_element = el->FindNextElement("engine");
-    engine_file_parser.reset();
+    ResetParser();
   }
 
   // Process tank definitions
@@ -282,7 +277,7 @@ string FGPropulsion::FindEngineFullPathname(string engine_filename)
 {
   string fullpath, localpath;
   string enginePath = FDMExec->GetEnginePath();
-  string aircraftPath = FDMExec->GetAircraftPath();
+  string aircraftPath = FDMExec->GetFullAircraftPath();
   ifstream* engine_file = new ifstream();
 
   string separator = "/";
@@ -313,7 +308,7 @@ ifstream* FGPropulsion::FindEngineFile(string engine_filename)
 {
   string fullpath, localpath;
   string enginePath = FDMExec->GetEnginePath();
-  string aircraftPath = FDMExec->GetAircraftPath();
+  string aircraftPath = FDMExec->GetFullAircraftPath();
   ifstream* engine_file = new ifstream();
 
   string separator = "/";
@@ -488,7 +483,7 @@ void FGPropulsion::SetCutoff(int setting)
 
 void FGPropulsion::SetActiveEngine(int engine)
 {
-  if (engine >= Engines.size() || engine < 0)
+  if (engine >= (int)Engines.size() || engine < 0)
     ActiveEngine = -1;
   else
     ActiveEngine = engine;
@@ -572,6 +567,8 @@ void FGPropulsion::bind(void)
 
 void FGPropulsion::unbind(void)
 {
+  if (!IsBound) return;
+
   if (HaveTurbineEngine) {
     PropertyManager->Untie("propulsion/starter_cmd");
     PropertyManager->Untie("propulsion/cutoff_cmd");
