@@ -8,20 +8,20 @@
  ------------- Copyright (C) 2003  David Culp (davidculp2@comcast.net) ---------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU General Public License as published by the Free Software
+ the terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU General Public License along with
+ You should have received a copy of the GNU Lesser General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Further information about the GNU General Public License can also be found on
+ Further information about the GNU Lesser General Public License can also be found on
  the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
@@ -46,7 +46,7 @@ INCLUDES
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGTurbine.cpp,v 1.1 2006-01-12 15:04:23 ehofman Exp $";
+static const char *IdSrc = "$Id: FGTurbine.cpp,v 1.1.2.1 2007-01-06 10:49:26 ehofman Exp $";
 static const char *IdHdr = ID_TURBINE;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -67,10 +67,10 @@ FGTurbine::FGTurbine(FGFDMExec* exec, Element *el, int engine_number)
 
 FGTurbine::~FGTurbine()
 {
-  if (IdleThrustLookup) delete IdleThrustLookup;
-  if (MilThrustLookup) delete MilThrustLookup;
-  if (MaxThrustLookup) delete MaxThrustLookup;
-  if (InjectionLookup) delete InjectionLookup;
+  delete IdleThrustLookup;
+  delete MilThrustLookup;
+  delete MaxThrustLookup;
+  delete InjectionLookup;
   unbind();
   Debug(1);
 }
