@@ -34,9 +34,9 @@
 	
 	CURRENT RCS HEADER:
 
-$Header: /var/cvs/FlightGear-0.9/FlightGear/src/FDM/LaRCsim/c172_gear.c,v 1.1.1.1 2002/09/10 01:14:01 curt Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/c172_gear.c,v 1.1.1.1 2002-09-10 01:14:01 curt Exp $
 $Log: c172_gear.c,v $
-Revision 1.1.1.1  2002/09/10 01:14:01  curt
+Revision 1.1.1.1  2002-09-10 01:14:01  curt
 Initial revision of FlightGear-0.9.0
 
 Revision 1.20  2001/07/30 20:53:54  curt
@@ -149,7 +149,7 @@ static void clear3( DATA v[] )
 
 void c172_gear()
 {
-char rcsid[] = "$Id: c172_gear.c,v 1.1.1.1 2002/09/10 01:14:01 curt Exp $";
+char rcsid[] = "$Id: c172_gear.c,v 1.1.1.1 2002-09-10 01:14:01 curt Exp $";
 #define NUM_WHEELS 4
 // char gear_strings[NUM_WHEELS][12]={"nose","right main", "left main", "tail skid"};
   /*
