@@ -34,9 +34,9 @@
 	
 	CURRENT RCS HEADER:
 
-$Header: /var/cvs/FlightGear-0.9/FlightGear/src/FDM/LaRCsim/cherokee_gear.c,v 1.1.1.1 2002/09/10 01:14:01 curt Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/cherokee_gear.c,v 1.1.1.1 2002-09-10 01:14:01 curt Exp $
 $Log: cherokee_gear.c,v $
-Revision 1.1.1.1  2002/09/10 01:14:01  curt
+Revision 1.1.1.1  2002-09-10 01:14:01  curt
 Initial revision of FlightGear-0.9.0
 
 Revision 1.3  2001/07/30 20:53:54  curt
@@ -135,7 +135,7 @@ static void clear3( DATA v[] )
 
 void cherokee_gear()
 {
-char rcsid[] = "$Id: cherokee_gear.c,v 1.1.1.1 2002/09/10 01:14:01 curt Exp $";
+char rcsid[] = "$Id: cherokee_gear.c,v 1.1.1.1 2002-09-10 01:14:01 curt Exp $";
 
   /*
    * Aircraft specific initializations and data goes here
