@@ -30,10 +30,10 @@
 
 	CURRENT RCS HEADER INFO:
 
-$Header: /var/cvs/FlightGear-0.9/FlightGear/src/FDM/LaRCsim/default_model_routines.c,v 1.1.1.1 2002/09/10 01:14:01 curt Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/default_model_routines.c,v 1.1.1.1 2002-09-10 01:14:01 curt Exp $
 
 $Log: default_model_routines.c,v $
-Revision 1.1.1.1  2002/09/10 01:14:01  curt
+Revision 1.1.1.1  2002-09-10 01:14:01  curt
 Initial revision of FlightGear-0.9.0
 
 Revision 1.1.1.1  1999/06/17 18:07:33  curt
