@@ -36,9 +36,9 @@
 	    contribution from Q_body*P_body*D_X_pilot term.
   940111    Changed DATA to SCALAR; updated header files
 	    
-$Header: /var/cvs/FlightGear-0.9/FlightGear/src/FDM/LaRCsim/ls_accel.c,v 1.1.1.1 2002/09/10 01:14:01 curt Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_accel.c,v 1.1.1.1 2002-09-10 01:14:01 curt Exp $
 $Log: ls_accel.c,v $
-Revision 1.1.1.1  2002/09/10 01:14:01  curt
+Revision 1.1.1.1  2002-09-10 01:14:01  curt
 Initial revision of FlightGear-0.9.0
 
 Revision 1.1.1.1  1999/06/17 18:07:33  curt
