@@ -33,9 +33,9 @@
 	
 	CURRENT RCS HEADER:
 
-$Header: /var/cvs/FlightGear-0.9/FlightGear/src/FDM/LaRCsim/ls_cockpit.h,v 1.1.1.1 2002/09/10 01:14:01 curt Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_cockpit.h,v 1.1.1.1 2002-09-10 01:14:01 curt Exp $
 $Log: ls_cockpit.h,v $
-Revision 1.1.1.1  2002/09/10 01:14:01  curt
+Revision 1.1.1.1  2002-09-10 01:14:01  curt
 Initial revision of FlightGear-0.9.0
 
 Revision 1.3  2000/06/12 18:52:37  curt
