@@ -38,9 +38,9 @@
 
 	CURRENT RCS HEADER:
 
-$Header: /var/cvs/FlightGear-0.9/FlightGear/src/FDM/LaRCsim/ls_geodesy.c,v 1.1.1.1 2002/09/10 01:14:02 curt Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_geodesy.c,v 1.1.1.1 2002-09-10 01:14:02 curt Exp $
 $Log: ls_geodesy.c,v $
-Revision 1.1.1.1  2002/09/10 01:14:02  curt
+Revision 1.1.1.1  2002-09-10 01:14:02  curt
 Initial revision of FlightGear-0.9.0
 
 Revision 1.3  2001/03/24 05:03:12  curt
