@@ -33,9 +33,9 @@
 		to SCALAR types.					EBJ
 		
 										
-$Header: /var/cvs/FlightGear-0.9/FlightGear/src/FDM/LaRCsim/ls_gravity.c,v 1.1.1.1 2002/09/10 01:14:02 curt Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_gravity.c,v 1.1.1.1 2002-09-10 01:14:02 curt Exp $
 $Log: ls_gravity.c,v $
-Revision 1.1.1.1  2002/09/10 01:14:02  curt
+Revision 1.1.1.1  2002-09-10 01:14:02  curt
 Initial revision of FlightGear-0.9.0
 
 Revision 1.1.1.1  1999/06/17 18:07:34  curt
