@@ -20,7 +20,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Id: ls_interface.c,v 1.2 2006-02-21 17:45:03 mfranz Exp $
+ * $Id: ls_interface.c,v 1.2.2.1 2007-03-01 17:53:27 mfranz Exp $
  * (Log is kept at end of this file)
  **************************************************************************/
 
@@ -90,7 +90,7 @@
 		routine ls_pause(); and added call to exit() on termination.
 
 
-$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_interface.c,v 1.2 2006-02-21 17:45:03 mfranz Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_interface.c,v 1.2.2.1 2007-03-01 17:53:27 mfranz Exp $
 $Original log: LaRCsim.c,v $
  * Revision 1.4.1.7  1995/04/07  01:04:37  bjax
  * Many changes made to support storage of sim options from run to run,
@@ -280,9 +280,9 @@ static char  matname[MAX_FILE_NAME_LENGTH]  = "run.m";
 
 
 void ls_stamp( void ) {
-    char rcsid[] = "$Id: ls_interface.c,v 1.2 2006-02-21 17:45:03 mfranz Exp $";
-    char revid[] = "$Revision: 1.2 $";
-    char dateid[] = "$Date: 2006-02-21 17:45:03 $";
+    char rcsid[] = "$Id: ls_interface.c,v 1.2.2.1 2007-03-01 17:53:27 mfranz Exp $";
+    char revid[] = "$Revision: 1.2.2.1 $";
+    char dateid[] = "$Date: 2007-03-01 17:53:27 $";
     struct tm *nowtime;
     time_t nowtime_t;
     long date;
@@ -292,7 +292,7 @@ void ls_stamp( void ) {
     
     nowtime_t = time( 0 );
     nowtime = localtime( &nowtime_t ); /* set fields to correct time values */
-    date = (nowtime->tm_year)*10000 
+    date = (nowtime->tm_year % 100)*10000
 	 + (nowtime->tm_mon + 1)*100
 	 + (nowtime->tm_mday);
     sprintf(sim_control_.date_string, "%06ld\0", date);
@@ -580,6 +580,21 @@ int ls_ForceAltitude(double alt_feet) {
 /* Flight Gear Modification Log
  *
  * $Log: ls_interface.c,v $
+ * Revision 1.2.2.1  2007-03-01 17:53:27  mfranz
+ * Hans Ulrich NIEDERMANN:
+ *
+ * """
+ * Fix Y2K bug triggering string overflow
+ *
+ * sim_control_.date_string is a char[7], so it can contain "yymmdd" and
+ * the terminating '\0'. However, nowtime->tm_year is 107 for the year 2007,
+ * so you'll end up with a 7 digit number and the string written to
+ * sim_control_.date_string is longer than sim_control_.date_string is.
+ * Ouch!
+ * """
+ *
+ * mf: ... and sim_control_.date_string isn't even used.
+ *
  * Revision 1.2  2006-02-21 17:45:03  mfranz
  * new FSF address (see http://www.gnu.org/licenses/gpl.html)
  *
