@@ -35,8 +35,13 @@
 		is requested.					EBJ
 
 	CURRENT RCS HEADER INFO:
-$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_model.c,v 1.4 2003/07/25 17:53:41 mselig Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_model.c,v 1.5 2005-12-19 12:53:21 ehofman Exp $
 $Log: ls_model.c,v $
+Revision 1.5  2005-12-19 12:53:21  ehofman
+Vassilii Khachaturov:
+
+clean up some build warnings caught with gcc-4.0.
+
 Revision 1.4  2003/07/25 17:53:41  mselig
 UIUC code initilization mods to tidy things up a bit.
 
@@ -154,6 +159,8 @@ Initial Flight Gear revision.
 	OUTPUTS:
 
 --------------------------------------------------------------------------*/
+#include <stdio.h>
+
 #include "ls_types.h"
 #include "ls_model.h"
 #include "default_model_routines.h"
