@@ -48,9 +48,9 @@
 
 	CURRENT RCS HEADER:
 
-$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_step.c,v 1.5 2003/07/25 17:53:47 mselig Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/ls_step.c,v 1.5 2003-07-25 17:53:47 mselig Exp $
 $Log: ls_step.c,v $
-Revision 1.5  2003/07/25 17:53:47  mselig
+Revision 1.5  2003-07-25 17:53:47  mselig
 UIUC code initilization mods to tidy things up a bit.
 
 Revision 1.4  2003/06/20 19:53:56  ehofman
