@@ -34,9 +34,9 @@
 	
 	CURRENT RCS HEADER:
 
-$Header: /var/cvs/FlightGear-0.9/FlightGear/src/FDM/LaRCsim/navion_gear.c,v 1.1.1.1 2002/09/10 01:14:02 curt Exp $
+$Header: /var/cvs/FlightGear-0.9/source/src/FDM/LaRCsim/navion_gear.c,v 1.1.1.1 2002-09-10 01:14:02 curt Exp $
 $Log: navion_gear.c,v $
-Revision 1.1.1.1  2002/09/10 01:14:02  curt
+Revision 1.1.1.1  2002-09-10 01:14:02  curt
 Initial revision of FlightGear-0.9.0
 
 Revision 1.4  2001/07/30 20:53:54  curt
@@ -159,7 +159,7 @@ static void clear3( DATA v[] )
 }
 
 void navion_gear( SCALAR dt, int Initialize ) {
-char rcsid[] = "$Id: navion_gear.c,v 1.1.1.1 2002/09/10 01:14:02 curt Exp $";
+char rcsid[] = "$Id: navion_gear.c,v 1.1.1.1 2002-09-10 01:14:02 curt Exp $";
 
   /*
    * Aircraft specific initializations and data goes here
