@@ -12,9 +12,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: ADA.cxx,v 1.3 2005/11/01 13:41:50 ehofman Exp $
+// $Id: ADA.cxx,v 1.5 2006-02-21 01:18:09 mfranz Exp $
 
 // Modified by Cdr. VS Renganthan <vsranga@ada.ernet.in>, 12 Oct 2K
 
@@ -36,7 +36,7 @@
 #define numberofbytes 472 // from FDM to visuals
 #define nbytes 8	//from visuals to FDM
 
-struct {
+static struct {
     double number_of_bytes;
     double lat_geoc;
     double lon_geoc;
@@ -111,7 +111,7 @@ struct {
 
 double view_offset; //if this zero, means center window
 
-struct {
+static struct {
 	double ground_elevation;
 } visuals_to_sixdof;
 
