@@ -3,18 +3,22 @@
 
 #include <simgear/compiler.h>
 
-#include STL_STRSTREAM
+//#include STL_STRSTREAM
+#include <sstream>
 
 #include "uiuc_parsefile.h"
 #include "uiuc_aircraft.h"
+#include "uiuc_warnings_errors.h"
 
-#if !defined (SG_HAVE_NATIVE_SGI_COMPILERS)
-SG_USING_STD(istrstream);
-#endif
+//SG_USING_STD(istrstream);
 
 int uiuc_1DdataFileReader( string file_name, 
-                            double x[100], 
-                            double y[100], 
+                            double x[], 
+                            double y[], 
                             int &xmax );
+int uiuc_1DdataFileReader( string file_name, 
+			   double x[], 
+			   int y[], 
+			   int &xmax );
 
 #endif // _1D_DATA_FILE_READER_H_
