@@ -15,7 +15,10 @@ public:
 		      INCIDENCE, FLAP0, FLAP1, SLAT, SPOILER, VECTOR,
                       BOOST, CASTERING, PROPPITCH, PROPFEATHER,
                       COLLECTIVE, CYCLICAIL, CYCLICELE, ROTORENGINEON,
-                      REVERSE_THRUST, WASTEGATE };
+                      TILTYAW, TILTPITCH, TILTROLL,
+                      ROTORBRAKE, ROTORENGINEMAXRELTORQUE, ROTORRELTARGET,
+                      ROTORBALANCE, REVERSE_THRUST, WASTEGATE,
+                      WINCHRELSPEED, HITCHOPEN, PLACEWINCH, FINDAITOW};
 
     enum { OPT_SPLIT  = 0x01,
            OPT_INVERT = 0x02,
