@@ -54,6 +54,11 @@ YASim::YASim(double dt)
     _fdm->getAirplane()->getModel()->getIntegrator()->setInterval(_dt);
 }
 
+YASim::~YASim()
+{
+    delete _fdm;
+}
+
 void YASim::report()
 {
     Airplane* a = _fdm->getAirplane();
@@ -108,6 +113,8 @@ void YASim::init()
     Airplane* a = _fdm->getAirplane();
     Model* m = a->getModel();
 
+    fgSetBool("/yasim/initialized", false);
+
     // Superclass hook
     common_init();
 
@@ -220,6 +227,11 @@ void YASim::update(double dt)
 
     // Reset the time increment.
     gr->setTimeOffset(0.0);
+
+    // Set this only after our first successful iteration, when all
+    // our output properties are there.
+    if(!fgGetBool("/yasim/initialized"))
+        fgSetBool("/yasim/initialized", true);
 }
 
 void YASim::copyToYASim(bool copyState)
@@ -371,13 +383,17 @@ void YASim::copyFromYASim()
     double lat, lon, alt;
     sgCartToGeod(s->pos, &lat, &lon, &alt);
     _set_Geodetic_Position(lat, lon, alt*M2FT);
-    _update_ground_elev_at_pos();
+    double groundlevel_m = get_groundlevel_m(lat, lon, alt);
+    _set_Runway_altitude(groundlevel_m*SG_METER_TO_FEET);
+    _set_Altitude_AGL((alt-groundlevel_m)*SG_METER_TO_FEET);
+
+    // the smallest agl of all gears
+    fgSetFloat("/position/gear-agl-m", model->getAGL());
+    fgSetFloat("/position/gear-agl-ft", model->getAGL()*M2FT);
 
     // UNUSED
     //_set_Geocentric_Position(Glue::geod2geocLat(lat), lon, alt*M2FT);
 
-    _set_Altitude_AGL(model->getAGL() * M2FT);
-
     // useful conversion matrix
     float xyz2ned[9];
     Glue::xyz2nedMat(lat, lon, xyz2ned);
@@ -437,7 +453,7 @@ void YASim::copyFromYASim()
 
     // orientation
     float alpha, beta;
-    Glue::calcAlphaBeta(s, &alpha, &beta);
+    Glue::calcAlphaBeta(s, wind, &alpha, &beta);
     _set_Alpha(alpha);
     _set_Beta(beta);
 
@@ -468,6 +484,8 @@ void YASim::copyFromYASim()
 	node->setFloatValue("compression-m", g->getCompressDist());
         node->setFloatValue("caster-angle-deg", g->getCasterAngle() * RAD2DEG);
         node->setFloatValue("rollspeed-ms", g->getRollSpeed());
+        node->setBoolValue("ground-is-solid", g->getGroundIsSolid()!=0);
+        node->setFloatValue("ground-friction-factor", g->getGroundFrictionFactor());
     }
 
     Hook* h = airplane->getHook();
