@@ -1,4 +1,3 @@
-#include <string.h>	// strncpy()
 
 #include <Main/globals.hxx>
 #include <Airports/simple.hxx>
@@ -8,6 +7,7 @@
 
 AirportList::AirportList (int x, int y, int width, int height)
     : puList(x, y, width, height),
+      GUI_ID(FGCLASS_AIRPORTLIST),
       _airports(globals->get_airports()),
       _content(0)
 {
@@ -22,11 +22,8 @@ AirportList::~AirportList ()
 void
 AirportList::create_list ()
 {
-    if (_content)
-        destroy_list();
-
     int num_apt = _airports->size();
-    _content = new char *[num_apt + 1];
+    char **content = new char *[num_apt + 1];
 
     int n = 0;
     for (int i = 0; i < num_apt; i++) {
@@ -36,12 +33,18 @@ AirportList::create_list ()
         if (!_filter.empty() && entry.find(_filter) == STD::string::npos)
             continue;
 
-        _content[n] = new char[entry.size() + 1];
-        strcpy(_content[n], entry.c_str());
+        content[n] = new char[entry.size() + 1];
+        strcpy(content[n], entry.c_str());
         n++;
     }
-    _content[n] = 0;
-    newList(_content);
+    content[n] = 0;
+    // work around plib 2006/04/18 bug: lists with no entries cause crash on arrow-up
+    newList(n > 0 ? content : 0);
+
+    if (_content)
+        destroy_list();
+
+    _content = content;
 }
 
 void
