@@ -4,18 +4,75 @@
 #  include "config.h"
 #endif
 
-#include <stdlib.h>		// atof()
-
 #include <Input/input.hxx>
 #include <Scripting/NasalSys.hxx>
 
 #include "dialog.hxx"
 #include "new_gui.hxx"
-
-#include "puList.hxx"
 #include "AirportList.hxx"
+#include "property_list.hxx"
 #include "layout.hxx"
 
+
+enum format_type { f_INVALID, f_INT, f_LONG, f_FLOAT, f_DOUBLE, f_STRING };
+static const int FORMAT_BUFSIZE = 255;
+
+/**
+ * Makes sure the format matches '%[ -+#]?\d*(\.\d*)?(l?[df]|s)', with
+ * only one number or string placeholder and otherwise arbitrary prefix
+ * and postfix containing only quoted percent signs (%%).
+ */
+static format_type
+validate_format(const char *f)
+{
+    bool l = false;
+    format_type type;
+    for (; *f; f++) {
+        if (*f == '%') {
+            if (f[1] == '%')
+                f++;
+            else
+                break;
+        }
+    }
+    if (*f++ != '%')
+        return f_INVALID;
+    while (*f == ' ' || *f == '+' || *f == '-' || *f == '#' || *f == '0')
+        f++;
+    while (*f && isdigit(*f))
+        f++;
+    if (*f == '.') {
+        f++;
+        while (*f && isdigit(*f))
+            f++;
+    }
+
+    if (*f == 'l')
+        l = true, f++;
+
+    if (*f == 'd') {
+        type = l ? f_LONG : f_INT;
+    } else if (*f == 'f')
+        type = l ? f_DOUBLE : f_FLOAT;
+    else if (*f == 's') {
+        if (l)
+            return f_INVALID;
+        type = f_STRING;
+    } else
+        return f_INVALID;
+
+    for (++f; *f; f++) {
+        if (*f == '%') {
+            if (f[1] == '%')
+                f++;
+            else
+                return f_INVALID;
+        }
+    }
+    return type;
+}
+
+
 ////////////////////////////////////////////////////////////////////////
 // Implementation of GUIInfo.
 ////////////////////////////////////////////////////////////////////////
@@ -25,17 +82,24 @@
  */
 struct GUIInfo
 {
-    GUIInfo (FGDialog * d);
-    virtual ~GUIInfo ();
+    GUIInfo(FGDialog * d);
+    virtual ~GUIInfo();
+    char *format(SGPropertyNode *);
 
     FGDialog * dialog;
     vector <FGBinding *> bindings;
     int key;
+    char *text;
+    char *fmt_string;
+    format_type fmt_type;
 };
 
 GUIInfo::GUIInfo (FGDialog * d)
     : dialog(d),
-      key(-1)
+      key(-1),
+      text(0),
+      fmt_string(0),
+      fmt_type(f_INVALID)
 {
 }
 
@@ -45,8 +109,28 @@ GUIInfo::~GUIInfo ()
         delete bindings[i];
         bindings[i] = 0;
     }
+    delete [] text;
+    delete [] fmt_string;
 }
 
+char *GUIInfo::format(SGPropertyNode *n)
+{
+    if (fmt_type == f_INT)
+        snprintf(text, FORMAT_BUFSIZE, fmt_string, n->getIntValue());
+    else if (fmt_type == f_LONG)
+        snprintf(text, FORMAT_BUFSIZE, fmt_string, n->getLongValue());
+    else if (fmt_type == f_FLOAT)
+        snprintf(text, FORMAT_BUFSIZE, fmt_string, n->getFloatValue());
+    else if (fmt_type == f_DOUBLE)
+        snprintf(text, FORMAT_BUFSIZE, fmt_string, n->getDoubleValue());
+    else
+        snprintf(text, FORMAT_BUFSIZE, fmt_string, n->getStringValue());
+
+    text[FORMAT_BUFSIZE] = '\0';
+    return text;
+}
+
+
 
 /**
  * Key handler.
@@ -77,7 +161,7 @@ int fgPopup::checkKey(int key, int updown)
 puObject *fgPopup::getKeyObject(puObject *object, int key)
 {
     puObject *ret;
-    if(object->getType() & PUCLASS_GROUP)
+    if (object->getType() & PUCLASS_GROUP)
         for (puObject *obj = ((puGroup *)object)->getFirstChild();
                 obj; obj = obj->getNextObject())
             if ((ret = getKeyObject(obj, key)))
@@ -93,7 +177,7 @@ puObject *fgPopup::getKeyObject(puObject *object, int key)
 puObject *fgPopup::getActiveInputField(puObject *object)
 {
     puObject *ret;
-    if(object->getType() & PUCLASS_GROUP)
+    if (object->getType() & PUCLASS_GROUP)
         for (puObject *obj = ((puGroup *)object)->getFirstChild();
                 obj; obj = obj->getNextObject())
             if ((ret = getActiveInputField(obj)))
@@ -112,7 +196,7 @@ int fgPopup::checkHit(int button, int updown, int x, int y)
 {
     int result = puPopup::checkHit(button, updown, x, y);
 
-    if ( !_draggable)
+    if (!_draggable)
        return result;
 
     // This is annoying.  We would really want a true result from the
@@ -121,12 +205,12 @@ int fgPopup::checkHit(int button, int updown, int x, int y)
     // intersection test (again) to make sure we don't start a drag
     // when inside controls.
 
-    if(updown == PU_DOWN && !_dragging) {
-        if(!result)
+    if (updown == PU_DOWN && !_dragging) {
+        if (!result)
             return 0;
 
         int hit = getHitObjects(this, x, y);
-        if(hit & (PUCLASS_BUTTON|PUCLASS_ONESHOT|PUCLASS_INPUT))
+        if (hit & (PUCLASS_BUTTON|PUCLASS_ONESHOT|PUCLASS_INPUT))
             return result;
 
         int px, py;
@@ -134,7 +218,7 @@ int fgPopup::checkHit(int button, int updown, int x, int y)
         _dragging = true;
         _dX = px - x;
         _dY = py - y;
-    } else if(updown == PU_DRAG && _dragging) {
+    } else if (updown == PU_DRAG && _dragging) {
         setPosition(x + _dX, y + _dY);
     } else {
         _dragging = false;
@@ -144,8 +228,11 @@ int fgPopup::checkHit(int button, int updown, int x, int y)
 
 int fgPopup::getHitObjects(puObject *object, int x, int y)
 {
+    if (!object->isVisible())
+        return 0;
+
     int type = 0;
-    if(object->getType() & PUCLASS_GROUP)
+    if (object->getType() & PUCLASS_GROUP)
         for (puObject *obj = ((puGroup *)object)->getFirstChild();
                 obj; obj = obj->getNextObject())
             type |= getHitObjects(obj, x, y);
@@ -153,7 +240,7 @@ int fgPopup::getHitObjects(puObject *object, int x, int y)
     int cx, cy, cw, ch;
     object->getAbsolutePosition(&cx, &cy);
     object->getSize(&cw, &ch);
-    if(x >= cx && x < cx + cw && y >= cy && y < cy + ch)
+    if (x >= cx && x < cx + cw && y >= cy && y < cy + ch)
         type |= object->getType();
     return type;
 }
@@ -183,71 +270,6 @@ action_callback (puObject * object)
 }
 
 
-static void
-format_callback(puObject *obj, int dx, int dy, void *n)
-{
-    SGPropertyNode *node = (SGPropertyNode *)n;
-    const char *format = node->getStringValue("format"), *f = format;
-    bool number, l = false;
-    // make sure the format matches '[ -+#]?\d*(\.\d*)?l?[fs]'
-    for (; *f; f++) {
-        if (*f == '%') {
-            if (f[1] == '%')
-                f++;
-            else
-                break;
-        }
-    }
-    if (*f++ != '%')
-        return;
-    if (*f == ' ' || *f == '+' || *f == '-' || *f == '#')
-        f++;
-    while (*f && isdigit(*f))
-        f++;
-    if (*f == '.') {
-        f++;
-        while (*f && isdigit(*f))
-            f++;
-    }
-    if (*f == 'l')
-        l = true, f++;
-
-    if (*f == 'f')
-        number = true;
-    else if (*f == 's') {
-        if (l)
-            return;
-        number = false;
-    } else
-        return;
-
-    for (++f; *f; f++) {
-        if (*f == '%') {
-            if (f[1] == '%')
-                f++;
-            else
-                return;
-        }
-    }
-
-    char buf[256];
-    const char *src = obj->getLabel();
-
-    if (number) {
-        float value = atof(src);
-        snprintf(buf, 256, format, value);
-    } else {
-        snprintf(buf, 256, format, src);
-    }
-
-    buf[255] = '\0';
-
-    SGPropertyNode *result = node->getNode("formatted", true);
-    result->setStringValue(buf);
-    obj->setLabel(result->getStringValue());
-}
-
-
 
 ////////////////////////////////////////////////////////////////////////
 // Static helper functions.
@@ -261,8 +283,12 @@ copy_to_pui (SGPropertyNode * node, puObject * object)
 {
     // Treat puText objects specially, so their "values" can be set
     // from properties.
-    if(object->getType() & PUCLASS_TEXT) {
-        object->setLabel(node->getStringValue());
+    if (object->getType() & PUCLASS_TEXT) {
+        GUIInfo *info = (GUIInfo *)object->getUserData();
+        if (info && info->fmt_string)
+            object->setLabel(info->format(node));
+        else
+            object->setLabel(node->getStringValue());
         return;
     }
 
@@ -287,7 +313,7 @@ static void
 copy_from_pui (puObject * object, SGPropertyNode * node)
 {
     // puText objects are immutable, so should not be copied out
-    if(object->getType() & PUCLASS_TEXT)
+    if (object->getType() & PUCLASS_TEXT)
         return;
 
     switch (node->getType()) {
@@ -335,7 +361,7 @@ FGDialog::FGDialog (SGPropertyNode * props)
         if (open) {
             const char *s = open->getStringValue();
             FGNasalSys *nas = (FGNasalSys *)globals->get_subsystem("nasal");
-            nas->createModule(_module.c_str(), _module.c_str(), s, strlen(s));
+            nas->createModule(_module.c_str(), _module.c_str(), s, strlen(s), props);
         }
     }
     display(props);
@@ -351,24 +377,13 @@ FGDialog::~FGDialog ()
     FGNasalSys *nas = (FGNasalSys *)globals->get_subsystem("nasal");
     if (_nasal_close) {
         const char *s = _nasal_close->getStringValue();
-        nas->createModule(_module.c_str(), _module.c_str(), s, strlen(s));
+        nas->createModule(_module.c_str(), _module.c_str(), s, strlen(s), _props);
     }
     nas->deleteModule(_module.c_str());
 
     puDeleteObject(_object);
 
     unsigned int i;
-
-                                // Delete all the arrays we made
-                                // and were forced to keep around
-                                // because PUI won't do its own
-                                // memory management.
-    for (i = 0; i < _char_arrays.size(); i++) {
-        for (int j = 0; _char_arrays[i][j] != 0; j++)
-            free(_char_arrays[i][j]); // added with strdup
-        delete[] _char_arrays[i];
-    }
-
                                 // Delete all the info objects we
                                 // were forced to keep around because
                                 // PUI cannot delete its own user data.
@@ -376,7 +391,6 @@ FGDialog::~FGDialog ()
         delete (GUIInfo *)_info[i];
         _info[i] = 0;
     }
-
                                 // Finally, delete the property links.
     for (i = 0; i < _propertyObjects.size(); i++) {
         delete _propertyObjects[i];
@@ -385,39 +399,39 @@ FGDialog::~FGDialog ()
 }
 
 void
-FGDialog::updateValue (const char * objectName)
+FGDialog::updateValues (const char * objectName)
 {
+    if (objectName && !objectName[0])
+        objectName = 0;
+
     for (unsigned int i = 0; i < _propertyObjects.size(); i++) {
         const string &name = _propertyObjects[i]->name;
-        if (name == objectName)
-            copy_to_pui(_propertyObjects[i]->node,
-                        _propertyObjects[i]->object);
+        if (objectName && name != objectName)
+            continue;
+
+        puObject *obj = _propertyObjects[i]->object;
+        if ((obj->getType() & PUCLASS_LIST) && (dynamic_cast<GUI_ID *>(obj)->id & FGCLASS_LIST)) {
+            fgList *pl = static_cast<fgList *>(obj);
+            pl->update();
+        } else
+            copy_to_pui(_propertyObjects[i]->node, obj);
     }
 }
 
 void
-FGDialog::applyValue (const char * objectName)
+FGDialog::applyValues (const char * objectName)
 {
+    if (objectName && !objectName[0])
+        objectName = 0;
+
     for (unsigned int i = 0; i < _propertyObjects.size(); i++) {
-        if (_propertyObjects[i]->name == objectName)
-            copy_from_pui(_propertyObjects[i]->object,
-                          _propertyObjects[i]->node);
-    }
-}
+        const string &name = _propertyObjects[i]->name;
+        if (objectName && name != objectName)
+            continue;
 
-void
-FGDialog::updateValues ()
-{
-    for (unsigned int i = 0; i < _propertyObjects.size(); i++)
-        copy_to_pui(_propertyObjects[i]->node, _propertyObjects[i]->object);
-}
-
-void
-FGDialog::applyValues ()
-{
-    for (unsigned int i = 0; i < _propertyObjects.size(); i++)
         copy_from_pui(_propertyObjects[i]->object,
                       _propertyObjects[i]->node);
+    }
 }
 
 void
@@ -453,7 +467,7 @@ FGDialog::display (SGPropertyNode * props)
 
     SGPropertyNode *fontnode = props->getNode("font");
     if (fontnode) {
-        FGFontCache *fc = _gui->get_fontcache();
+        FGFontCache *fc = globals->get_fontcache();
         _font = fc->get(fontnode);
     } else {
         _font = _gui->getDefaultFont();
@@ -521,7 +535,7 @@ FGDialog::makeObject (SGPropertyNode * props, int parentWidth, int parentHeight)
     int y = props->getIntValue("y", (parentHeight - height) / 2);
     string type = props->getName();
 
-    if (type == "")
+    if (type.empty())
         type = "dialog";
 
     if (type == "dialog") {
@@ -554,13 +568,8 @@ FGDialog::makeObject (SGPropertyNode * props, int parentWidth, int parentHeight)
         return obj;
 
     } else if (type == "list") {
-        vector<SGPropertyNode_ptr> value_nodes = props->getChildren("value");
-        char ** entries = make_char_array(value_nodes.size());
-        for (unsigned int i = 0; i < value_nodes.size(); i++)
-            entries[i] = strdup((char *)value_nodes[i]->getStringValue());
-
         int slider_width = props->getIntValue("slider", 20);
-        puList * obj = new puList(x, y, x + width, y + height, entries, slider_width);
+        fgList * obj = new fgList(x, y, x + width, y + height, props, slider_width);
         if (presetSize)
             obj->setSize(width, height);
         setupObject(obj, props);
@@ -575,6 +584,14 @@ FGDialog::makeObject (SGPropertyNode * props, int parentWidth, int parentHeight)
         setColor(obj, props);
         return obj;
 
+    } else if (type == "property-list") {
+        PropertyList * obj = new PropertyList(x, y, x + width, y + height, globals->get_props());
+        if (presetSize)
+            obj->setSize(width, height);
+        setupObject(obj, props);
+        setColor(obj, props);
+        return obj;
+
     } else if (type == "input") {
         puInput * obj = new puInput(x, y, x + width, y + height);
         setupObject(obj, props);
@@ -585,13 +602,6 @@ FGDialog::makeObject (SGPropertyNode * props, int parentWidth, int parentHeight)
         puText * obj = new puText(x, y);
         setupObject(obj, props);
 
-        if (props->getNode("format")) {
-            SGPropertyNode *live = props->getNode("live");
-            if (live && live->getBoolValue())
-                obj->setRenderCallback(format_callback, props);
-            else
-                format_callback(obj, x, y, props);
-        }
         // Layed-out objects need their size set, and non-layout ones
         // get a different placement.
         if (presetSize)
@@ -629,12 +639,7 @@ FGDialog::makeObject (SGPropertyNode * props, int parentWidth, int parentHeight)
         return obj;
 
     } else if (type == "combo") {
-        vector<SGPropertyNode_ptr> value_nodes = props->getChildren("value");
-        char ** entries = make_char_array(value_nodes.size());
-        for (unsigned int i = 0; i < value_nodes.size(); i++)
-            entries[i] = strdup((char *)value_nodes[i]->getStringValue());
-
-        puComboBox * obj = new puComboBox(x, y, x + width, y + height, entries,
+        fgComboBox * obj = new fgComboBox(x, y, x + width, y + height, props,
                            props->getBoolValue("editable", false));
         setupObject(obj, props);
 #ifdef PUCOL_EDITFIELD  // plib > 0.8.4
@@ -667,7 +672,7 @@ FGDialog::makeObject (SGPropertyNode * props, int parentWidth, int parentHeight)
     } else if (type == "textbox") {
         int slider_width = props->getIntValue("slider", 20);
         int wrap = props->getBoolValue("wrap", true);
-        puLargeInput * obj = new puLargeInput(x, y,
+        puaLargeInput * obj = new puaLargeInput(x, y,
                 x+width, x+height, 2, slider_width, wrap);
 
         if (props->hasValue("editable")) {
@@ -683,19 +688,7 @@ FGDialog::makeObject (SGPropertyNode * props, int parentWidth, int parentHeight)
         return obj;
 
     } else if (type == "select") {
-        vector<SGPropertyNode_ptr> value_nodes;
-        SGPropertyNode * selection_node =
-                fgGetNode(props->getChild("selection")->getStringValue(), true);
-
-        for (int q = 0; q < selection_node->nChildren(); q++)
-            value_nodes.push_back(selection_node->getChild(q));
-
-        char ** entries = make_char_array(value_nodes.size());
-        for (unsigned int i = 0; i < value_nodes.size(); i++)
-            entries[i] = strdup((char *)value_nodes[i]->getName());
-
-        puSelectBox * obj =
-            new puSelectBox(x, y, x + width, y + height, entries);
+        fgSelectBox * obj = new fgSelectBox(x, y, x + width, y + height, props);
         setupObject(obj, props);
 #ifdef PUCOL_EDITFIELD  // plib > 0.8.4
         setColor(obj, props, EDITFIELD);
@@ -724,7 +717,7 @@ FGDialog::setupObject (puObject * object, SGPropertyNode * props)
         object->setBorderThickness( props->getIntValue("border", 2) );
 
     if ( SGPropertyNode *nft = props->getNode("font", false) ) {
-       FGFontCache *fc = _gui->get_fontcache();
+       FGFontCache *fc = globals->get_fontcache();
        puFont *lfnt = fc->get(nft);
        object->setLabelFont(*lfnt);
     } else {
@@ -738,15 +731,16 @@ FGDialog::setupObject (puObject * object, SGPropertyNode * props)
         const char * propname = props->getStringValue("property");
         SGPropertyNode_ptr node = fgGetNode(propname, true);
         copy_to_pui(node, object);
+
         PropertyObject* po = new PropertyObject(name, object, node);
         _propertyObjects.push_back(po);
-        if(props->getBoolValue("live"))
+        if (props->getBoolValue("live"))
             _liveObjects.push_back(po);
     }
 
     SGPropertyNode * dest = fgGetNode("/sim/bindings/gui", true);
     vector<SGPropertyNode_ptr> bindings = props->getChildren("binding");
-    if (bindings.size() > 0) {
+    if (type == "text" || bindings.size() > 0) {
         GUIInfo * info = new GUIInfo(this);
         info->key = props->getIntValue("keynum", -1);
         if (props->hasValue("key"))
@@ -771,6 +765,21 @@ FGDialog::setupObject (puObject * object, SGPropertyNode * props)
         if (type == "input" && props->getBoolValue("live"))
             object->setDownCallback(action_callback);
 
+        if (type == "text") {
+            const char *format = props->getStringValue("format", 0);
+            if (format) {
+                info->fmt_type = validate_format(props->getStringValue("format", 0));
+                if (info->fmt_type != f_INVALID) {
+                    info->text = new char[FORMAT_BUFSIZE + 1];
+                    info->fmt_string = new char[strlen(format) + 1];
+                    strcpy(info->fmt_string, format);
+                } else {
+                    SG_LOG(SG_GENERAL, SG_ALERT, "DIALOG: invalid <format> '"
+                            << format << '\'');
+                }
+            }
+        }
+
         object->setUserData(info);
         _info.push_back(info);
     }
@@ -799,7 +808,7 @@ void
 FGDialog::setColor(puObject * object, SGPropertyNode * props, int which)
 {
     string type = props->getName();
-    if (type == "")
+    if (type.empty())
         type = "dialog";
     if (type == "textbox" && props->getBoolValue("editable"))
         type += "-editable";
@@ -841,6 +850,8 @@ FGDialog::setColor(puObject * object, SGPropertyNode * props, int which)
         if ((pucol[i].mask == LABEL) && !c->isValid())
             dirty |= c->merge(_gui->getColor("label"));
 
+        dirty |= c->merge(props->getNode(pucol[i].cname));
+
         if (c->isValid() && dirty)
             object->setColor(pucol[i].id, c->red(), c->green(), c->blue(), c->alpha());
     }
@@ -941,16 +952,6 @@ FGDialog::getKeyCode(const char *str)
     return key;
 }
 
-char **
-FGDialog::make_char_array (int size)
-{
-    char ** list = new char*[size+1];
-    for (int i = 0; i <= size; i++)
-        list[i] = 0;
-    _char_arrays.push_back(list);
-    return list;
-}
-
 
 
 ////////////////////////////////////////////////////////////////////////
@@ -967,4 +968,58 @@ FGDialog::PropertyObject::PropertyObject (const char * n,
 }
 
 
+
+
+////////////////////////////////////////////////////////////////////////
+// Implementation of fgValueList and derived pui widgets
+////////////////////////////////////////////////////////////////////////
+
+
+fgValueList::fgValueList(SGPropertyNode *p) :
+    _props(p)
+{
+    make_list();
+}
+
+void
+fgValueList::update()
+{
+    destroy_list();
+    make_list();
+}
+
+fgValueList::~fgValueList()
+{
+    destroy_list();
+}
+
+void
+fgValueList::make_list()
+{
+    vector<SGPropertyNode_ptr> value_nodes = _props->getChildren("value");
+    _list = new char *[value_nodes.size() + 1];
+    unsigned int i;
+    for (i = 0; i < value_nodes.size(); i++)
+        _list[i] = strdup((char *)value_nodes[i]->getStringValue());
+    _list[i] = 0;
+}
+
+void
+fgValueList::destroy_list()
+{
+    for (int i = 0; _list[i] != 0; i++)
+        if (_list[i])
+            free(_list[i]);
+    delete[] _list;
+}
+
+
+
+void
+fgList::update()
+{
+    fgValueList::update();
+    newList(_list);
+}
+
 // end of dialog.cxx
