@@ -20,7 +20,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Id: gui.cxx,v 1.21 2006-02-21 01:18:35 mfranz Exp $
+ * $Id: gui.cxx,v 1.22.2.1 2006-11-01 22:54:32 mfranz Exp $
  **************************************************************************/
 
 
@@ -45,15 +45,14 @@
 #include <Main/main.hxx>
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
+#include <GUI/new_gui.hxx>
 
 #include "gui.h"
 #include "gui_local.hxx"
-#include "preset_dlg.hxx"
 #include "layout.hxx"
 
 
 puFont guiFnt = 0;
-fntTexFont *guiFntHandle = 0;
 
 
 /* -------------------------------------------------------------------------
@@ -66,30 +65,18 @@ void guiInit()
     char *mesa_win_state;
 
     // Initialize PUI
+#ifndef PU_USE_NONE
     puInit();
+#endif
     puSetDefaultStyle         ( PUSTYLE_SMALL_SHADED ); //PUSTYLE_DEFAULT
     puSetDefaultColourScheme  (0.8, 0.8, 0.9, 1);
 
-    // Next check home directory
-    SGPath fntpath;
-    char* envp = ::getenv( "FG_FONTS" );
-    if ( envp != NULL ) {
-        fntpath.set( envp );
-    } else {
-        fntpath.set( globals->get_fg_root() );
-	fntpath.append( "Fonts" );
-    }
-
-    // Install our fast fonts
-    SGPropertyNode *locale = globals->get_locale();
-    fntpath.append(locale->getStringValue("font", "typewriter.txf"));
-    guiFntHandle = new fntTexFont ;
-    guiFntHandle -> load ( (char *)fntpath.c_str() ) ;
-    puFont GuiFont ( guiFntHandle, 15 ) ;
-    puSetDefaultFonts( GuiFont, GuiFont ) ;
+    FGFontCache *fc = globals->get_fontcache();
+    puFont *GuiFont = fc->get(globals->get_locale()->getStringValue("font", "typewriter.txf"), 15);
+    puSetDefaultFonts(*GuiFont, *GuiFont);
     guiFnt = puGetDefaultLabelFont();
 
-    LayoutWidget::setDefaultFont(&GuiFont, 15);
+    LayoutWidget::setDefaultFont(GuiFont, 15);
   
     if (!fgHasNode("/sim/startup/mouse-pointer")) {
         // no preference specified for mouse pointer, attempt to autodetect...
@@ -116,7 +103,4 @@ void guiInit()
 	
     // MOUSE_VIEW mode stuff
 	initMouseQuat();
-
-    // Set up our Dialog Boxes
-    fgPresetInit();
 }
