@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Id: gui.h,v 1.13 2006-02-21 01:18:36 mfranz Exp $
+ * $Id: gui.h,v 1.13.2.3 2006-11-01 23:36:39 mfranz Exp $
  **************************************************************************/
 
 
@@ -53,8 +53,6 @@ extern fntTexFont *guiFntHandle;
 extern int gui_menu_on;
 
 // from gui_funcs.cxx
-extern void saveFlight(puObject *);
-extern void loadFlight(puObject *);
 extern void reInit(puObject *);
 extern void fgDumpSnapShotWrapper(puObject *);
 #ifdef TR_HIRES_SNAP
@@ -64,24 +62,7 @@ extern void fgHiResDump();
 #if defined( WIN32 ) && !defined( __CYGWIN__) && !defined(__MINGW32__)
 extern void printScreen(puObject *);
 #endif
-extern void guiTogglePanel(puObject *);
 extern void PilotOffsetAdjust(puObject *);
-extern void fgHUDalphaAdjust(puObject *);
-extern void prop_pickerView(puObject *);
-extern void NewAirport(puObject *);
-#ifdef FG_NETWORK_OLK
-extern void net_display_toggle(puObject *);
-extern void NewCallSign(puObject *);
-extern void net_fgd_scan(puObject *);
-extern void net_register(puObject *);
-extern void net_unregister(puObject *);
-#endif
-extern void NewAltitude(puObject *);
-extern void AddWayPoint(puObject *);
-extern void PopWayPoint(puObject *);
-extern void ClearRoute(puObject *);
-extern void fgAPAdjust(puObject *);
-extern void fgLatLonFormatToggle(puObject *);
 extern void helpCb(puObject *);
 extern void fgReshape(int, int);
 
