@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Id: gui_funcs.cxx,v 1.41 2006-02-21 01:18:36 mfranz Exp $
+ * $Id: gui_funcs.cxx,v 1.46.2.6 2007-07-03 15:35:57 mfranz Exp $
  **************************************************************************/
 
 
@@ -60,6 +60,8 @@
 # endif
 #endif
 
+#include <sstream>
+
 #include <plib/ssg.h>
 
 #include <simgear/constants.h>
@@ -71,7 +73,6 @@
 #include <Aircraft/aircraft.hxx>
 #include <Aircraft/controls.hxx>
 #include <Airports/simple.hxx>
-#include <Autopilot/auto_gui.hxx>
 #include <Cockpit/panel.hxx>
 #include <FDM/flight.hxx>
 #include <Main/main.hxx>
@@ -90,15 +91,11 @@
 
 #include "gui.h"
 #include "gui_local.hxx"
-#include "preset_dlg.hxx"
-#include "prop_picker.hxx"
 #include "sgVec3Slider.hxx"
 
 SG_USING_STD(string);
 SG_USING_STD(cout);
 
-// from cockpit.cxx
-extern void fgLatLonFormatToggle( puObject *);
 
 #if defined( TR_HIRES_SNAP)
 #include <simgear/screen/tr.h>
@@ -126,28 +123,6 @@ const __fg_gui_fn_t __fg_gui_fn[] = {
 #if defined( WIN32 ) && !defined( __CYGWIN__) && !defined(__MINGW32__)
         {"printScreen", printScreen},
 #endif
-
-        //View
-        {"guiTogglePanel", guiTogglePanel},
-        {"prop_pickerView", prop_pickerView},
-
-        // Environment
-        {"fgPresetAirport", fgPresetAirport},
-        {"fgPresetRunway", fgPresetRunway},
-        {"fgPresetOffsetDistance", fgPresetOffsetDistance},
-        {"fgPresetAltitude", fgPresetAltitude},
-        {"fgPresetGlideslope", fgPresetGlideslope},
-        {"fgPresetAirspeed", fgPresetAirspeed},
-        {"fgPresetCommit", fgPresetCommit},
-
-        // Autopilot
-        {"NewAltitude", NewAltitude},
-	{"NewHeading", NewHeading},
-        {"AddWayPoint", AddWayPoint},
-        {"PopWayPoint", PopWayPoint},
-        {"ClearRoute", ClearRoute},
-        {"fgLatLonFormatToggle", fgLatLonFormatToggle},
-
         // Help
         {"helpCb", helpCb},
 
@@ -158,19 +133,45 @@ const __fg_gui_fn_t __fg_gui_fn[] = {
 
 /* ================ General Purpose Functions ================ */
 
-// General Purpose Message Box
+// General Purpose Message Box. Makes sure no more than 5 different
+// messages are displayed at the same time, and none of them are
+// duplicates. (5 is a *lot*, but this will hardly ever be reached
+// and we don't want to miss any, either.)
 void mkDialog (const char *txt)
 {
     NewGUI *gui = (NewGUI *)globals->get_subsystem("gui");
     if (!gui)
         return;
-    SGPropertyNode_ptr dlg = gui->getDialog("message");
-    if (!dlg)
+    SGPropertyNode *master = gui->getDialogProperties("message");
+    if (!master)
         return;
 
-    dlg->setStringValue("text/label", txt);
-    dlg->setStringValue("button/legend", "OK");
-    gui->showDialog("message");
+    const int maxdialogs = 5;
+    string name;
+    SGPropertyNode *msg = fgGetNode("/sim/gui/dialogs", true);
+    int i;
+    for (i = 0; i < maxdialogs; i++) {
+        std::ostringstream s;
+        s << "message-" << i;
+        name = s.str();
+
+        if (!msg->getNode(name.c_str(), false))
+            break;
+
+        if (!strcmp(txt, msg->getNode(name.c_str())->getStringValue("message"))) {
+            SG_LOG(SG_GENERAL, SG_WARN, "mkDialog(): duplicate of message " << txt);
+            return;
+        }
+    }
+    if (i == maxdialogs)
+        return;
+    msg = msg->getNode(name.c_str(), true);
+    msg->setStringValue("message", txt);
+    msg = msg->getNode("dialog", true);
+    copyProperties(master, msg);
+    msg->setStringValue("name", name.c_str());
+    gui->newDialog(msg);
+    gui->showDialog(name.c_str());
 }
 
 // Message Box to report an error.
@@ -187,9 +188,9 @@ void guiErrorMessage (const char *txt, const sg_throwable &throwable)
     msg += '\n';
     msg += throwable.getFormattedMessage();
     if (!throwable.getOrigin().empty()) {
-      msg += "\n (reported by ";
-      msg += throwable.getOrigin();
-      msg += ')';
+        msg += "\n (reported by ";
+        msg += throwable.getOrigin();
+        msg += ')';
     }
     SG_LOG(SG_GENERAL, SG_ALERT, msg);
     mkDialog(msg.c_str());
@@ -207,18 +208,6 @@ ____________________________________________________________________*/
 // on Sunday 3rd of December
 
 
-// This is the accessor function
-void guiTogglePanel(puObject *cb)
-{
-  if (fgGetBool("/sim/panel/visibility"))
-    fgSetBool("/sim/panel/visibility", false);
-  else
-    fgSetBool("/sim/panel/visibility", true);
-
-  globals->get_renderer()->resize(fgGetInt("/sim/startup/xsize"),
-                                  fgGetInt("/sim/startup/ysize"));
-}
-
 void helpCb (puObject *)
 {
     string command;
@@ -546,7 +535,6 @@ void fgHiResDumpWrapper ( puObject *obj ) {
 void fgDumpSnapShot () {
     bool show_pu_cursor = false;
     char *filename = new char [24];
-    string message;
     static int count = 1;
 
     static const SGPropertyNode *master_freeze
@@ -562,6 +550,7 @@ void fgDumpSnapShot () {
 	show_pu_cursor = true;
 	puHideCursor();
     }
+    fgSetBool("/sim/signals/screenshot", true);
 
     FGRenderer *renderer = globals->get_renderer();
     renderer->init();
@@ -582,19 +571,13 @@ void fgDumpSnapShot () {
         fclose(fp);
     }
 
-    if ( sg_glDumpWindow( filename,
-			  fgGetInt("/sim/startup/xsize"), 
-			  fgGetInt("/sim/startup/ysize")) ) {
-	message = "Snapshot saved to \"";
-	message += filename;
-	message += "\".";
-    } else {
-        message = "Failed to save to \"";
-	message += filename;
-	message += "\".";
-    }
+    int result = sg_glDumpWindow(filename,
+                                 fgGetInt("/sim/startup/xsize"),
+                                 fgGetInt("/sim/startup/ysize"));
+    fgSetString("/sim/last-screenshot", result ? filename : "");
 
-    mkDialog (message.c_str());
+    fgSetBool("/sim/signals/screenshot", false);
+    //mkDialog (message.c_str());
 
     delete [] filename;
 
