@@ -26,8 +26,7 @@ extern puFont FONT_SANS_12B;
 
 
 NewGUI::NewGUI ()
-    : _fontcache(new FGFontCache),
-      _menubar(new FGMenuBar),
+    : _menubar(new FGMenuBar),
       _active_dialog(0)
 {
 }
@@ -36,6 +35,8 @@ NewGUI::~NewGUI ()
 {
     delete _menubar;
     _dialog_props.clear();
+    for (_itt_t it = _colors.begin(); it != _colors.end(); ++it)
+        delete it->second;
 }
 
 void
@@ -54,6 +55,7 @@ void
 NewGUI::reinit ()
 {
     reset(true);
+    fgSetBool("/sim/signals/reinit-gui", true);
 }
 
 void
@@ -166,12 +168,22 @@ NewGUI::closeDialog (const string& name)
 }
 
 SGPropertyNode_ptr
-NewGUI::getDialog (const string &name)
+NewGUI::getDialogProperties (const string &name)
 {
     if(_dialog_props.find(name) != _dialog_props.end())
         return _dialog_props[name];
 
-    SG_LOG(SG_GENERAL, SG_ALERT, "dialog '" << name << "' missing");
+    SG_LOG(SG_GENERAL, SG_DEBUG, "dialog '" << name << "' missing");
+    return 0;
+}
+
+FGDialog *
+NewGUI::getDialog (const string &name)
+{
+    if(_active_dialogs.find(name) != _active_dialogs.end())
+        return _active_dialogs[name];
+
+    SG_LOG(SG_GENERAL, SG_DEBUG, "dialog '" << name << "' missing");
     return 0;
 }
 
@@ -324,29 +336,10 @@ NewGUI::setStyle (void)
 }
 
 
-
-
-static const struct {
-    char *name;
-    puFont *font;
-} guifonts[] = {
-    { "default",      &FONT_HELVETICA_14 },
-    { "FIXED_8x13",   &PUFONT_8_BY_13 },
-    { "FIXED_9x15",   &PUFONT_9_BY_15 },
-    { "TIMES_10",     &PUFONT_TIMES_ROMAN_10 },
-    { "TIMES_24",     &PUFONT_TIMES_ROMAN_24 },
-    { "HELVETICA_10", &PUFONT_HELVETICA_10 },
-    { "HELVETICA_12", &PUFONT_HELVETICA_12 },
-    { "HELVETICA_14", &FONT_HELVETICA_14 },
-    { "HELVETICA_18", &PUFONT_HELVETICA_18 },
-    { "SANS_12B",     &FONT_SANS_12B },
-    { 0, 0 }
-};
-
 void
 NewGUI::setupFont (SGPropertyNode *node)
 {
-    _font = _fontcache->get(node);
+    _font = globals->get_fontcache()->get(node);
     puSetDefaultFonts(*_font, *_font);
     return;
 }
@@ -393,33 +386,63 @@ FGColor::merge(const FGColor *color)
 }
 
 
-//
-FGFontCache::FGFontCache()
-{
-    char *envp = ::getenv("FG_FONTS");
-    if (envp != NULL) {
-        _path.set(envp);
-    } else {
-        _path.set(globals->get_fg_root());
-        _path.append("Fonts");
-    }
 
-    for (int i=0; guifonts[i].name; i++)
-        _fonts[guifonts[i].name] = new fnt(guifonts[i].font);
+
+////////////////////////////////////////////////////////////////////////
+// FGFontCache class.
+////////////////////////////////////////////////////////////////////////
+
+static const struct {
+    char *name;
+    puFont *font;
+} guifonts[] = {
+    { "default",      &FONT_HELVETICA_14 },
+    { "FIXED_8x13",   &PUFONT_8_BY_13 },
+    { "FIXED_9x15",   &PUFONT_9_BY_15 },
+    { "TIMES_10",     &PUFONT_TIMES_ROMAN_10 },
+    { "TIMES_24",     &PUFONT_TIMES_ROMAN_24 },
+    { "HELVETICA_10", &PUFONT_HELVETICA_10 },
+    { "HELVETICA_12", &PUFONT_HELVETICA_12 },
+    { "HELVETICA_14", &FONT_HELVETICA_14 },
+    { "HELVETICA_18", &PUFONT_HELVETICA_18 },
+    { "SANS_12B",     &FONT_SANS_12B },
+    { 0, 0 }
+};
+
+
+FGFontCache::FGFontCache() :
+    _initialized(false)
+{
 }
 
 FGFontCache::~FGFontCache()
 {
-   _fonts.clear();
+   map<const string, fnt *>::iterator it, end = _fonts.end();
+   for (it = _fonts.begin(); it != end; ++it)
+       delete it->second;
 }
 
-puFont *
-FGFontCache::get(const char *name, float size, float slant)
+struct FGFontCache::fnt *
+FGFontCache::getfnt(const char *name, float size, float slant)
 {
-    _itt_t it;
+    if (!_initialized) {
+        char *envp = ::getenv("FG_FONTS");
+        if (envp != NULL) {
+            _path.set(envp);
+        } else {
+            _path.set(globals->get_fg_root());
+            _path.append("Fonts");
+        }
 
+        for (int i = 0; guifonts[i].name; i++)
+            _fonts[guifonts[i].name] = new fnt(guifonts[i].font);
+
+        _initialized = true;
+    }
+
+    _itt_t it;
     if ((it = _fonts.find(name)) != _fonts.end())
-        return it->second->pufont;
+        return it->second;
 
     SGPath path(_path);
     path.append(name);
@@ -430,17 +453,31 @@ FGFontCache::get(const char *name, float size, float slant)
     if (f->texfont->load((char *)path.c_str())) {
         f->pufont = new puFont;
         f->pufont->initialize(static_cast<fntFont *>(f->texfont), size, slant);
-        _fonts[name] = f;
-        return f->pufont;
+        return _fonts[name] = f;
     }
 
     delete f;
-    return _fonts["default"]->pufont;
+    return _fonts["default"];
+}
+
+puFont *
+FGFontCache::get(const char *name, float size, float slant)
+{
+    return getfnt(name, size, slant)->pufont;
+}
+
+fntTexFont *
+FGFontCache::getTexFont(const char *name, float size, float slant)
+{
+    return getfnt(name, size, slant)->texfont;
 }
 
 puFont *
 FGFontCache::get(SGPropertyNode *node)
 {
+    if (!node)
+        return get("Helvetica.txf", 15.0, 0.0);
+
     const char *name = node->getStringValue("name", "Helvetica.txf");
     float size = node->getFloatValue("size", 15.0);
     float slant = node->getFloatValue("slant", 0.0);
