@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: input.cxx,v 1.85 2006-03-21 17:33:54 mfranz Exp $
+// $Id: input.cxx,v 1.87.2.10 2007-12-01 23:37:44 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -32,6 +32,7 @@
 
 #include <math.h>
 #include <ctype.h>
+#include <sstream>
 
 #include STL_FSTREAM
 #include STL_STRING
@@ -63,6 +64,7 @@
 #include <simgear/math/sg_geodesy.hxx>
 
 SG_USING_STD(ifstream);
+SG_USING_STD(ostringstream);
 SG_USING_STD(string);
 SG_USING_STD(vector);
 
@@ -77,6 +79,40 @@ void keyHandler(int key, int keymod, int mousex, int mousey);
 
 static FGInput * default_input = 0;
 
+
+////////////////////////////////////////////////////////////////////////
+// Local functions.
+////////////////////////////////////////////////////////////////////////
+
+static bool
+getModShift ()
+{
+  return bool(fgGetKeyModifiers() & KEYMOD_SHIFT);
+}
+
+static bool
+getModCtrl ()
+{
+  return bool(fgGetKeyModifiers() & KEYMOD_CTRL);
+}
+
+static bool
+getModAlt ()
+{
+  return bool(fgGetKeyModifiers() & KEYMOD_ALT);
+}
+
+static bool
+getModMeta ()
+{
+  return bool(fgGetKeyModifiers() & KEYMOD_META);
+}
+
+static bool
+getModSuper ()
+{
+  return bool(fgGetKeyModifiers() & KEYMOD_SUPER);
+}
 
 
 ////////////////////////////////////////////////////////////////////////
@@ -100,7 +136,8 @@ FGBinding::FGBinding (const SGPropertyNode * node)
 
 FGBinding::~FGBinding ()
 {
-  _arg->getParent()->removeChild(_arg->getName(), _arg->getIndex(), false);
+  if (_arg && _arg->getParent())
+    _arg->getParent()->removeChild(_arg->getName(), _arg->getIndex(), false);
 }
 
 void
@@ -165,7 +202,8 @@ FGBinding::fire (double setting) const
 ////////////////////////////////////////////////////////////////////////
 
 
-FGInput::FGInput ()
+FGInput::FGInput () :
+    _key_event(fgGetNode("/devices/status/keyboard/event", true))
 {
     if (default_input == 0)
         default_input = this;
@@ -180,7 +218,6 @@ FGInput::~FGInput ()
 void
 FGInput::init ()
 {
-  _init_keyboard();
   _init_joystick();
   _init_mouse();
 
@@ -199,14 +236,58 @@ void
 FGInput::postinit ()
 {
   _postinit_joystick();
+  _postinit_keyboard();
+}
+
+void
+FGInput::bind ()
+{
+  fgTie("/devices/status/keyboard/shift", getModShift);
+  fgTie("/devices/status/keyboard/ctrl", getModCtrl);
+  fgTie("/devices/status/keyboard/alt", getModAlt);
+  fgTie("/devices/status/keyboard/meta", getModMeta);
+  fgTie("/devices/status/keyboard/super", getModSuper);
+
+  _key_event->tie("key", SGRawValuePointer<int>(&_key_code));
+  _key_event->tie("pressed", SGRawValuePointer<bool>(&_key_pressed));
+  _key_event->tie("modifier", SGRawValuePointer<int>(&_key_modifiers));
+  _key_event->tie("modifier/shift", SGRawValuePointer<bool>(&_key_shift));
+  _key_event->tie("modifier/ctrl", SGRawValuePointer<bool>(&_key_ctrl));
+  _key_event->tie("modifier/alt", SGRawValuePointer<bool>(&_key_alt));
+  _key_event->tie("modifier/meta", SGRawValuePointer<bool>(&_key_meta));
+  _key_event->tie("modifier/super", SGRawValuePointer<bool>(&_key_super));
+}
+
+void
+FGInput::unbind ()
+{
+  fgUntie("/devices/status/keyboard/shift");
+  fgUntie("/devices/status/keyboard/ctrl");
+  fgUntie("/devices/status/keyboard/alt");
+  fgUntie("/devices/status/keyboard/meta");
+  fgUntie("/devices/status/keyboard/super");
+
+  _key_event->untie("key");
+  _key_event->untie("pressed");
+  _key_event->untie("modifier");
+  _key_event->untie("modifier/shift");
+  _key_event->untie("modifier/ctrl");
+  _key_event->untie("modifier/alt");
+  _key_event->untie("modifier/meta");
+  _key_event->untie("modifier/super");
 }
 
 void 
 FGInput::update (double dt)
-{
+{  
+  timingInfo.clear();
+  stamp("Input_begin");
   _update_keyboard();
+  stamp("Input_keyboard");
   _update_joystick(dt);
+  stamp("Input_JS");
   _update_mouse(dt);
+  stamp("Input_Mouse");
 }
 
 void
@@ -245,6 +326,20 @@ FGInput::doKey (int k, int modifiers, int x, int y)
     return;
   }
 
+  _key_code = k;
+  _key_modifiers = modifiers & ~KEYMOD_RELEASED;
+  _key_pressed = !bool(modifiers & KEYMOD_RELEASED);
+  _key_shift = bool(modifiers & KEYMOD_SHIFT);
+  _key_ctrl = bool(modifiers & KEYMOD_CTRL);
+  _key_alt = bool(modifiers & KEYMOD_ALT);
+  _key_meta = bool(modifiers & KEYMOD_META);
+  _key_super = bool(modifiers & KEYMOD_SUPER);
+  _key_event->fireValueChanged();
+  if (!_key_code)
+    return;
+
+  k = _key_code;
+  modifiers = _key_modifiers | (_key_pressed ? KEYMOD_NONE : KEYMOD_RELEASED);
   button &b = _key_bindings[k];
 
                                 // Key pressed.
@@ -268,23 +363,6 @@ FGInput::doKey (int k, int modifiers, int x, int y)
       for (unsigned int i = 0; i < bindings.size(); i++)
         bindings[i]->fire();
       b.last_state = 0;
-    } else {
-      if (k >= 1 && k <= 26) {
-        if (_key_bindings[k + '@'].last_state)
-          doKey(k + '@', KEYMOD_RELEASED, x, y);
-        if (_key_bindings[k + '`'].last_state)
-          doKey(k + '`', KEYMOD_RELEASED, x, y);
-      } else if (k >= 'A' && k <= 'Z') {
-        if (_key_bindings[k - '@'].last_state)
-          doKey(k - '@', KEYMOD_RELEASED, x, y);
-        if (_key_bindings[tolower(k)].last_state)
-          doKey(tolower(k), KEYMOD_RELEASED, x, y);
-      } else if (k >= 'a' && k <= 'z') {
-        if (_key_bindings[k - '`'].last_state)
-          doKey(k - '`', KEYMOD_RELEASED, x, y);
-        if (_key_bindings[toupper(k)].last_state)
-          doKey(toupper(k), KEYMOD_RELEASED, x, y);
-      }
     }
   }
 }
@@ -318,17 +396,17 @@ FGInput::doMouseClick (int b, int updown, int x, int y)
       // terrain intersection point corresponding to the mouse click
       // and be happy.
       FGScenery* scenery = globals->get_scenery();
-      sgdVec3 start, dir, hit;
-      if (!b && updown == MOUSE_BUTTON_DOWN && FGRenderer::getPickInfo(start, dir, x, y)
+      SGVec3d start, dir, hit;
+      if (!b && updown == MOUSE_BUTTON_DOWN
+          && FGRenderer::getPickInfo(start, dir, x, y)
           && scenery->get_cart_ground_intersection(start, dir, hit)) {
 
-        Point3D geod = sgCartToGeod(Point3D(hit[0], hit[1], hit[2]));
-
+        SGGeod geod = SGGeod::fromCart(hit);
         SGPropertyNode *c = fgGetNode("/sim/input/click", true);
-        c->setDoubleValue("longitude-deg", geod.lon() * SGD_RADIANS_TO_DEGREES);
-        c->setDoubleValue("latitude-deg", geod.lat() * SGD_RADIANS_TO_DEGREES);
-        c->setDoubleValue("elevation-m", geod.elev());
-        c->setDoubleValue("elevation-ft", geod.elev() * SG_METER_TO_FEET);
+        c->setDoubleValue("longitude-deg", geod.getLongitudeDeg());
+        c->setDoubleValue("latitude-deg", geod.getLatitudeDeg());
+        c->setDoubleValue("elevation-m", geod.getElevationM());
+        c->setDoubleValue("elevation-ft", geod.getElevationFt());
 
         fgSetBool("/sim/signals/click", 1);
       }
@@ -402,31 +480,12 @@ FGInput::doMouseMotion (int x, int y)
     if (need_warp)
       fgWarpMouse(x, y);
   }
-  m.x = x;
-  m.y = y;
-}
 
-void
-FGInput::_init_keyboard ()
-{
-  SG_LOG(SG_INPUT, SG_DEBUG, "Initializing key bindings");
-  _module[0] = 0;
-  SGPropertyNode * key_nodes = fgGetNode("/input/keyboard");
-  if (key_nodes == 0) {
-    SG_LOG(SG_INPUT, SG_WARN, "No key bindings (/input/keyboard)!!");
-    key_nodes = fgGetNode("/input/keyboard", true);
-  }
-  
-  vector<SGPropertyNode_ptr> keys = key_nodes->getChildren("key");
-  for (unsigned int i = 0; i < keys.size(); i++) {
-    int index = keys[i]->getIndex();
-    SG_LOG(SG_INPUT, SG_DEBUG, "Binding key " << index);
+  if (m.x != x)
+      fgSetInt("/devices/status/mice/mouse/x", m.x = x);
 
-    _key_bindings[index].bindings->clear();
-    _key_bindings[index].is_repeatable = keys[i]->getBoolValue("repeatable");
-    _key_bindings[index].last_state = 0;
-    _read_bindings(keys[i], _key_bindings[index].bindings, KEYMOD_NONE);
-  }
+  if (m.y != y)
+      fgSetInt("/devices/status/mice/mouse/y", m.y = y);
 }
 
 
@@ -526,6 +585,37 @@ FGInput::_init_joystick ()
 }
 
 
+void
+FGInput::_postinit_keyboard()
+{
+  SG_LOG(SG_INPUT, SG_DEBUG, "Initializing key bindings");
+  _module = "__kbd";
+  SGPropertyNode * key_nodes = fgGetNode("/input/keyboard");
+  if (key_nodes == 0) {
+    SG_LOG(SG_INPUT, SG_WARN, "No key bindings (/input/keyboard)!!");
+    key_nodes = fgGetNode("/input/keyboard", true);
+  }
+
+  FGNasalSys *nasalsys = (FGNasalSys *)globals->get_subsystem("nasal");
+  vector<SGPropertyNode_ptr> nasal = key_nodes->getChildren("nasal");
+  for (unsigned int j = 0; j < nasal.size(); j++) {
+    nasal[j]->setStringValue("module", _module.c_str());
+    nasalsys->handleCommand(nasal[j]);
+  }
+
+  vector<SGPropertyNode_ptr> keys = key_nodes->getChildren("key");
+  for (unsigned int i = 0; i < keys.size(); i++) {
+    int index = keys[i]->getIndex();
+    SG_LOG(SG_INPUT, SG_DEBUG, "Binding key " << index);
+
+    _key_bindings[index].bindings->clear();
+    _key_bindings[index].is_repeatable = keys[i]->getBoolValue("repeatable");
+    _key_bindings[index].last_state = 0;
+    _read_bindings(keys[i], _key_bindings[index].bindings, KEYMOD_NONE);
+  }
+}
+
+
 void
 FGInput::_postinit_joystick()
 {
@@ -571,15 +661,15 @@ FGInput::_postinit_joystick()
     //
     // Initialize nasal groups.
     //
-    string init;
-    init = "this=\"" + string(js_node->getPath()) + "\"";
-    sprintf(_module, "__js%d", i);
-    nasalsys->createModule(_module, _module, init.c_str(), init.size());
+    ostringstream str;
+    str << "__js" << i;
+    _module = str.str();
+    nasalsys->createModule(_module.c_str(), _module.c_str(), "", 0);
 
     vector<SGPropertyNode_ptr> nasal = js_node->getChildren("nasal");
     unsigned int j;
     for (j = 0; j < nasal.size(); j++) {
-      nasal[j]->setStringValue("module", _module);
+      nasal[j]->setStringValue("module", _module.c_str());
       nasalsys->handleCommand(nasal[j]);
     }
 
@@ -690,7 +780,7 @@ void
 FGInput::_init_mouse ()
 {
   SG_LOG(SG_INPUT, SG_DEBUG, "Initializing mouse bindings");
-  _module[0] = 0;
+  _module = "";
 
   SGPropertyNode * mouse_nodes = fgGetNode("/input/mice");
   if (mouse_nodes == 0) {
@@ -789,6 +879,7 @@ FGInput::_update_keyboard ()
 void
 FGInput::_update_joystick (double dt)
 {
+  stamp("update_joystick_01");
   int modifiers = KEYMOD_NONE;  // FIXME: any way to get the real ones?
   int buttons;
   // float js_val, diff;
@@ -802,25 +893,33 @@ FGInput::_update_joystick (double dt)
     jsJoystick * js = _joystick_bindings[i].js;
     if (js == 0 || js->notWorking())
       continue;
-
+    //stamp("update_joystick_02");
     js->read(&buttons, axis_values);
 
                                 // Fire bindings for the axes.
+    //stamp("update_joystick_03");
     for ( j = 0; j < _joystick_bindings[i].naxes; j++) {
+      //stamp("update_joystick_04a");
       axis &a = _joystick_bindings[i].axes[j];
-      
+      //stamp("update_joystick_04aX1");
                                 // Do nothing if the axis position
                                 // is unchanged; only a change in
                                 // position fires the bindings.
       if (fabs(axis_values[j] - a.last_value) > a.tolerance) {
+         //stamp("update_joystick_04aX2");
 //      SG_LOG(SG_INPUT, SG_DEBUG, "Axis " << j << " has moved");
         a.last_value = axis_values[j];
+	//stamp("update_joystick_04aX3");
 //      SG_LOG(SG_INPUT, SG_DEBUG, "There are "
 //             << a.bindings[modifiers].size() << " bindings");
-        for (unsigned int k = 0; k < a.bindings[modifiers].size(); k++)
+        for (unsigned int k = 0; k < a.bindings[modifiers].size(); k++) {
+          std::ostringstream desc;
+          stamp("update_joystick_fire_prep ");
           a.bindings[modifiers][k]->fire(axis_values[j]);
+          stamp("update_joystick_fire_end");
+        }
+        stamp("update_joystick_04");
       }
-     
                                 // do we have to emulate axis buttons?
       a.last_dt += dt;
       if(a.last_dt >= a.interval_sec) {
@@ -837,7 +936,9 @@ FGInput::_update_joystick (double dt)
                          -1, -1);
          a.last_dt -= a.interval_sec;
       }
+      //stamp("update_joystick_04");
     }
+    stamp("update_joystick_05");
 
                                 // Fire bindings for the buttons.
     for (j = 0; j < _joystick_bindings[i].nbuttons; j++) {
@@ -851,7 +952,9 @@ FGInput::_update_joystick (double dt)
         b.last_dt -= b.interval_sec;
       }
     }
+    //stamp("update_joystick_06");
   }
+  stamp("update_joystick_07");
 }
 
 void
@@ -924,8 +1027,8 @@ FGInput::_read_bindings (const SGPropertyNode * node,
     const char *cmd = bindings[i]->getStringValue("command");
     SG_LOG(SG_INPUT, SG_DEBUG, "Reading binding " << cmd);
 
-    if (!strcmp(cmd, "nasal") && _module[0])
-      bindings[i]->setStringValue("module", _module);
+    if (!strcmp(cmd, "nasal") && !_module.empty())
+      bindings[i]->setStringValue("module", _module.c_str());
     binding_list[modifiers].push_back(new FGBinding(bindings[i]));
   }
 
@@ -945,6 +1048,14 @@ FGInput::_read_bindings (const SGPropertyNode * node,
   if (node->getChild("mod-alt") != 0)
     _read_bindings(node->getChild("mod-alt"), binding_list,
                    modifiers|KEYMOD_ALT);
+
+  if (node->getChild("mod-meta") != 0)
+    _read_bindings(node->getChild("mod-meta"), binding_list,
+                   modifiers|KEYMOD_META);
+
+  if (node->getChild("mod-super") != 0)
+    _read_bindings(node->getChild("mod-super"), binding_list,
+                   modifiers|KEYMOD_SUPER);
 }
 
 
