@@ -15,31 +15,9 @@
 
 HeadingIndicator::HeadingIndicator ( SGPropertyNode *node )
     :
-    name("heading-indicator"),
-    num(0),
-    vacuum_system("/systems/vacuum")
-{
-    int i;
-    for ( i = 0; i < node->nChildren(); ++i ) {
-        SGPropertyNode *child = node->getChild(i);
-        string cname = child->getName();
-        string cval = child->getStringValue();
-        if ( cname == "name" ) {
-            name = cval;
-        } else if ( cname == "number" ) {
-            num = child->getIntValue();
-        } else if ( cname == "vacuum-system" ) {
-            vacuum_system = cval;
-        } else {
-            SG_LOG( SG_INSTR, SG_WARN, "Error in heading-indicator config logic" );
-            if ( name.length() ) {
-                SG_LOG( SG_INSTR, SG_WARN, "Section = " << name );
-            }
-        }
-    }
-}
-
-HeadingIndicator::HeadingIndicator ()
+    _name(node->getStringValue("name", "heading-indicator")),
+    _num(node->getIntValue("number", 0)),
+    _suction(node->getStringValue("suction", "/systems/vacuum/suction-inhg"))
 {
 }
 
@@ -51,13 +29,12 @@ void
 HeadingIndicator::init ()
 {
     string branch;
-    branch = "/instrumentation/" + name;
-    vacuum_system += "/suction-inhg";
+    branch = "/instrumentation/" + _name;
 
-    SGPropertyNode *node = fgGetNode(branch.c_str(), num, true );
+    SGPropertyNode *node = fgGetNode(branch.c_str(), _num, true );
     _offset_node = node->getChild("offset-deg", 0, true);
     _heading_in_node = fgGetNode("/orientation/heading-deg", true);
-    _suction_node = fgGetNode(vacuum_system.c_str(), true);
+    _suction_node = fgGetNode(_suction.c_str(), true);
     _heading_out_node = node->getChild("indicated-heading-deg", 0, true);
     _last_heading_deg = (_heading_in_node->getDoubleValue() +
                          _offset_node->getDoubleValue());
@@ -68,8 +45,8 @@ HeadingIndicator::bind ()
 {
     std::ostringstream temp;
     string branch;
-    temp << num;
-    branch = "/instrumentation/" + name + "[" + temp.str() + "]";
+    temp << _num;
+    branch = "/instrumentation/" + _name + "[" + temp.str() + "]";
 
     fgTie((branch + "/serviceable").c_str(),
           &_gyro, &Gyro::is_serviceable, &Gyro::set_serviceable);
@@ -82,8 +59,8 @@ HeadingIndicator::unbind ()
 {
     std::ostringstream temp;
     string branch;
-    temp << num;
-    branch = "/instrumentation/" + name + "[" + temp.str() + "]";
+    temp << _num;
+    branch = "/instrumentation/" + _name + "[" + temp.str() + "]";
 
     fgUntie((branch + "/serviceable").c_str());
     fgUntie((branch + "/spin").c_str());
@@ -92,10 +69,15 @@ HeadingIndicator::unbind ()
 void
 HeadingIndicator::update (double dt)
 {
+    timingInfo.clear();
+    stamp("begin");
                                 // Get the spin from the gyro
     _gyro.set_power_norm(_suction_node->getDoubleValue()/5.0);
+    //stamp("HDG_01");
     _gyro.update(dt);
+    //stamp("HDG_02");
     double spin = _gyro.get_spin_norm();
+    //stamp("HDG_03");
 
                                 // Next, calculate time-based precession
     double offset = _offset_node->getDoubleValue();
@@ -120,8 +102,9 @@ HeadingIndicator::update (double dt)
         _last_heading_deg += 360;
     while ((heading - _last_heading_deg) < -180)
         _last_heading_deg -= 360;
-
+    //stamp("HDG_04");
     heading = fgGetLowPass(_last_heading_deg, heading, dt/factor);
+    //stamp("HDG_05");
     _last_heading_deg = heading;
 
     heading += offset;
@@ -129,8 +112,9 @@ HeadingIndicator::update (double dt)
         heading += 360;
     while (heading > 360)
         heading -= 360;
-
+    stamp("HDG_06");
     _heading_out_node->setDoubleValue(heading);
+    stamp("HDG_07");
 }
 
 // end of heading_indicator.cxx
