@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: bootstrap.cxx,v 1.22 2006-03-28 21:18:32 curt Exp $
+// $Id: bootstrap.cxx,v 1.23.2.2 2007-05-13 08:43:42 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -117,6 +117,7 @@ int main ( int argc, char **argv );
 int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                              LPSTR lpCmdLine, int nCmdShow) {
 
+  logbuf::has_no_console();
   main( __argc, __argv );
 }
 #endif
@@ -142,6 +143,16 @@ void flush_fpe(void)
 }
 #endif
 
+static void fg_terminate() {
+    cerr << endl <<
+            "Uncaught Exception: you should see a meaningful error message\n"
+            "here, but your GLUT (or SDL) library was apparently compiled\n"
+            "and/or linked without exception support. Please complain to\n"
+            "its provider!"
+            << endl << endl;
+    abort();
+}
+
 int _bootstrap_OSInit;
 
 // Main entry point; catch any exceptions that have made it this far.
@@ -199,6 +210,7 @@ int main ( int argc, char **argv ) {
     // FIXME: add other, more specific
     // exceptions.
     try {
+        std::set_terminate(fg_terminate);
         atexit(fgExitCleanup);
         fgMainInit(argc, argv);
     } catch (const sg_throwable &t) {
@@ -212,6 +224,9 @@ int main ( int argc, char **argv ) {
     } catch (const string &s) {
         cerr << "Fatal error: " << s << endl;
 
+    } catch (const char *s) {
+        cerr << "Fatal error: " << s << endl;
+
     } catch (...) {
         cerr << "Unknown exception in the main loop. Aborting..." << endl;
         if (errno)
