@@ -14,6 +14,7 @@
 #include <simgear/sg_inlines.h>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/math/sg_random.h>
+#include <simgear/scene/material/mat.hxx>
 #include <simgear/structure/exception.hxx>
 #include <simgear/structure/commands.hxx>
 #include <simgear/props/props.hxx>
@@ -28,6 +29,7 @@
 #include <GUI/dialog.hxx>
 #include <Aircraft/replay.hxx>
 #include <Scenery/tilemgr.hxx>
+#include <Scenery/scenery.hxx>
 #if defined(HAVE_PLIB_PSL)
 #  include <Scripting/scriptmgr.hxx>
 #endif
@@ -211,14 +213,14 @@ do_exit (const SGPropertyNode * arg)
 #endif
             config.append( "autosave.xml" );
             config.create_dir( 0700 );
-            SG_LOG(SG_IO, SG_INFO, "Saving user settings to autosave.xml");
+            SG_LOG(SG_IO, SG_INFO, "Saving user settings to " << config.str());
             try {
                 writeProperties(config.str(), globals->get_props(), false, SGPropertyNode::USERARCHIVE);
             } catch (const sg_exception &e) {
                 guiErrorMessage("Error writing autosave.xml: ", e);
             }
 
-            SG_LOG(SG_INPUT, SG_BULK, "Finished Saving user settings");
+            SG_LOG(SG_INPUT, SG_DEBUG, "Finished Saving user settings");
         }
     }
     fgExit(arg->getIntValue("status", 0));
@@ -331,7 +333,7 @@ do_load (const SGPropertyNode * arg)
     SG_LOG(SG_INPUT, SG_INFO, "Restored flight from " << file);
     return true;
   } else {
-    SG_LOG(SG_INPUT, SG_ALERT, "Cannot load flight from " << file);
+    SG_LOG(SG_INPUT, SG_WARN, "Cannot load flight from " << file);
     return false;
   }
 }
@@ -472,7 +474,6 @@ do_view_cycle (const SGPropertyNode * arg)
   globals->get_viewmgr()->next_view();
   fix_hud_visibility();
   globals->get_tile_mgr()->refresh_view_timestamps();
-//   fgReshape(fgGetInt("/sim/startup/xsize"), fgGetInt("/sim/startup/ysize"));
   return true;
 }
 
@@ -511,7 +512,6 @@ do_tile_cache_reload (const SGPropertyNode * arg)
     if ( !freeze ) {
 	fgSetBool("/sim/freeze/master", true);
     }
-    // BusyCursor(0);
     if ( globals->get_tile_mgr()->init() ) {
 	// Load the local scenery data
         double visibility_meters = fgGetDouble("/environment/visibility-m");
@@ -521,7 +521,6 @@ do_tile_cache_reload (const SGPropertyNode * arg)
 		"Error in Tile Manager initialization!" );
 	exit(-1);
     }
-    // BusyCursor(1);
     if ( !freeze ) {
 	fgSetBool("/sim/freeze/master", false);
     }
@@ -1097,13 +1096,14 @@ static bool
 do_dialog_update (const SGPropertyNode * arg)
 {
     NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
-    FGDialog * dialog = gui->getActiveDialog();
+    FGDialog * dialog;
+    if (arg->hasValue("dialog-name"))
+        dialog = gui->getDialog(arg->getStringValue("dialog-name"));
+    else
+        dialog = gui->getActiveDialog();
+
     if (dialog != 0) {
-        if (arg->hasValue("object-name")) {
-            dialog->updateValue(arg->getStringValue("object-name"));
-        } else {
-            dialog->updateValues();
-        }
+        dialog->updateValues(arg->getStringValue("object-name"));
         return true;
     } else {
         return false;
@@ -1120,14 +1120,14 @@ static bool
 do_dialog_apply (const SGPropertyNode * arg)
 {
     NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
-    FGDialog * dialog = gui->getActiveDialog();
+    FGDialog * dialog;
+    if (arg->hasValue("dialog-name"))
+        dialog = gui->getDialog(arg->getStringValue("dialog-name"));
+    else
+        dialog = gui->getActiveDialog();
+
     if (dialog != 0) {
-        if (arg->hasValue("object-name")) {
-            const char * name = arg->getStringValue("object-name");
-            dialog->applyValue(name);
-        } else {
-            dialog->applyValues();
-        }
+        dialog->applyValues(arg->getStringValue("object-name"));
         return true;
     } else {
         return false;
@@ -1154,15 +1154,22 @@ do_gui_redraw (const SGPropertyNode * arg)
  * overlap.
  */
 static bool
-do_play_audio_message (const SGPropertyNode * arg)
+do_play_audio_sample (const SGPropertyNode * arg)
 {
     FGFX *fx = (FGFX *)globals->get_subsystem("fx");
     string path = arg->getStringValue("path");
     string file = arg->getStringValue("file");
+    double volume = arg->getDoubleValue("volume");
     // cout << "playing " << path << " / " << file << endl;
-    fx->play_message( path, file );
+    try {
+        fx->play_message( path, file, volume );
+        return true;
 
-    return true;
+    } catch (const sg_io_exception& e) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "play-audio-sample: "
+                "failed to load" << path << '/' << file);
+        return false;
+    }
 }
 
 /**
@@ -1175,11 +1182,10 @@ do_presets_commit (const SGPropertyNode * arg)
     // don't get lost when we subsequently delete this fdm
     // and create a new one.
     cur_fdm_state->unbind();
-        
+
     // set position from presets
     fgInitPosition();
 
-    // BusyCursor(0);
     fgReInitSubsystems();
 
     globals->get_tile_mgr()->update( fgGetDouble("/environment/visibility-m") );
@@ -1231,8 +1237,6 @@ do_replay (const SGPropertyNode * arg)
     return true;
 }
 
-
-
 static bool
 do_decrease_visibility (const SGPropertyNode * arg)
 {
@@ -1255,20 +1259,6 @@ do_increase_visibility (const SGPropertyNode * arg)
     return true;
 }
 
-static bool
-do_hud_brightkey(const SGPropertyNode *)
-{
-    HUD_brightkey( true );
-    return true;
-}
-
-static bool
-do_hud_masterswitch(const SGPropertyNode *)
-{
-    HUD_masterswitch( true );
-    return true;
-}
-
 static bool
 do_hud_init(const SGPropertyNode *)
 {
@@ -1283,30 +1273,39 @@ do_hud_init2(const SGPropertyNode *)
     return true;
 }
 
+
 /**
  * An fgcommand to allow loading of xml files via nasal,
  * the xml file's structure will be made available within
- * a (definable) property tree node
+ * a property tree node defined under argument "targetnode",
+ * or in the given argument tree under "data" otherwise.
  *
- * @param filename a string to hold the complete path & filename of a XML file
+ * @param filename a string to hold the complete path & filename of an XML file
  * @param targetnode a string pointing to a location within the property tree
- * where to store the parsed XML file
+ * where to store the parsed XML file. If <targetnode> is undefined, then the
+ * file contents are stored under a node <data> in the argument tree.
  */
 
-static bool 
-do_load_xml_to_proptree(const SGPropertyNode * node)
+static bool
+do_load_xml_to_proptree(const SGPropertyNode * arg)
 {
-    // SG_LOG(SG_GENERAL, SG_ALERT, "fgcommand loadxml executed");
+    SGPath file(arg->getStringValue("filename"));
+    if (file.str().empty())
+        return false;
+
+    if (file.extension() != "xml")
+        file.concat(".xml");
+
+    SGPropertyNode *targetnode;
+    if (arg->hasValue("targetnode"))
+        targetnode = fgGetNode(arg->getStringValue("targetnode"), true);
+    else
+        targetnode = const_cast<SGPropertyNode *>(arg)->getNode("data", true);
 
-    SGPropertyNode * targetnode;
-    targetnode = fgGetNode(node->getNode("targetnode")->getStringValue(),true);
-		
-    const char *filename = node->getNode("filename")->getStringValue();
     try {
-        fgLoadProps(filename, targetnode);
+        readProperties(file.c_str(), targetnode, true);
     } catch (const sg_exception &e) {
-        string errmsg = "Error reading file " + string(filename) + ":\n";
-        guiErrorMessage(errmsg.c_str(), e);
+        SG_LOG(SG_IO, SG_WARN, "loadxml: " << e.getFormattedMessage());
         return false;
     }
 
@@ -1315,7 +1314,7 @@ do_load_xml_to_proptree(const SGPropertyNode * node)
 
 
 /**
- * an fgcommand to allow saving of xml files via nasal,
+ * An fgcommand to allow saving of xml files via nasal,
  * the file's structure will be determined based on what's
  * encountered in the passed (source) property tree node
  *
@@ -1323,28 +1322,32 @@ do_load_xml_to_proptree(const SGPropertyNode * node)
  * XML file
  * @param sourcenode a string pointing to a location within the property tree
  * where to find the nodes that should be written recursively into an XML file
- *
- * TODO: 
- *   deal with already existing filenames, optionally return error/success
- *   values in a separate node to provide status information
- *
- * note: it's directly using writeProperties, which is not necessarily
- *       preferable, but for now it should work ...
+ * @param data if no sourcenode is given, then the file contents are taken from
+ * the argument tree's "data" node.
  */
 
-static bool 
-do_save_xml_from_proptree(const SGPropertyNode * node)
+static bool
+do_save_xml_from_proptree(const SGPropertyNode * arg)
 {
-    //TODO: do Parameter validation !
-    SGPropertyNode * sourcenode;
-    sourcenode = fgGetNode(node->getNode("sourcenode")->getStringValue(),true);
+    SGPath file(arg->getStringValue("filename"));
+    if (file.str().empty())
+        return false;
+
+    if (file.extension() != "xml")
+        file.concat(".xml");
+
+    SGPropertyNode *sourcenode;
+    if (arg->hasValue("sourcenode"))
+        sourcenode = fgGetNode(arg->getStringValue("sourcenode"), true);
+    else if (arg->getNode("data", false))
+        sourcenode = const_cast<SGPropertyNode *>(arg)->getNode("data");
+    else
+        return false;
 
-    const char *filename = node->getNode("filename")->getStringValue();
     try {
-        writeProperties (filename, sourcenode, true);
+        writeProperties (file.c_str(), sourcenode, true);
     } catch (const sg_exception &e) {
-        string errmsg = "Error writing file " + string(filename) + ":\n";
-        guiErrorMessage(errmsg.c_str(), e);
+        SG_LOG(SG_IO, SG_WARN, "savexml: " << e.getFormattedMessage());
         return false;
     }
 
@@ -1448,18 +1451,16 @@ static struct {
     { "dialog-update", do_dialog_update },
     { "dialog-apply", do_dialog_apply },
     { "gui-redraw", do_gui_redraw },
-    { "play-audio-message", do_play_audio_message },
+    { "play-audio-sample", do_play_audio_sample },
     { "presets-commit", do_presets_commit },
     { "log-level", do_log_level },
     { "replay", do_replay },
     { "decrease-visibility", do_decrease_visibility },
     { "increase-visibility", do_increase_visibility },
-    { "hud-brightkey", do_hud_brightkey },
-    { "hud-masterswitch", do_hud_masterswitch },
     { "hud-init", do_hud_init },
     { "hud-init2", do_hud_init2 },
     { "loadxml", do_load_xml_to_proptree},
-    { "savexml", do_save_xml_from_proptree },    
+    { "savexml", do_save_xml_from_proptree },
     { "press-cockpit-button", do_press_cockpit_button },
     { "release-cockpit-button", do_release_cockpit_button },
     { 0, 0 }			// zero-terminated
