@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: fg_init.cxx,v 1.161 2006-04-02 08:42:36 fredb Exp $
+// $Id: fg_init.cxx,v 1.172.2.16 2007-11-03 21:01:48 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -37,6 +37,10 @@
 #if defined( unix ) || defined( __CYGWIN__ )
 #  include <unistd.h>           // for gethostname()
 #endif
+#if defined( _MSC_VER) || defined(__MINGW32__)
+#  include <direct.h>           // for getcwd()
+#  define getcwd _getcwd
+#endif
 
 // work around a stdc++ lib bug in some versions of linux, but doesn't
 // seem to hurt to have this here for all versions of Linux.
@@ -70,13 +74,12 @@
 #include <ATC/ATCdisplay.hxx>
 #include <ATC/ATCmgr.hxx>
 #include <ATC/AIMgr.hxx>
-#include <Autopilot/auto_gui.hxx>
 #include <Autopilot/route_mgr.hxx>
 #include <Autopilot/xmlauto.hxx>
 #include <Cockpit/cockpit.hxx>
 #include <Cockpit/panel.hxx>
 #include <Cockpit/panel_io.hxx>
-#ifdef ENABLE_SP_FMDS
+#ifdef ENABLE_SP_FDM
 #include <FDM/SP/ADA.hxx>
 #include <FDM/SP/ACMS.hxx>
 #endif
@@ -318,10 +321,18 @@ bool fgInitFGAircraft ( int argc, char **argv ) {
     // First parse command line options looking for --aircraft=, this
     // will override anything specified in a config file
     aircraft = fgScanForOption( "--aircraft=", argc, argv );
+    if ( aircraft.empty() ) {
+        // check synonym option
+        aircraft = fgScanForOption( "--vehicle=", argc, argv );
+    }
 
     // Check in one of the user configuration files.
-    if ( aircraft.empty() )
+    if ( aircraft.empty() ) {
         aircraft = fgScanForOption( "--aircraft=" );
+    }
+    if ( aircraft.empty() ) {
+        aircraft = fgScanForOption( "--vehicle=" );
+    }
 
     // if an aircraft was specified, set the property name
     if ( !aircraft.empty() ) {
@@ -405,13 +416,12 @@ SGPropertyNode *fgInitLocale(const char *language) {
 
    const char *d_path_str = d_node->getStringValue("strings");
    if (!d_path_str) {
-      SG_LOG(SG_GENERAL, SG_ALERT, "Incorrect path in configuration file.");
+      SG_LOG(SG_GENERAL, SG_ALERT, "No path in " << d_node->getPath() << "/strings.");
       return NULL;
    }
 
    d_path.append(d_path_str);
-   SG_LOG(SG_GENERAL, SG_INFO, "Reading localized strings from "
-                                  << d_path.str());
+   SG_LOG(SG_GENERAL, SG_INFO, "Reading localized strings from " << d_path.str());
 
    SGPropertyNode *strings = c_node->getNode("strings");
    try {
@@ -429,18 +439,18 @@ SGPropertyNode *fgInitLocale(const char *language) {
 
       const char *c_path_str = c_node->getStringValue("strings");
       if (!c_path_str) {
-         SG_LOG(SG_GENERAL, SG_ALERT, "Incorrect path in configuration file.");
+         SG_LOG(SG_GENERAL, SG_ALERT, "No path in " << c_node->getPath() << "/strings");
          return NULL;
       }
 
       c_path.append(c_path_str);
-      SG_LOG(SG_GENERAL, SG_INFO, "Reading localized strings from "
-                                     << c_path.str());
+      SG_LOG(SG_GENERAL, SG_INFO, "Reading localized strings from " << c_path.str());
 
       try {
          readProperties(c_path.str(), strings);
       } catch (const sg_exception &e) {
-         SG_LOG(SG_GENERAL, SG_ALERT, "Unable to read the localized strings");
+         SG_LOG(SG_GENERAL, SG_ALERT,
+                 "Unable to read the localized strings from " << c_path.str());
          return NULL;
       }
    }
@@ -508,7 +518,11 @@ do_options (int argc, char ** argv)
 }
 
 
-static string fgFindAircraftPath( const SGPath &path, const string &aircraft ) {
+static string fgFindAircraftPath( const SGPath &path, const string &aircraft,
+                                  SGPropertyNode *cache, int depth = 0 )
+{
+    const int MAXDEPTH = 1;
+
     ulDirEnt* dire;
     ulDir *dirp = ulOpenDir(path.str().c_str());
     if (dirp == NULL) {
@@ -519,20 +533,45 @@ static string fgFindAircraftPath( const SGPath &path, const string &aircraft ) {
     string result;
     while ((dire = ulReadDir(dirp)) != NULL) {
         if (dire->d_isdir) {
-            if ( strcmp("CVS", dire->d_name) && strcmp(".", dire->d_name)
-                 && strcmp("..", dire->d_name) && strcmp("AI", dire->d_name))
-            {
-                SGPath next = path;
-                next.append(dire->d_name);
+            if (depth > MAXDEPTH)
+                continue;
 
-                result = fgFindAircraftPath( next, aircraft );
-                if ( ! result.empty() ) {
+            if (!strcmp("CVS", dire->d_name) || !strcmp(".", dire->d_name)
+                    || !strcmp("..", dire->d_name) || !strcmp("AI", dire->d_name))
+                continue;
+
+            SGPath next = path;
+            next.append(dire->d_name);
+
+            result = fgFindAircraftPath( next, aircraft, cache, depth + 1 );
+            if ( ! result.empty() )
+                break;
+
+        } else {
+            int len = strlen(dire->d_name);
+            if (len < 9 || strcmp(dire->d_name + len - 8, "-set.xml"))
+                continue;
+
+            // create cache node
+            int i = 0;
+            while (1)
+                if (!cache->getChild("aircraft", i++, false))
                     break;
-                }
+
+            SGPropertyNode *n, *entry = cache->getChild("aircraft", --i, true);
+
+            n = entry->getNode("file", true);
+            n->setStringValue(dire->d_name);
+            n->setAttribute(SGPropertyNode::USERARCHIVE, true);
+
+            n = entry->getNode("path", true);
+            n->setStringValue(path.str().c_str());
+            n->setAttribute(SGPropertyNode::USERARCHIVE, true);
+
+            if ( !strcmp(dire->d_name, aircraft.c_str()) ) {
+                result = path.str();
+                break;
             }
-        } else if ( !strcmp(dire->d_name, aircraft.c_str()) ) {
-            result = path.str();
-            break;
         }
     }
 
@@ -557,6 +596,30 @@ bool fgInitConfig ( int argc, char **argv ) {
         return false;
     }
 
+    SGPropertyNode autosave;
+#ifdef _MSC_VER
+    char *envp = ::getenv( "APPDATA" );
+    if (envp != NULL ) {
+        SGPath config( envp );
+        config.append( "flightgear.org" );
+#else
+    if ( homedir != NULL ) {
+        SGPath config( homedir );
+        config.append( ".fgfs" );
+#endif
+        fgSetString("/sim/fg-home", config.c_str());
+        config.append( "autosave.xml" );
+        SG_LOG(SG_INPUT, SG_INFO, "Reading user settings from " << config.str());
+        try {
+            readProperties(config.str(), &autosave, SGPropertyNode::USERARCHIVE);
+        } catch (...) {
+            SG_LOG(SG_INPUT, SG_DEBUG, "First time reading user settings");
+        }
+        SG_LOG(SG_INPUT, SG_DEBUG, "Finished Reading user settings");
+    }
+    SGPropertyNode *cache_root = autosave.getNode("sim/startup/path-cache", true);
+
+
     // Scan user config files and command line for a specified aircraft.
     fgInitFGAircraft(argc, argv);
 
@@ -566,8 +629,34 @@ bool fgInitConfig ( int argc, char **argv ) {
         aircraft_search.append( "Aircraft" );
 
         string aircraft_set = aircraft + "-set.xml";
+        string result;
+
+        // check if the *-set.xml file is already in the cache
+        if (globals->get_fg_root() == cache_root->getStringValue("fg-root", "")) {
+            vector<SGPropertyNode_ptr> cache = cache_root->getChildren("aircraft");
+            for (unsigned int i = 0; i < cache.size(); i++) {
+                const char *name = cache[i]->getStringValue("file", "");
+                if (aircraft_set == name) {
+                    const char *path = cache[i]->getStringValue("path", "");
+                    SGPath xml(path);
+                    xml.append(name);
+                    if (xml.exists())
+                        result = path;
+                    break;
+                }
+            }
+        }
+
+        if (result.empty()) {
+            // prepare cache for rescan
+            SGPropertyNode *n = cache_root->getNode("fg-root", true);
+            n->setStringValue(globals->get_fg_root().c_str());
+            n->setAttribute(SGPropertyNode::USERARCHIVE, true);
+            cache_root->removeChildren("aircraft");
+
+            result = fgFindAircraftPath( aircraft_search, aircraft_set, cache_root );
+        }
 
-        string result = fgFindAircraftPath( aircraft_search, aircraft_set );
         if ( !result.empty() ) {
             fgSetString( "/sim/aircraft-dir", result.c_str() );
             SGPath full_name( result );
@@ -593,27 +682,7 @@ bool fgInitConfig ( int argc, char **argv ) {
         SG_LOG( SG_INPUT, SG_ALERT, "No default aircraft specified" );
     }
 
-#ifdef _MSC_VER
-    char *envp = ::getenv( "APPDATA" );
-    if (envp != NULL ) {
-        SGPath config( envp );
-        config.append( "flightgear.org" );
-#else
-    if ( homedir != NULL ) {
-        SGPath config( homedir );
-        config.append( ".fgfs" );
-#endif
-        fgSetString("/sim/fg-home", config.c_str());
-        config.append( "autosave.xml" );
-        SG_LOG(SG_INPUT, SG_INFO, "Reading user settings from autosave.xml");
-        try {
-            fgLoadProps(config.str().c_str(), globals->get_props(), false,
-                        SGPropertyNode::USERARCHIVE);
-        } catch (...) {
-            SG_LOG(SG_INPUT, SG_BULK, "First time reading user settings");
-        }
-        SG_LOG(SG_INPUT, SG_BULK, "Finished Reading user settings");
-    }
+    copyProperties(&autosave, globals->get_props());
 
     // parse options after loading aircraft to ensure any user
     // overrides of defaults are honored.
@@ -691,30 +760,29 @@ struct FGTowerLocationListener : SGPropertyChangeListener {
     }
 };
 
-static void fgInitTowerLocationListener() {
+void fgInitTowerLocationListener() {
     fgGetNode("/sim/tower/airport-id",  true)
-	->addChangeListener( new FGTowerLocationListener() );
+        ->addChangeListener( new FGTowerLocationListener(), true );
 }
 
 // Set current_options lon/lat given an airport id and heading (degrees)
 static bool fgSetPosFromAirportIDandHdg( const string& id, double tgt_hdg ) {
     FGRunway r;
 
-    if ( id.length() ) {
-        // set initial position from runway and heading
+    if ( id.empty() )
+        return false;
 
-        SG_LOG( SG_GENERAL, SG_INFO,
-                "Attempting to set starting position from airport code "
-                << id << " heading " << tgt_hdg );
-		
-        if ( ! globals->get_runways()->search( id, (int)tgt_hdg, &r ) ) {
-            SG_LOG( SG_GENERAL, SG_ALERT,
-                    "Failed to find a good runway for " << id << '\n' );
-            return false;
-        }	
-    } else {
+    // set initial position from runway and heading
+    SG_LOG( SG_GENERAL, SG_INFO,
+            "Attempting to set starting position from airport code "
+            << id << " heading " << tgt_hdg );
+
+    if ( ! globals->get_runways()->search( id, (int)tgt_hdg, &r ) ) {
+        SG_LOG( SG_GENERAL, SG_ALERT,
+                "Failed to find a good runway for " << id << '\n' );
         return false;
     }
+    fgSetString("/sim/atc/runway", r._rwy_no.c_str());
 
     double lat2, lon2, az2;
     double heading = r._heading;
@@ -766,25 +834,24 @@ static bool fgSetPosFromAirportIDandHdg( const string& id, double tgt_hdg ) {
 
 
 // Set current_options lon/lat given an airport id and runway number
-static bool fgSetPosFromAirportIDandRwy( const string& id, const string& rwy ) {
+static bool fgSetPosFromAirportIDandRwy( const string& id, const string& rwy, bool rwy_req ) {
     FGRunway r;
 
-    if ( id.length() ) {
-        // set initial position from airport and runway number
+    if ( id.empty() )
+        return false;
 
-        SG_LOG( SG_GENERAL, SG_INFO,
-                "Attempting to set starting position for "
-                << id << ":" << rwy );
+    // set initial position from airport and runway number
+    SG_LOG( SG_GENERAL, SG_INFO,
+            "Attempting to set starting position for "
+            << id << ":" << rwy );
 
-        if ( ! globals->get_runways()->search( id, rwy, &r ) ) {
-            SG_LOG( SG_GENERAL, SG_ALERT,
-                    "Failed to find runway " << rwy << 
-                    " at airport " << id );
-            return false;
-        }
-    } else {
+    if ( ! globals->get_runways()->search( id, rwy, &r ) ) {
+        SG_LOG( SG_GENERAL, rwy_req ? SG_ALERT : SG_INFO,
+                "Failed to find runway " << rwy <<
+                " at airport " << id << ". Using default runway." );
         return false;
     }
+    fgSetString("/sim/atc/runway", r._rwy_no.c_str());
 
     double lat2, lon2, az2;
     double heading = r._heading;
@@ -1021,29 +1088,6 @@ static bool fgSetPosFromFix( const string& id ) {
         return false;
     }
 }
- 
-static void parseWaypoints() {
-    string_list *waypoints = globals->get_initial_waypoints();
-    if (waypoints) {
-	vector<string>::iterator i;
-	for (i = waypoints->begin(); 
-	     i  != waypoints->end();
-	     i++)
-        {
-            NewWaypoint(*i);
-        }
-	// Now were done using the way points we can deallocate the
-	// memory they used
-	while (waypoints->begin() != waypoints->end()) {
-            waypoints->pop_back();
-        }
-	delete waypoints;
-	globals->set_initial_waypoints(0);
-    }
-}
-
- 
- 
 
 
 /**
@@ -1109,6 +1153,16 @@ fgInitNav ()
     fixlist->init( p_fix );
     globals->set_fixlist( fixlist );
 
+    SG_LOG(SG_GENERAL, SG_INFO, "  Airways");
+    SGPath p_awy( globals->get_fg_root() );
+    p_awy.append( "Navaids/awy.dat" );
+    FGAirwayNetwork *awyNet = new FGAirwayNetwork;
+    //cerr << "Loading Airways" << endl;
+    awyNet->load (p_awy );
+    awyNet->init();
+    //cerr << "initializing airways" << endl;
+    globals->set_airwaynet( awyNet );
+
     return true;
 }
 
@@ -1147,7 +1201,7 @@ bool fgInitPosition() {
 
     string apt = fgGetString("/sim/presets/airport-id");
     string rwy_no = fgGetString("/sim/presets/runway");
-    double hdg = fgGetDouble("/sim/presets/heading-deg");
+    bool rwy_req = fgGetBool("/sim/presets/runway-requested");
     string vor = fgGetString("/sim/presets/vor-id");
     double vor_freq = fgGetDouble("/sim/presets/vor-freq");
     string ndb = fgGetString("/sim/presets/ndb-id");
@@ -1155,13 +1209,29 @@ bool fgInitPosition() {
     string carrier = fgGetString("/sim/presets/carrier");
     string parkpos = fgGetString("/sim/presets/parkpos");
     string fix = fgGetString("/sim/presets/fix");
+    SGPropertyNode *hdg_preset = fgGetNode("/sim/presets/heading-deg", true);
+    double hdg = hdg_preset->getDoubleValue();
 
-    fgSetDouble( "/orientation/heading-deg", hdg );
-    fgInitTowerLocationListener();
+    // save some start parameters, so that we can later say what the
+    // user really requested. TODO generalize that and move it to options.cxx
+    static bool start_options_saved = false;
+    if (!start_options_saved) {
+        start_options_saved = true;
+        SGPropertyNode *opt = fgGetNode("/sim/startup/options", true);
+
+        opt->setDoubleValue("latitude-deg", lat_deg);
+        opt->setDoubleValue("longitude-deg", lon_deg);
+        opt->setDoubleValue("heading-deg", hdg);
+        opt->setStringValue("airport", apt.c_str());
+        opt->setStringValue("runway", rwy_no.c_str());
+    }
+
+    if (hdg > 9990.0)
+        hdg = fgGetDouble("/environment/config/boundary/entry/wind-from-heading-deg", 270);
 
     if ( !set_pos && !apt.empty() && !rwy_no.empty() ) {
         // An airport + runway is requested
-        if ( fgSetPosFromAirportIDandRwy( apt, rwy_no ) ) {
+        if ( fgSetPosFromAirportIDandRwy( apt, rwy_no, rwy_req ) ) {
             // set tower position (a little off the heading for single
             // runway airports)
 	    fgSetString("/sim/tower/airport-id",  apt.c_str());
@@ -1179,6 +1249,9 @@ bool fgInitPosition() {
         }
     }
 
+    if (hdg_preset->getDoubleValue() > 9990.0)
+        hdg_preset->setDoubleValue(hdg);
+
     if ( !set_pos && !vor.empty() ) {
         // a VOR is requested
         if ( fgSetPosFromNAV( vor, vor_freq ) ) {
@@ -1218,13 +1291,14 @@ bool fgInitPosition() {
                  fgGetDouble("/sim/presets/longitude-deg") );
     fgSetDouble( "/position/latitude-deg",
                  fgGetDouble("/sim/presets/latitude-deg") );
+    fgSetDouble( "/orientation/heading-deg", hdg_preset->getDoubleValue());
 
     // determine if this should be an on-ground or in-air start
     if ((fabs(gs) > 0.01 || fabs(od) > 0.1 || alt > 0.1) && carrier.empty()) {
         fgSetBool("/sim/presets/onground", false);
     } else {
         fgSetBool("/sim/presets/onground", true);
-    }                              
+    }
 
     return true;
 }
@@ -1266,6 +1340,8 @@ bool fgInitGeneral() {
     }
 #endif
 
+    char buf[512], *cwd = getcwd(buf, 512);
+    fgSetString("/sim/fg-current", cwd ? cwd : "");
     return true;
 }
 
@@ -1284,90 +1360,83 @@ void fgInitFDM() {
     double dt = 1.0 / fgGetInt("/sim/model-hz");
     string model = fgGetString("/sim/flight-model");
 
-    try {
-        if ( model == "larcsim" ) {
-            cur_fdm_state = new FGLaRCsim( dt );
-        } else if ( model == "jsb" ) {
-            cur_fdm_state = new FGJSBsim( dt );
-#if ENABLE_SP_FDMS
-        } else if ( model == "ada" ) {
-            cur_fdm_state = new FGADA( dt );
-        } else if ( model == "acms" ) {
-            cur_fdm_state = new FGACMS( dt );
+    if ( model == "larcsim" ) {
+        cur_fdm_state = new FGLaRCsim( dt );
+    } else if ( model == "jsb" ) {
+        cur_fdm_state = new FGJSBsim( dt );
+#if ENABLE_SP_FDM
+    } else if ( model == "ada" ) {
+        cur_fdm_state = new FGADA( dt );
+    } else if ( model == "acms" ) {
+        cur_fdm_state = new FGACMS( dt );
 #endif
-        } else if ( model == "balloon" ) {
-            cur_fdm_state = new FGBalloonSim( dt );
-        } else if ( model == "magic" ) {
-            cur_fdm_state = new FGMagicCarpet( dt );
-        } else if ( model == "ufo" ) {
-            cur_fdm_state = new FGUFO( dt );
-        } else if ( model == "external" ) {
-            // external is a synonym for "--fdm=null" and is
-            // maintained here for backwards compatibility
-            cur_fdm_state = new FGNullFDM( dt );
-        } else if ( model.find("network") == 0 ) {
-            string host = "localhost";
-            int port1 = 5501;
-            int port2 = 5502;
-            int port3 = 5503;
-            string net_options = model.substr(8);
-            string::size_type begin, end;
-            begin = 0;
-            // host
-            end = net_options.find( ",", begin );
-            if ( end != string::npos ) {
-                host = net_options.substr(begin, end - begin);
-                begin = end + 1;
-            }
-            // port1
-            end = net_options.find( ",", begin );
-            if ( end != string::npos ) {
-                port1 = atoi( net_options.substr(begin, end - begin).c_str() );
-                begin = end + 1;
-            }
-            // port2
-            end = net_options.find( ",", begin );
-            if ( end != string::npos ) {
-                port2 = atoi( net_options.substr(begin, end - begin).c_str() );
-                begin = end + 1;
-            }
-            // port3
-            end = net_options.find( ",", begin );
-            if ( end != string::npos ) {
-                port3 = atoi( net_options.substr(begin, end - begin).c_str() );
-                begin = end + 1;
-            }
-            cur_fdm_state = new FGExternalNet( dt, host, port1, port2, port3 );
-        } else if ( model.find("pipe") == 0 ) {
-            // /* old */ string pipe_path = model.substr(5);
-            // /* old */ cur_fdm_state = new FGExternalPipe( dt, pipe_path );
-            string pipe_path = "";
-            string pipe_protocol = "";
-            string pipe_options = model.substr(5);
-            string::size_type begin, end;
-            begin = 0;
-            // pipe file path
-            end = pipe_options.find( ",", begin );
-            if ( end != string::npos ) {
-                pipe_path = pipe_options.substr(begin, end - begin);
-                begin = end + 1;
-            }
-            // protocol (last option)
-            pipe_protocol = pipe_options.substr(begin);
-            cur_fdm_state = new FGExternalPipe( dt, pipe_path, pipe_protocol );
-        } else if ( model == "null" ) {
-            cur_fdm_state = new FGNullFDM( dt );
-        } else if ( model == "yasim" ) {
-            cur_fdm_state = new YASim( dt );
-        } else {
-            SG_LOG(SG_GENERAL, SG_ALERT,
-                   "Unrecognized flight model '" << model
-                   << "', cannot init flight dynamics model.");
-            exit(-1);
+    } else if ( model == "balloon" ) {
+        cur_fdm_state = new FGBalloonSim( dt );
+    } else if ( model == "magic" ) {
+        cur_fdm_state = new FGMagicCarpet( dt );
+    } else if ( model == "ufo" ) {
+        cur_fdm_state = new FGUFO( dt );
+    } else if ( model == "external" ) {
+        // external is a synonym for "--fdm=null" and is
+        // maintained here for backwards compatibility
+        cur_fdm_state = new FGNullFDM( dt );
+    } else if ( model.find("network") == 0 ) {
+        string host = "localhost";
+        int port1 = 5501;
+        int port2 = 5502;
+        int port3 = 5503;
+        string net_options = model.substr(8);
+        string::size_type begin, end;
+        begin = 0;
+        // host
+        end = net_options.find( ",", begin );
+        if ( end != string::npos ) {
+            host = net_options.substr(begin, end - begin);
+            begin = end + 1;
         }
-    } catch ( ... ) {
-        SG_LOG(SG_GENERAL, SG_ALERT, "FlightGear aborting\n\n");
-        exit(-1);
+        // port1
+        end = net_options.find( ",", begin );
+        if ( end != string::npos ) {
+            port1 = atoi( net_options.substr(begin, end - begin).c_str() );
+            begin = end + 1;
+        }
+        // port2
+        end = net_options.find( ",", begin );
+        if ( end != string::npos ) {
+            port2 = atoi( net_options.substr(begin, end - begin).c_str() );
+            begin = end + 1;
+        }
+        // port3
+        end = net_options.find( ",", begin );
+        if ( end != string::npos ) {
+            port3 = atoi( net_options.substr(begin, end - begin).c_str() );
+            begin = end + 1;
+        }
+        cur_fdm_state = new FGExternalNet( dt, host, port1, port2, port3 );
+    } else if ( model.find("pipe") == 0 ) {
+        // /* old */ string pipe_path = model.substr(5);
+        // /* old */ cur_fdm_state = new FGExternalPipe( dt, pipe_path );
+        string pipe_path = "";
+        string pipe_protocol = "";
+        string pipe_options = model.substr(5);
+        string::size_type begin, end;
+        begin = 0;
+        // pipe file path
+        end = pipe_options.find( ",", begin );
+        if ( end != string::npos ) {
+            pipe_path = pipe_options.substr(begin, end - begin);
+            begin = end + 1;
+        }
+        // protocol (last option)
+        pipe_protocol = pipe_options.substr(begin);
+        cur_fdm_state = new FGExternalPipe( dt, pipe_path, pipe_protocol );
+    } else if ( model == "null" ) {
+        cur_fdm_state = new FGNullFDM( dt );
+    } else if ( model == "yasim" ) {
+        cur_fdm_state = new YASim( dt );
+    } else {
+        throw sg_throwable(string("Unrecognized flight model '") + model
+               + "', cannot init flight dynamics model.");
     }
 }
 
@@ -1440,7 +1509,7 @@ void fgInitTimeOffset() {
     time_t systemLocalTime = sgTimeGetGMT( localtime(&cur_time) );
     time_t aircraftLocalTime = 
         sgTimeGetGMT( fgLocaltime(&cur_time, t->get_zonename() ) );
-
+    
     // Okay, we now have several possible scenarios
     int offset = fgGetInt("/sim/startup/time-offset");
     string offset_type = fgGetString("/sim/startup/time-offset-type");
@@ -1499,16 +1568,19 @@ void fgInitTimeOffset() {
                                            180.0, false ); 
     } else if ( offset_type == "system-offset" ) {
         warp = offset;
+	orig_warp = 0;
     } else if ( offset_type == "gmt-offset" ) {
         warp = offset - (currGMT - systemLocalTime);
+	orig_warp = 0;
     } else if ( offset_type == "latitude-offset" ) {
         warp = offset - (aircraftLocalTime - systemLocalTime);
+	orig_warp = 0;
     } else if ( offset_type == "system" ) {
-        warp = offset - cur_time;
+      warp = offset - (systemLocalTime - currGMT) - cur_time;
     } else if ( offset_type == "gmt" ) {
-        warp = offset - currGMT;
+        warp = offset - cur_time;
     } else if ( offset_type == "latitude" ) {
-        warp = offset - (aircraftLocalTime - systemLocalTime) - cur_time; 
+        warp = offset - (aircraftLocalTime - currGMT)- cur_time; 
     } else {
         SG_LOG( SG_GENERAL, SG_ALERT,
                 "FG_TIME::Unsupported offset type " << offset_type );
@@ -1547,9 +1619,10 @@ bool fgInitSubsystems() {
     globals->get_event_mgr()->setRealtimeProperty(fgGetNode("/sim/time/delta-realtime-sec", true));
 
     ////////////////////////////////////////////////////////////////////
-    // Initialize the property interpolator subsystem
+    // Initialize the property interpolator subsystem. Put into the INIT
+    // group because the "nasal" subsystem may need it at GENERAL take-down.
     ////////////////////////////////////////////////////////////////////
-    globals->add_subsystem("interpolator", new SGInterpolator);
+    globals->add_subsystem("interpolator", new SGInterpolator, SGSubsystemMgr::INIT);
 
 
     ////////////////////////////////////////////////////////////////////
@@ -1602,6 +1675,22 @@ bool fgInitSubsystems() {
     fgAircraftInit();   // In the future this might not be the case.
 
 
+    ////////////////////////////////////////////////////////////////////
+    // Initialize the weather subsystem.
+    ////////////////////////////////////////////////////////////////////
+
+    // Initialize the weather modeling subsystem
+    globals->add_subsystem("environment", new FGEnvironmentMgr);
+
+
+    ////////////////////////////////////////////////////////////////////
+    // Initialize the aircraft systems and instrumentation (before the
+    // autopilot.)
+    ////////////////////////////////////////////////////////////////////
+
+    globals->add_subsystem("instrumentation", new FGInstrumentMgr);
+    globals->add_subsystem("systems", new FGSystemMgr);
+
     ////////////////////////////////////////////////////////////////////
     // Initialize the XML Autopilot subsystem.
     ////////////////////////////////////////////////////////////////////
@@ -1639,14 +1728,6 @@ bool fgInitSubsystems() {
     fgInitTimeOffset();		// the environment subsystem needs this
 
 
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the weather subsystem.
-    ////////////////////////////////////////////////////////////////////
-
-    // Initialize the weather modeling subsystem
-    globals->add_subsystem("environment", new FGEnvironmentMgr);
-
-
     ////////////////////////////////////////////////////////////////////
     // Initialize the lighting subsystem.
     ////////////////////////////////////////////////////////////////////
@@ -1710,21 +1791,10 @@ bool fgInitSubsystems() {
     globals->add_subsystem("submodel_mgr", new FGSubmodelMgr);
 
 
-     // It's probably a good idea to initialize the top level traffic manager
-     // After the AI and ATC systems have been initialized properly.
-     // AI Traffic manager
-     globals->add_subsystem("Traffic Manager", new FGTrafficManager);
-	     FGTrafficManager *dispatcher = 
-	     (FGTrafficManager*) globals->get_subsystem("Traffic Manager");
-	     SGPath path = globals->get_fg_root();
-	     path.append("/Traffic/fgtraffic.xml");
-     readXML(path.str(),
-	*dispatcher);
-	     //globals->get_subsystem("Traffic Manager")->init();
-
-    globals->add_subsystem("instrumentation", new FGInstrumentMgr);
-    globals->add_subsystem("systems", new FGSystemMgr);
-
+    // It's probably a good idea to initialize the top level traffic manager
+    // After the AI and ATC systems have been initialized properly.
+    // AI Traffic manager
+    globals->add_subsystem("Traffic Manager", new FGTrafficManager);
 
 
     ////////////////////////////////////////////////////////////////////
@@ -1738,17 +1808,6 @@ bool fgInitSubsystems() {
     }
 
 
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the autopilot subsystem.
-    ////////////////////////////////////////////////////////////////////
-
-                                // FIXME: these should go in the
-                                // GUI initialization code, not here.
-    // fgAPAdjustInit();
-    NewTgtAirportInit();
-    NewHeadingInit();
-    NewAltitudeInit();
-
     ////////////////////////////////////////////////////////////////////
     // Initialize I/O subsystem.
     ////////////////////////////////////////////////////////////////////
@@ -1766,7 +1825,7 @@ bool fgInitSubsystems() {
 
     globals->set_current_panel( fgReadPanel(panel_path) );
     if (globals->get_current_panel() == 0) {
-        SG_LOG( SG_INPUT, SG_ALERT, 
+        SG_LOG( SG_INPUT, SG_ALERT,
                 "Error reading new panel from " << panel_path );
     } else {
         SG_LOG( SG_INPUT, SG_INFO, "Loaded new panel from " << panel_path );
@@ -1774,7 +1833,7 @@ bool fgInitSubsystems() {
         globals->get_current_panel()->bind();
     }
 
-    
+
     ////////////////////////////////////////////////////////////////////
     // Initialize the controls subsystem.
     ////////////////////////////////////////////////////////////////////
@@ -1814,14 +1873,9 @@ bool fgInitSubsystems() {
     // Do this last, so that the loaded scripts see initialized state
     ////////////////////////////////////////////////////////////////////////
     FGNasalSys* nasal = new FGNasalSys();
-    globals->add_subsystem("nasal", nasal);
+    globals->add_subsystem("nasal", nasal, SGSubsystemMgr::INIT);
     nasal->init();
 
-    ////////////////////////////////////////////////////////////////////
-    // At this point we could try and parse the waypoint options
-    ///////////////////////////////////////////////////////////////////
-    parseWaypoints();
-
     // initialize methods that depend on other subsystems.
     globals->get_subsystem_mgr()->postinit();
 
