@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: fg_io.cxx,v 1.22 2006-02-21 01:19:18 mfranz Exp $
+// $Id: fg_io.cxx,v 1.24.2.1 2007-07-25 15:17:10 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -43,6 +43,7 @@
 #include <Network/protocol.hxx>
 #include <Network/ATC-Main.hxx>
 #include <Network/atlas.hxx>
+#include <Network/AV400.hxx>
 #include <Network/garmin.hxx>
 #include <Network/httpd.hxx>
 #ifdef FG_JPEG_SERVER
@@ -113,6 +114,7 @@ FGIO::parse_port_config( const string& config )
                 SG_LOG( SG_IO, SG_ALERT, "Usage: --atcsim=[no-]pedals,"
                         << "input0_config,input1_config,"
                         << "output0_config,output1_config,file.nas" );
+		delete atcsim;
                 return NULL;
             }
             if ( tokens[1] == "no-pedals" ) {
@@ -130,6 +132,9 @@ FGIO::parse_port_config( const string& config )
 	    // printf("Parsed opengc\n"); cin >> wait;
 	    FGOpenGC *opengc = new FGOpenGC;
 	    io = opengc;
+	} else if ( protocol == "AV400" ) {
+	    FGAV400 *av400 = new FGAV400;
+	    io = av400;
 	} else if ( protocol == "garmin" ) {
 	    FGGarmin *garmin = new FGGarmin;
 	    io = garmin;
@@ -182,8 +187,12 @@ FGIO::parse_port_config( const string& config )
             else if (tokens[1] == "file") n--;
             FGGeneric *generic = new FGGeneric( tokens[n] );
             io = generic;
-	} else if ( protocol == "multiplay" ) {\
-	    //Determine dir, rate, host & port
+	} else if ( protocol == "multiplay" ) {
+	    if ( tokens.size() != 5 ) {
+		SG_LOG( SG_IO, SG_ALERT, "Ignoring invalid --multiplay option "
+			"(4 arguments expected: --multiplay=dir,hz,hostname,port)" );
+		return NULL;
+	    }
 	    string dir = tokens[1];
 	    string rate = tokens[2];
 	    string host = tokens[3];
