@@ -112,6 +112,14 @@ void fgOSOpenWindow(int w, int h, int bpp,
     // int realh = screen->h;
 }
 
+
+struct keydata {
+    keydata() : unicode(0), mod(0) {};
+    Uint16 unicode;
+    unsigned int mod;
+} keys[SDLK_LAST];
+
+
 static void handleKey(int key, int raw, int keyup)
 {
     int modmask = 0;
@@ -122,6 +130,10 @@ static void handleKey(int key, int raw, int keyup)
     case SDLK_LCTRL:  modmask = KEYMOD_CTRL;  break;
     case SDLK_RALT:   modmask = KEYMOD_ALT;   break;
     case SDLK_LALT:   modmask = KEYMOD_ALT;   break;
+    case SDLK_RMETA:  modmask = KEYMOD_META;  break;
+    case SDLK_LMETA:  modmask = KEYMOD_META;  break;
+    case SDLK_RSUPER: modmask = KEYMOD_SUPER; break;
+    case SDLK_LSUPER: modmask = KEYMOD_SUPER; break;
 
     case SDLK_LEFT:     key = PU_KEY_LEFT;      break;
     case SDLK_UP:       key = PU_KEY_UP;        break;
@@ -173,8 +185,17 @@ static void handleKey(int key, int raw, int keyup)
         CurrentModifiers |= modmask;
         keymod = CurrentModifiers & ~KEYMOD_RELEASED;
     }
-    if(modmask == 0 && KeyHandler)
+
+    if(modmask == 0 && KeyHandler) {
+        if (keymod & KEYMOD_RELEASED) {
+            key = keys[raw].unicode;
+
+        } else {
+            keys[raw].mod = keymod;
+            keys[raw].unicode = key;
+        }
         (*KeyHandler)(key, keymod, CurrentMouseX, CurrentMouseY);
+    }
 }
 
 // FIXME: Integrate with existing fgExit() in util.cxx.
