@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: fg_props.cxx,v 1.25 2006-02-21 01:19:18 mfranz Exp $
+// $Id: fg_props.cxx,v 1.28.2.4 2007-11-24 08:37:46 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -96,7 +96,8 @@ static const char *
 getLoggingClasses ()
 {
   sgDebugClass classes = logbuf::get_log_classes();
-  static string result = "";	// FIXME
+  static string result;
+  result = "";
   for (int i = 0; log_class_mappings[i].c != SG_UNDEFD; i++) {
     if ((classes&log_class_mappings[i].c) > 0) {
       if (!result.empty())
@@ -232,7 +233,7 @@ setFreeze (bool f)
     if ( s != NULL ) {
         if ( f ) {
             s->pause();
-        } else {
+        } else if (!fgGetBool("/sim/sound/pause")) {
             s->resume();
         }
     }
@@ -257,7 +258,7 @@ getDateString ()
 {
   static char buf[64];		// FIXME
   struct tm * t = globals->get_time_params()->getGmt();
-  sprintf(buf, "%.4d-%.2d-%.2dT%.2d:%.2d:%.2d",
+  snprintf(buf, 64, "%.4d-%.2d-%.2dT%.2d:%.2d:%.2d",
 	  t->tm_year + 1900, t->tm_mon + 1, t->tm_mday,
 	  t->tm_hour, t->tm_min, t->tm_sec);
   return buf;
@@ -316,11 +317,10 @@ setDateString (const char * date_string)
 static const char *
 getGMTString ()
 {
-  static char buf[16];		// FIXME
+  static char buf[16];
   struct tm *t = globals->get_time_params()->getGmt();
-  sprintf(buf, " %.2d:%.2d:%.2d",
-	  t->tm_hour, t->tm_min, t->tm_sec);
-  // cout << t << " " << buf << endl;
+  snprintf(buf, 16, "%.2d:%.2d:%.2d",
+      t->tm_hour, t->tm_min, t->tm_sec);
   return buf;
 }
 
@@ -433,6 +433,72 @@ setFDMDataLogging (bool state)
   }
 }
 
+static const char *
+getLongitudeString ()
+{
+  static SGConstPropertyNode_ptr n = fgGetNode("/position/longitude-deg", true);
+  static SGConstPropertyNode_ptr f = fgGetNode("/sim/lon-lat-format", true);
+  static char buf[32];
+  double d = n->getDoubleValue();
+  int format = f->getIntValue();
+  char c = d < 0.0 ? 'W' : 'E';
+
+  if (format == 0) {
+    snprintf(buf, 32, "%3.6f%c", d, c);
+
+  } else if (format == 1) {
+    // dd mm.mmm' (DMM-Format) -- uses a round-off factor tailored to the
+    // required precision of the minutes field (three decimal places),
+    // preventing minute values of 60.
+    double deg = fabs(d) + 5.0E-4 / 60.0;
+    double min = fabs(deg - int(deg)) * 60.0 - 4.999E-4;
+    snprintf(buf, 32, "%d*%06.3f%c", int(d < 0.0 ? -deg : deg), min, c);
+
+  } else {
+    // mm'ss.s'' (DMS-Format) -- uses a round-off factor tailored to the
+    // required precision of the seconds field (one decimal place),
+    // preventing second values of 60.
+    double deg = fabs(d) + 0.05 / 3600.0;
+    double min = (deg - int(deg)) * 60.0;
+    double sec = (min - int(min)) * 60.0 - 0.049;
+    snprintf(buf, 32, "%d*%02d %04.1f%c", int(d < 0.0 ? -deg : deg),
+        int(min), fabs(sec), c);
+  }
+  buf[31] = '\0';
+  return buf;
+}
+
+static const char *
+getLatitudeString ()
+{
+  static SGConstPropertyNode_ptr n = fgGetNode("/position/latitude-deg", true);
+  static SGConstPropertyNode_ptr f = fgGetNode("/sim/lon-lat-format", true);
+  static char buf[32];
+  double d = n->getDoubleValue();
+  int format = f->getIntValue();
+  char c = d < 0.0 ? 'S' : 'N';
+
+  if (format == 0) {
+    snprintf(buf, 32, "%3.6f%c", d, c);
+
+  } else if (format == 1) {
+    double deg = fabs(d) + 5.0E-4 / 60.0;
+    double min = fabs(deg - int(deg)) * 60.0 - 4.999E-4;
+    snprintf(buf, 32, "%d*%06.3f%c", int(d < 0.0 ? -deg : deg), min, c);
+
+  } else {
+    double deg = fabs(d) + 0.05 / 3600.0;
+    double min = (deg - int(deg)) * 60.0;
+    double sec = (min - int(min)) * 60.0 - 0.049;
+    snprintf(buf, 32, "%d*%02d %04.1f%c", int(d < 0.0 ? -deg : deg),
+        int(min), fabs(sec), c);
+  }
+  buf[31] = '\0';
+  return buf;
+}
+
+
+
 
 ////////////////////////////////////////////////////////////////////////
 // Tie the properties.
@@ -464,6 +530,10 @@ FGProperties::bind ()
   fgSetArchivable("/sim/time/gmt");
   fgTie("/sim/time/gmt-string", getGMTString);
 
+				// Position
+  fgTie("/position/latitude-string", getLatitudeString);
+  fgTie("/position/longitude-string", getLongitudeString);
+
 				// Orientation
   fgTie("/orientation/heading-magnetic-deg", getHeadingMag);
 
@@ -490,6 +560,9 @@ FGProperties::unbind ()
   fgUntie("/sim/time/elapsed-sec");
   fgUntie("/sim/time/gmt");
   fgUntie("/sim/time/gmt-string");
+				// Position
+  fgUntie("/position/latitude-string");
+  fgUntie("/position/longitude-string");
 
 				// Orientation
   fgUntie("/orientation/heading-magnetic-deg");
