@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: main.cxx,v 1.218 2006-03-21 18:52:19 curt Exp $
+// $Id: main.cxx,v 1.227.2.15 2007-12-11 21:32:51 durk Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -105,7 +105,7 @@ long global_multi_loop;
 SGTimeStamp last_time_stamp;
 SGTimeStamp current_time_stamp;
 
-// The atexit() functio handler should know when the graphical subsystem
+// The atexit() function handler should know when the graphical subsystem
 // is initialized.
 extern int _bootstrap_OSInit;
 
@@ -133,20 +133,20 @@ void fgUpdateTimeDepCalcs() {
     //      << " cur_elev = " << scenery.get_cur_elev() << endl;
 
     if (!cur_fdm_state->get_inited()) {
-      // Check for scenery around the aircraft.
-      double lon = fgGetDouble("/sim/presets/longitude-deg");
-      double lat = fgGetDouble("/sim/presets/latitude-deg");
-      // We require just to have 50 meter scenery availabe around
-      // the aircraft.
-      double range = 50.0;
-      if (globals->get_tile_mgr()->scenery_available(lat, lon, range)) {
-        SG_LOG(SG_FLIGHT,SG_INFO, "Finally initializing fdm");
-        cur_fdm_state->init();
-        if ( cur_fdm_state->get_bound() ) {
-            cur_fdm_state->unbind();
+        // Check for scenery around the aircraft.
+        double lon = fgGetDouble("/sim/presets/longitude-deg");
+        double lat = fgGetDouble("/sim/presets/latitude-deg");
+        // We require just to have 50 meter scenery availabe around
+        // the aircraft.
+        double range = 50.0;
+        if (globals->get_tile_mgr()->scenery_available(lat, lon, range)) {
+            SG_LOG(SG_FLIGHT,SG_INFO, "Finally initializing fdm");
+            cur_fdm_state->init();
+            if ( cur_fdm_state->get_bound() ) {
+                cur_fdm_state->unbind();
+            }
+            cur_fdm_state->bind();
         }
-        cur_fdm_state->bind();
-      }
     }
 
     // conceptually, the following block could be done for each fdm
@@ -154,25 +154,27 @@ void fgUpdateTimeDepCalcs() {
     if ( cur_fdm_state->get_inited() ) {
         // we have been inited, and  we are good to go ...
 
-        if ( !inited ) {
-            inited = true;
-        }
-
         if ( replay_state->getIntValue() == 0 ) {
-	    // replay off, run fdm
+            // replay off, run fdm
             cur_fdm_state->update( delta_time_sec );
         } else {
             FGReplay *r = (FGReplay *)(globals->get_subsystem( "replay" ));
             r->replay( replay_time->getDoubleValue() );
-  	    if ( replay_state->getIntValue() == 1 ) {
+            if ( replay_state->getIntValue() == 1 ) {
                 // normal playback
                 replay_time->setDoubleValue( replay_time->getDoubleValue()
                                              + ( delta_time_sec
                                                * fgGetInt("/sim/speed-up") ) );
-	    } else if ( replay_state->getIntValue() == 2 ) {
-	        // paused playback (don't advance replay time)
-	    }
-	}
+            } else if ( replay_state->getIntValue() == 2 ) {
+                // paused playback (don't advance replay time)
+            }
+        }
+
+        if ( !inited ) {
+            inited = true;
+            fgSetBool("/sim/signals/fdm-initialized", true);
+        }
+
     } else {
         // do nothing, fdm isn't inited yet
     }
@@ -180,9 +182,6 @@ void fgUpdateTimeDepCalcs() {
     globals->get_model_mgr()->update(delta_time_sec);
     globals->get_aircraft_model()->update(delta_time_sec);
 
-    // update the view angle
-    globals->get_viewmgr()->update(delta_time_sec);
-
     // Update solar system
     globals->get_ephem()->update( globals->get_time_params()->getMjd(),
                                   globals->get_time_params()->getLst(),
@@ -205,22 +204,26 @@ static const double alt_adjust_m = alt_adjust_ft * SG_FEET_TO_METER;
 static void fgMainLoop( void ) {
     int model_hz = fgGetInt("/sim/model-hz");
 
-    static const SGPropertyNode *longitude
+    static SGConstPropertyNode_ptr longitude
         = fgGetNode("/position/longitude-deg");
-    static const SGPropertyNode *latitude
+    static SGConstPropertyNode_ptr latitude
         = fgGetNode("/position/latitude-deg");
-    static const SGPropertyNode *altitude
+    static SGConstPropertyNode_ptr altitude
         = fgGetNode("/position/altitude-ft");
-    static const SGPropertyNode *clock_freeze
+    static SGConstPropertyNode_ptr clock_freeze
         = fgGetNode("/sim/freeze/clock", true);
-    static const SGPropertyNode *cur_time_override
+    static SGConstPropertyNode_ptr cur_time_override
         = fgGetNode("/sim/time/cur-time-override", true);
-    static const SGPropertyNode *max_simtime_per_frame
+    static SGConstPropertyNode_ptr max_simtime_per_frame
         = fgGetNode("/sim/max-simtime-per-frame", true);
+    static SGPropertyNode_ptr frame_signal
+        = fgGetNode("/sim/signals/frame", true);
 
+    frame_signal->setBoolValue(true);
     SGCloudLayer::enable_bump_mapping = fgGetBool("/sim/rendering/bump-mapping");
 
-    bool scenery_loaded = fgGetBool("sim/sceneryloaded") || fgGetBool("sim/sceneryloaded-override");
+    bool scenery_loaded = fgGetBool("sim/sceneryloaded");
+    bool wait_for_scenery = !(scenery_loaded || fgGetBool("sim/sceneryloaded-override"));
 
     // Update the elapsed time.
     static bool first_time = true;
@@ -230,8 +233,8 @@ static void fgMainLoop( void ) {
     }
 
     double throttle_hz = fgGetDouble("/sim/frame-rate-throttle-hz", 0.0);
-    if ( throttle_hz > 0.0 && scenery_loaded ) {
-        // optionally throttle the frame rate (to get consistant frame
+    if ( throttle_hz > 0.0 && !wait_for_scenery ) {
+        // optionally throttle the frame rate (to get consistent frame
         // rates or reduce cpu usage.
 
         double frame_us = 1000000.0 / throttle_hz;
@@ -242,18 +245,18 @@ static void fgMainLoop( void ) {
         //
         // Calling sleep, even usleep() on linux is less accurate than
         // we like, but it does free up the cpu for other tasks during
-        // the sleep so it is desireable.  Because of the way sleep()
-        // is implimented in consumer operating systems like windows
+        // the sleep so it is desirable.  Because of the way sleep()
+        // is implemented in consumer operating systems like windows
         // and linux, you almost always sleep a little longer than the
         // requested amount.
-        // 
-        // To combat the problem of sleeping to long, we calculate the
+        //
+        // To combat the problem of sleeping too long, we calculate the
         // desired wait time and shorten it by 2000us (2ms) to avoid
         // [hopefully] over-sleep'ing.  The 2ms value was arrived at
         // via experimentation.  We follow this up at the end with a
         // simple busy-wait loop to get the final pause timing exactly
         // right.
-        // 
+        //
         // Assuming we don't oversleep by more than 2000us, this
         // should be a reasonable compromise between sleep based
         // waiting, and busy waiting.
@@ -265,6 +268,7 @@ static void fgMainLoop( void ) {
             frame_us = 0.0;
         }
         current_time_stamp.stamp();
+
         /* Convert to ms */
         double elapsed_us = current_time_stamp - last_time_stamp;
         if ( elapsed_us < frame_us ) {
@@ -274,9 +278,9 @@ static void fgMainLoop( void ) {
 #endif
 
         // busy wait timing loop.
-        // 
+        //
         // This yields the most accurate timing.  If the previous
-        // ulMilliSecondSleep() call is ommitted this will peg the cpu
+        // ulMilliSecondSleep() call is omitted this will peg the cpu
         // (which is just fine if FG is the only app you care about.)
         current_time_stamp.stamp();
         while ( current_time_stamp - last_time_stamp < frame_us ) {
@@ -293,29 +297,29 @@ static void fgMainLoop( void ) {
     // Limit the time we need to spend in simulation loops
     // That means, if the /sim/max-simtime-per-frame value is strictly positive
     // you can limit the maximum amount of time you will do simulations for
-    // one frame to display. The cpu time spent in simulations code is roughtly
+    // one frame to display. The cpu time spent in simulations code is roughly
     // at least O(real_delta_time_sec). If this is (due to running debug
     // builds or valgrind or something different blowing up execution times)
-    // larger than the real time you will no more get any response
+    // larger than the real time you will no longer get any response
     // from flightgear. This limits that effect. Just set to property from
     // your .fgfsrc or commandline ...
     double dtMax = max_simtime_per_frame->getDoubleValue();
     if (0 < dtMax && dtMax < real_delta_time_sec)
-      real_delta_time_sec = dtMax;
+        real_delta_time_sec = dtMax;
 
     // round the real time down to a multiple of 1/model-hz.
     // this way all systems are updated the _same_ amount of dt.
     {
-      static double rem = 0.0;
-      real_delta_time_sec += rem;
-      double hz = model_hz;
-      double nit = floor(real_delta_time_sec*hz);
-      rem = real_delta_time_sec - nit/hz;
-      real_delta_time_sec = nit/hz;
+        static double rem = 0.0;
+        real_delta_time_sec += rem;
+        double hz = model_hz;
+        double nit = floor(real_delta_time_sec*hz);
+        rem = real_delta_time_sec - nit/hz;
+        real_delta_time_sec = nit/hz;
     }
 
 
-    if (clock_freeze->getBoolValue() || !scenery_loaded) {
+    if (clock_freeze->getBoolValue() || wait_for_scenery) {
         delta_time_sec = 0;
     } else {
         delta_time_sec = real_delta_time_sec;
@@ -340,8 +344,6 @@ static void fgMainLoop( void ) {
 
     SGTime *t = globals->get_time_params();
 
-    globals->get_event_mgr()->update(delta_time_sec);
-
     SG_LOG( SG_ALL, SG_DEBUG, "Running Main Loop");
     SG_LOG( SG_ALL, SG_DEBUG, "======= ==== ====");
 
@@ -400,22 +402,22 @@ static void fgMainLoop( void ) {
                globals->get_warp() );
 
     if (globals->get_warp_delta() != 0) {
-	FGLight *l = (FGLight *)(globals->get_subsystem("lighting"));
-	l->update( 0.5 );
+        FGLight *l = (FGLight *)(globals->get_subsystem("lighting"));
+        l->update( 0.5 );
     }
 
     // update magvar model
     globals->get_mag()->update( longitude->getDoubleValue()
-                              * SGD_DEGREES_TO_RADIANS,
-                            latitude->getDoubleValue()
-                              * SGD_DEGREES_TO_RADIANS,
-                            altitude->getDoubleValue() * SG_FEET_TO_METER,
-                            globals->get_time_params()->getJD() );
+                                * SGD_DEGREES_TO_RADIANS,
+                                latitude->getDoubleValue()
+                                * SGD_DEGREES_TO_RADIANS,
+                                altitude->getDoubleValue() * SG_FEET_TO_METER,
+                                globals->get_time_params()->getJD() );
 
     // Get elapsed time (in usec) for this past frame
     elapsed = fgGetTimeInterval();
-    SG_LOG( SG_ALL, SG_DEBUG, 
-            "Elapsed time interval is = " << elapsed 
+    SG_LOG( SG_ALL, SG_DEBUG,
+            "Elapsed time interval is = " << elapsed
             << ", previous remainder is = " << remainder );
 
     // Calculate frame rate average
@@ -441,7 +443,7 @@ static void fgMainLoop( void ) {
     if ( (t->get_cur_time() != last_time) && (last_time > 0) ) {
         general.set_frame_rate( frames );
         fgSetInt("/sim/frame-rate", frames);
-        SG_LOG( SG_ALL, SG_DEBUG, 
+        SG_LOG( SG_ALL, SG_DEBUG,
             "--> Frame rate is = " << general.get_frame_rate() );
         frames = 0;
     }
@@ -458,7 +460,7 @@ static void fgMainLoop( void ) {
         globals->get_ATC_mgr()->update(delta_time_sec);
 
     // Run the AI subsystem
-    // FIXME: run that also if we have multiplying enabled since the
+    // FIXME: run that also if we have multiplaying enabled since the
     // multiplayer information is interpreted by an AI model
     if (fgGetBool("/sim/ai-traffic/enabled"))
         globals->get_AI_mgr()->update(delta_time_sec);
@@ -470,12 +472,12 @@ static void fgMainLoop( void ) {
 
     global_multi_loop = (long)(((double)elapsed * 0.000001) * model_hz );
     remainder = elapsed - ( (global_multi_loop*1000000) / model_hz );
-    SG_LOG( SG_ALL, SG_DEBUG, 
+    SG_LOG( SG_ALL, SG_DEBUG,
             "Model iterations needed = " << global_multi_loop
             << ", new remainder = " << remainder );
-        
-    // chop max interations to something reasonable if the sim was
-    // delayed for an excesive amount of time
+
+    // chop max iterations to something reasonable if the sim was
+    // delayed for an excessive amount of time
     if ( global_multi_loop > 2.0 * model_hz ) {
         global_multi_loop = (int)(2.0 * model_hz );
         remainder = 0;
@@ -483,11 +485,11 @@ static void fgMainLoop( void ) {
 
     // flight model
     if ( global_multi_loop > 0) {
-        // first run the flight model each frame until it is intialized
+        // first run the flight model each frame until it is initialized
         // then continue running each frame only after initial scenery load is complete.
         fgUpdateTimeDepCalcs();
     } else {
-        SG_LOG( SG_ALL, SG_DEBUG, 
+        SG_LOG( SG_ALL, SG_DEBUG,
             "Elapsed time is zero ... we're zinging" );
     }
 
@@ -509,11 +511,10 @@ static void fgMainLoop( void ) {
     //
     // Tile Manager updates - see if we need to load any new scenery tiles.
     //   this code ties together the fdm, viewer and scenery classes...
-    //   we may want to move this to it's own class at some point
+    //   we may want to move this to its own class at some point
     //
     double visibility_meters = fgGetDouble("/environment/visibility-m");
     FGViewer *current_view = globals->get_current_view();
-
     // Let the scenery center follow the current view position with
     // 30m increments.
     //
@@ -531,78 +532,152 @@ static void fgMainLoop( void ) {
     // position. This roundoff is still visible but you will most
     // propably not notice.
     double *vp = globals->get_current_view()->get_absolute_view_pos();
-    Point3D cntr(vp[0], vp[1], vp[2]);
-    if (30.0*30.0 < cntr.distance3Dsquared(globals->get_scenery()->get_center())) {
-      globals->get_scenery()->set_next_center( cntr );
-    }
+    SGVec3d cntr(vp);
+    if (30.0*30.0 < distSqr(cntr, globals->get_scenery()->get_center()))
+      globals->get_scenery()->set_center( cntr );
 
     globals->get_tile_mgr()->prep_ssg_nodes( current_view->getSGLocation(),
                                              visibility_meters );
     // update tile manager for view...
     SGLocation *view_location = globals->get_current_view()->getSGLocation();
     globals->get_tile_mgr()->update( view_location, visibility_meters );
-    globals->get_scenery()->set_center( cntr );
     {
-      double lon = view_location->getLongitude_deg();
-      double lat = view_location->getLatitude_deg();
-      double alt = view_location->getAltitudeASL_ft() * SG_FEET_TO_METER;
+        double lon = view_location->getLongitude_deg();
+        double lat = view_location->getLatitude_deg();
+        double alt = view_location->getAltitudeASL_ft() * SG_FEET_TO_METER;
 
-      // check if we can reuse the groundcache for that purpose.
-      double ref_time, pt[3], r;
-      bool valid = cur_fdm_state->is_valid_m(&ref_time, pt, &r);
-      if (valid &&
-          cntr.distance3Dsquared(Point3D(pt[0], pt[1], pt[2])) < r*r) {
-        // Reuse the cache ...
-        double lev
-          = cur_fdm_state->get_groundlevel_m(lat*SGD_DEGREES_TO_RADIANS,
-                                             lon*SGD_DEGREES_TO_RADIANS,
-                                             alt + 2.0);
-        view_location->set_cur_elev_m( lev );
-      } else {
-        // Do full intersection test.
-        double lev;
-        if (globals->get_scenery()->get_elevation_m(lat, lon, alt+2, lev))
-          view_location->set_cur_elev_m( lev );
-        else
-          view_location->set_cur_elev_m( -9999.0 );
-      }
+        // check if we can reuse the groundcache for that purpose.
+        double ref_time, r;
+        SGVec3d pt;
+        bool valid = cur_fdm_state->is_valid_m(&ref_time, pt.sg(), &r);
+        double *vp = globals->get_current_view()->get_absolute_view_pos();
+        SGVec3d viewpos(vp);
+        if (valid && distSqr(viewpos, pt) < r*r) {
+            // Reuse the cache ...
+            double lev
+                = cur_fdm_state->get_groundlevel_m(lat*SGD_DEGREES_TO_RADIANS,
+                                                   lon*SGD_DEGREES_TO_RADIANS,
+                                                   alt + 2.0);
+            view_location->set_cur_elev_m( lev );
+        } else {
+            // Do full intersection test.
+            double lev;
+            if (globals->get_scenery()->get_elevation_m(lat, lon, alt+2, lev, 0))
+                view_location->set_cur_elev_m( lev );
+            else
+                view_location->set_cur_elev_m( -9999.0 );
+        }
     }
 
+    // run Nasal's settimer() loops right before the view manager
+    globals->get_event_mgr()->update(delta_time_sec);
+
+    // update the view angle as late as possible, but before sound calculations
+    globals->get_viewmgr()->update(delta_time_sec);
+
 #ifdef ENABLE_AUDIO_SUPPORT
     // Right now we make a simplifying assumption that the primary
     // aircraft is the source of all sounds and that all sounds are
     // positioned relative to the current view position.
 
-    static sgVec3 last_pos_offset;
+    static sgdVec3 last_visitor_pos = {0, 0, 0};
+    static sgdVec3 last_model_pos = {0, 0, 0};
 
-    // get the location data for the primary FDM (now hardcoded to ac model)...
+    // get the orientation
+    const sgVec4 *view = current_view->get_VIEW();
+
+    SGQuatd surf_or = SGQuatd::fromLonLatDeg(
+        current_view->getLongitude_deg(), current_view->getLatitude_deg());
+
+    SGQuatd model_or = SGQuatd::fromYawPitchRollDeg(
+        globals->get_aircraft_model()->get3DModel()->getHeadingDeg(),
+        globals->get_aircraft_model()->get3DModel()->getPitchDeg(),
+        globals->get_aircraft_model()->get3DModel()->getRollDeg());
+
+    // get the up and at vector in the aircraft base
+    sgVec3 u;
+    sgSetVec3 ( u, 0, 0, 1 ); //up
+    sgXformVec3( u, view );
+    SGVec3d sgv_up = model_or.rotateBack(
+        surf_or.rotateBack(SGVec3d(u[0], u[1], u[2])));
+    sgVec3 up;
+    sgSetVec3(up, sgv_up[0], sgv_up[1], sgv_up[2]);
+
+    sgSetVec3 ( u, 0, -1, 0 ); //at
+    sgXformVec3( u, view );
+    SGVec3d sgv_at = model_or.rotateBack(
+        surf_or.rotateBack(SGVec3d(u[0], u[1], u[2])));
+    sgVec3 at;
+    sgSetVec3(at, sgv_at[0], sgv_at[1], sgv_at[2]);
+
+     // get the location data for the primary FDM (now hardcoded to ac model)...
     SGLocation *acmodel_loc = NULL;
     acmodel_loc = (SGLocation *)globals->
         get_aircraft_model()->get3DModel()->getSGLocation();
 
+    // calculate speed of visitor and model
+    sgVec3 listener_vel, model_vel;
+    sgdVec3 v;
+    SGVec3d w;
+    sgdVec3 null = {0, 0, 0};
+
+    double *abs_view_pos = current_view->get_absolute_view_pos();
+    sgdSubVec3( v, last_visitor_pos, abs_view_pos);
+    sgdAddVec3( last_visitor_pos, null, abs_view_pos);
+    w = model_or.rotateBack( surf_or.rotateBack(SGVec3d(v[0], v[1], v[2])));
+    sgSetVec3( listener_vel, w[0], w[1], w[2]);
+
+    double *model_abs_view_pos = acmodel_loc->get_absolute_view_pos();
+    sgdSubVec3( v, last_model_pos, model_abs_view_pos);
+    sgdAddVec3( last_model_pos, null, model_abs_view_pos);
+    w = model_or.rotateBack( surf_or.rotateBack(SGVec3d(v[0], v[1], v[2])));
+    sgSetVec3( model_vel, w[0], w[1], w[2]);
+
+    if (delta_time_sec > 0) {
+        sgScaleVec3( model_vel, 1 / delta_time_sec );
+        sgScaleVec3( listener_vel, 1 / delta_time_sec );
+    }
+
+    //checking, if the listener pos has moved suddenly
+    if (sgLengthVec3(listener_vel) > 1000)
+    {
+        //check if the relative speed model vs listener has moved suddenly, too
+        sgVec3 delta_vel;
+        sgSubVec3(delta_vel, listener_vel, model_vel );
+        if (sgLengthVec3(delta_vel) > 1000)
+            sgSetVec3(listener_vel, model_vel[0], model_vel[1], model_vel[2] );
+        else
+            globals->get_soundmgr()->set_listener_vel( listener_vel );
+    }
+    else
+        globals->get_soundmgr()->set_listener_vel( listener_vel );
+
     // set positional offset for sources
-    sgdVec3 dsource_pos_offset;
-    sgdSubVec3( dsource_pos_offset,
-                view_location->get_absolute_view_pos(),
-                acmodel_loc->get_absolute_view_pos() );
-    // cout << "pos all = " << source_pos_offset[0] << " " << source_pos_offset[1] << " " << source_pos_offset[2] << endl;
+    sgdVec3 dsource_pos_offset = { abs_view_pos[0], abs_view_pos[1], abs_view_pos[2] };
+
+    sgdSubVec3( dsource_pos_offset, dsource_pos_offset, model_abs_view_pos );
+    SGVec3d sgv_dsource_pos_offset = model_or.rotateBack(
+         surf_or.rotateBack(SGVec3d(dsource_pos_offset[0],
+         dsource_pos_offset[1], dsource_pos_offset[2])));
+
     sgVec3 source_pos_offset;
-    sgSetVec3(source_pos_offset, dsource_pos_offset);
+    sgSetVec3(source_pos_offset, sgv_dsource_pos_offset[0],
+         sgv_dsource_pos_offset[1], sgv_dsource_pos_offset[2]);
     globals->get_soundmgr()->set_source_pos_all( source_pos_offset );
 
-    // set the velocity
-    sgVec3 source_vel;
-    sgSubVec3( source_vel, source_pos_offset, last_pos_offset );
-    sgScaleVec3( source_vel, delta_time_sec );
-    sgCopyVec3( last_pos_offset, source_pos_offset );
-    // cout << "vel = " << source_vel[0] << " " << source_vel[1] << " " << source_vel[2] << endl;
-    globals->get_soundmgr()->set_source_vel_all( source_vel );
+    float orient[6];
+    for (int i = 0; i < 3; i++) {
+        orient[i] = sgv_at[i];
+        orient[i + 3] = sgv_up[i];
+     }
+     globals->get_soundmgr()->set_listener_orientation( orient );
 
-    // Right now we make a simplifying assumption that the listener is
-    // always positioned at the origin.
+    // set the velocity
+    globals->get_soundmgr()->set_source_vel_all( model_vel );
+
+    // The listener is always positioned at the origin.
     sgVec3 listener_pos;
     sgSetVec3( listener_pos, 0.0, 0.0, 0.0 );
-    // cout << "listener = " << listener_pos[0] << " " << listener_pos[1] << " " << listener_pos[2] << endl;
     globals->get_soundmgr()->set_listener_pos( listener_pos );
 #endif
 
@@ -638,14 +713,14 @@ static void fgMainLoop( void ) {
 static void fgIdleFunction ( void ) {
     if ( idle_state == 0 ) {
         idle_state++;
-        fgSplashProgress("setting up scenegraph & user interface");
 
-
-    } else if ( idle_state == 1 ) {
-        idle_state++;
+#ifdef GL_EXT_texture_lod_bias
+        // negative values sharpen, positive values blur mipmapped textures
+        glTexEnvf( GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, -0.5 ) ;
+#endif
         // This seems to be the absolute earliest in the init sequence
         // that these calls will return valid info.  Too bad it's after
-        // we've already created and sized out window. :-(
+        // we've already created and sized our window. :-(
         general.set_glVendor( (char *)glGetString ( GL_VENDOR ) );
         general.set_glRenderer( (char *)glGetString ( GL_RENDERER ) );
         general.set_glVersion( (char *)glGetString ( GL_VERSION ) );
@@ -660,43 +735,41 @@ static void fgIdleFunction ( void ) {
         general.set_glDepthBits( tmp );
         SG_LOG ( SG_GENERAL, SG_INFO, "Depth buffer bits = " << tmp );
 
+        // Initialize the user interface so that we can use fonts
+        guiInit();
+        fgSplashProgress("setting up scenegraph");
+
+
+    } else if ( idle_state == 1 ) {
+        idle_state++;
+
         // Initialize ssg (from plib).  Needs to come before we do any
         // other ssg stuff, but after opengl has been initialized.
         ssgInit();
 
-        // Initialize the user interface (we need to do this before
-        // passing off control to the OS main loop and before
-         // fgInitGeneral to get our fonts !!!
-        guiInit();
         fgSplashProgress("reading aircraft list");
 
 
     } else if ( idle_state == 2 ) {
         idle_state++;
-        // Read the list of available aircrafts
+        // Read the list of available aircraft
         fgReadAircraft();
 
-#ifdef GL_EXT_texture_lod_bias
-        glTexEnvf( GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, -0.5 ) ;
-#endif
-
         // get the address of our OpenGL extensions
-        if ( fgGetBool("/sim/rendering/distance-attenuation") ) {
-            if (SGIsOpenGLExtensionSupported("GL_EXT_point_parameters") ) {
-                glPointParameterIsSupported = true;
-                glPointParameterfPtr = (glPointParameterfProc)
-                                       SGLookupFunction("glPointParameterfEXT");
-                glPointParameterfvPtr = (glPointParameterfvProc)
-                                        SGLookupFunction("glPointParameterfvEXT");
-
-            } else if ( SGIsOpenGLExtensionSupported("GL_ARB_point_parameters") ) {
-                glPointParameterIsSupported = true;
-                glPointParameterfPtr = (glPointParameterfProc)
-                                       SGLookupFunction("glPointParameterfARB");
-                glPointParameterfvPtr = (glPointParameterfvProc)
-                                        SGLookupFunction("glPointParameterfvARB");
-            } else
-                glPointParameterIsSupported = false;
+        if (SGIsOpenGLExtensionSupported("GL_EXT_point_parameters") ) {
+            glPointParameterIsSupported = true;
+            glPointParameterfPtr = (glPointParameterfProc)
+                SGLookupFunction("glPointParameterfEXT");
+            glPointParameterfvPtr = (glPointParameterfvProc)
+                SGLookupFunction("glPointParameterfvEXT");
+        } else if ( SGIsOpenGLExtensionSupported("GL_ARB_point_parameters") ) {
+            glPointParameterIsSupported = true;
+            glPointParameterfPtr = (glPointParameterfProc)
+                SGLookupFunction("glPointParameterfARB");
+            glPointParameterfvPtr = (glPointParameterfvProc)
+                SGLookupFunction("glPointParameterfvARB");
+        } else {
+            glPointParameterIsSupported = false;
         }
         fgSplashProgress("reading airport & navigation data");
 
@@ -712,14 +785,15 @@ static void fgIdleFunction ( void ) {
         // based on the requested presets, calculate the true starting
         // lon, lat
         fgInitPosition();
+        fgInitTowerLocationListener();
 
         SGTime *t = fgInitTime();
         globals->set_time_params( t );
 
         // Do some quick general initializations
         if( !fgInitGeneral()) {
-            SG_LOG( SG_GENERAL, SG_ALERT, 
-                "General initializations failed ..." );
+            SG_LOG( SG_GENERAL, SG_ALERT,
+                "General initialization failed ..." );
             exit(-1);
         }
 
@@ -787,13 +861,6 @@ static void fgIdleFunction ( void ) {
 
         // TODO: move to environment mgr
         thesky = new SGSky;
-        SGPath texture_path(globals->get_fg_root());
-        texture_path.append("Textures");
-        texture_path.append("Sky");
-        for (int i = 0; i < FGEnvironmentMgr::MAX_CLOUD_LAYERS; i++) {
-            SGCloudLayer * layer = new SGCloudLayer(texture_path.str());
-            thesky->add_cloud_layer(layer);
-        }
 
         SGPath sky_tex_path( globals->get_fg_root() );
         sky_tex_path.append( "Textures" );
@@ -801,16 +868,26 @@ static void fgIdleFunction ( void ) {
         thesky->texture_path( sky_tex_path.str() );
 
         // The sun and moon diameters are scaled down numbers of the
-        // actual diameters. This was needed to fit bot the sun and the
+        // actual diameters. This was needed to fit both the sun and the
         // moon within the distance to the far clip plane.
         // Moon diameter:    3,476 kilometers
         // Sun diameter: 1,390,000 kilometers
         thesky->build( 80000.0, 80000.0,
                        463.3, 361.8,
-                       globals->get_ephem()->getNumPlanets(), 
+                       globals->get_ephem()->getNumPlanets(),
                        globals->get_ephem()->getPlanets(),
                        globals->get_ephem()->getNumStars(),
-                       globals->get_ephem()->getStars() );
+                       globals->get_ephem()->getStars(),
+                       fgGetNode("/environment", true));
+
+
+        SGPath texture_path(globals->get_fg_root());
+        texture_path.append("Textures");
+        texture_path.append("Sky");
+        for (int i = 0; i < FGEnvironmentMgr::MAX_CLOUD_LAYERS; i++) {
+            SGCloudLayer * layer = new SGCloudLayer(texture_path.str());
+            thesky->add_cloud_layer(layer);
+        }
 
         // Initialize MagVar model
         SGMagVar *magvar = new SGMagVar();
@@ -830,6 +907,8 @@ static void fgIdleFunction ( void ) {
                                     globals->get_time_params()->getJD() );
         double var = globals->get_mag()->get_magvar() * SGD_RADIANS_TO_DEGREES;
         fgSetDouble("/instrumentation/heading-indicator/offset-deg", -var);
+        fgSetDouble("/instrumentation/heading-indicator-fg/offset-deg", -var);
+
 
         // airport = new ssgBranch;
         // airport->setName( "Airport Lighting" );
@@ -850,7 +929,7 @@ static void fgIdleFunction ( void ) {
             SGPath mp3file( globals->get_fg_root() );
             mp3file.append( "Sounds/intro.mp3" );
 
-            SG_LOG( SG_GENERAL, SG_INFO, 
+            SG_LOG( SG_GENERAL, SG_INFO,
                 "Starting intro music: " << mp3file.str() );
 
 #if defined( __CYGWIN__ )
@@ -876,11 +955,11 @@ static void fgIdleFunction ( void ) {
 
         // This is the top level init routine which calls all the
         // other subsystem initialization routines.  If you are adding
-        // a subsystem to flightgear, its initialization call should
+        // a subsystem to flightgear, its initialization call should be
         // located in this routine.
         if( !fgInitSubsystems()) {
             SG_LOG( SG_GENERAL, SG_ALERT,
-                "Subsystem initializations failed ..." );
+                "Subsystem initialization failed ..." );
             exit(-1);
         }
         fgSplashProgress("setting up time & renderer");
@@ -956,7 +1035,7 @@ bool fgMainInit( int argc, char **argv ) {
 
     globals = new FGGlobals;
 
-    // seed the random number generater
+    // seed the random number generator
     sg_srandom_time();
 
     FGControls *controls = new FGControls;
@@ -974,7 +1053,7 @@ bool fgMainInit( int argc, char **argv ) {
     fgInitFGRoot(argc, argv);
 
     // Check for the correct base package version
-    static char required_version[] = "0.9.10";
+    static char required_version[] = "1.0.0";
     string base_version = fgBasePackageVersion();
     if ( !(base_version == required_version) ) {
         // tell the operator how to use this application
@@ -994,7 +1073,7 @@ bool fgMainInit( int argc, char **argv ) {
     sgUseDisplayList = fgGetBool( "/sim/rendering/use-display-list", true );
 
     // Load the configuration parameters.  (Command line options
-    // overrides config file options.  Config file options override
+    // override config file options.  Config file options override
     // defaults.)
     if ( !fgInitConfig(argc, argv) ) {
         SG_LOG( SG_GENERAL, SG_ALERT, "Config option parsing failed ..." );
