@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: options.cxx,v 1.82 2006-03-21 14:53:38 fredb Exp $
+// $Id: options.cxx,v 1.89.2.8 2007-11-02 18:45:59 curt Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -98,7 +98,6 @@ atoi( const string& str )
 #endif
 }
 
-
 /**
  * Set a few fail-safe default property values.
  *
@@ -126,8 +125,8 @@ fgSetDefaults ()
     fgSetDouble("/position/altitude-ft", -9999.0);
 
 				// Orientation
-    fgSetDouble("/orientation/heading-deg", 270);
-    fgSetDouble("/orientation/roll-deg", 0);
+    fgSetDouble("/orientation/heading-deg", 9999.0);
+    fgSetDouble("/orientation/roll-deg", 0.0);
     fgSetDouble("/orientation/pitch-deg", 0.424);
 
 				// Velocities
@@ -145,8 +144,8 @@ fgSetDefaults ()
     fgSetDouble("/sim/presets/latitude-deg", 9999.0);
     fgSetDouble("/sim/presets/altitude-ft", -9999.0);
 
-    fgSetDouble("/sim/presets/heading-deg", 270);
-    fgSetDouble("/sim/presets/roll-deg", 0);
+    fgSetDouble("/sim/presets/heading-deg", 9999.0);
+    fgSetDouble("/sim/presets/roll-deg", 0.0);
     fgSetDouble("/sim/presets/pitch-deg", 0.424);
 
     fgSetString("/sim/presets/speed-set", "knots");
@@ -185,7 +184,7 @@ fgSetDefaults ()
     fgSetString("/sim/logging/priority", "alert");
 
 				// Features
-    fgSetBool("/sim/hud/antialiased", false);
+    fgSetBool("/sim/hud/color/antialiased", false);
     fgSetBool("/sim/hud/enable3d", true);
     fgSetBool("/sim/hud/visibility", false);
     fgSetBool("/sim/panel/visibility", true);
@@ -504,7 +503,7 @@ parse_fov( const string& arg ) {
 //
 // Format is "--protocol=medium,direction,hz,medium_options,..."
 //
-//   protocol = { native, nmea, garmin, fgfs, rul, pve, etc. }
+//   protocol = { native, nmea, garmin, AV400, fgfs, rul, pve, etc. }
 //   medium = { serial, socket, file, etc. }
 //   direction = { in, out, bi }
 //   hz = number of times to process channel per second (floating
@@ -528,64 +527,62 @@ parse_fov( const string& arg ) {
 static bool
 add_channel( const string& type, const string& channel_str ) {
     SG_LOG(SG_GENERAL, SG_INFO, "Channel string = " << channel_str );
-
     globals->get_channel_options_list()->push_back( type + "," + channel_str );
-    
-    // cout << "here" << endl;
-
     return true;
 }
 
-// The parse wp and parse flight-plan options don't work anymore, because 
+
+// The parse wp and parse flight-plan options don't work anymore, because
 // the route manager and the airport subsystems have not yet been initialized
-// at this stage. 
+// at this stage.
 
 // Parse --wp=ID[@alt]
-static void 
+static void
 parse_wp( const string& arg ) {
     string_list *waypoints = globals->get_initial_waypoints();
     if (!waypoints) {
         waypoints = new string_list;
+        globals->set_initial_waypoints(waypoints);
     }
     waypoints->push_back(arg);
-    globals->set_initial_waypoints(waypoints);
 }
 
 
 // Parse --flight-plan=[file]
-static bool 
+static bool
 parse_flightplan(const string& arg)
 {
-  string_list *waypoints = globals->get_initial_waypoints();
-  if (!waypoints)
-    waypoints = new string_list;
-  sg_gzifstream in(arg.c_str());
-  if ( !in.is_open() ) {
-    return false;
-  }
-  while ( true ) {
-    string line;
-    
+    string_list *waypoints = globals->get_initial_waypoints();
+    if (!waypoints) {
+        waypoints = new string_list;
+        globals->set_initial_waypoints(waypoints);
+    }
+
+    sg_gzifstream in(arg.c_str());
+    if ( !in.is_open() )
+        return false;
+
+    while ( true ) {
+        string line;
+
 #if defined( macintosh )
-    getline( in, line, '\r' );
+        getline( in, line, '\r' );
 #else
-    getline( in, line, '\n' );
+        getline( in, line, '\n' );
 #endif
-    
-    // catch extraneous (DOS) line ending character
-    if ( line[line.length() - 1] < 32 ) {
-      line = line.substr( 0, line.length()-1 );
+        // catch extraneous (DOS) line ending character
+        if ( line[line.length() - 1] < 32 )
+            line = line.substr( 0, line.length()-1 );
+
+        if ( in.eof() )
+            break;
+
+        waypoints->push_back(line);
     }
-    
-    if ( in.eof() ) {
-      break;
-    }
-    waypoints->push_back(line);
-  }
-  globals->set_initial_waypoints(waypoints);
-  return true;
+    return true;
 }
 
+
 static int
 fgOptLanguage( const char *arg )
 {
@@ -1163,6 +1160,32 @@ fgOptLivery( const char *arg )
     return FG_OPTIONS_OK;
 }
 
+static int
+fgOptScenario( const char *arg )
+{
+    SGPropertyNode_ptr ai_node = fgGetNode( "/sim/ai", true );
+    vector<SGPropertyNode_ptr> scenarii = ai_node->getChildren( "scenario" );
+    int index = -1;
+    for ( size_t i = 0; i < scenarii.size(); ++i ) {
+        int ind = scenarii[i]->getIndex();
+        if ( index < ind ) {
+            index = ind;
+        }
+    }
+    SGPropertyNode_ptr scenario = ai_node->getNode( "scenario", index + 1, true );
+    scenario->setStringValue( arg );
+    ai_node->setBoolValue( "enabled", true );
+    return FG_OPTIONS_OK;
+}
+
+static int
+fgOptRunway( const char *arg )
+{
+    fgSetString("/sim/presets/runway", arg );
+    fgSetBool("/sim/presets/runway-requested", true );
+    return FG_OPTIONS_OK;
+}
+
 static map<string,size_t> fgOptionMap;
 
 /*
@@ -1226,8 +1249,8 @@ struct OptionDesc {
     {"enable-clock-freeze",          false, OPTION_BOOL,   "/sim/freeze/clock", true, "", 0 },
     {"disable-hud-3d",               false, OPTION_BOOL,   "/sim/hud/enable3d", false, "", 0 },
     {"enable-hud-3d",                false, OPTION_BOOL,   "/sim/hud/enable3d", true, "", 0 },
-    {"disable-anti-alias-hud",       false, OPTION_BOOL,   "/sim/hud/antialiased", false, "", 0 },
-    {"enable-anti-alias-hud",        false, OPTION_BOOL,   "/sim/hud/antialiased", true, "", 0 },
+    {"disable-anti-alias-hud",       false, OPTION_BOOL,   "/sim/hud/color/antialiased", false, "", 0 },
+    {"enable-anti-alias-hud",        false, OPTION_BOOL,   "/sim/hud/color/antialiased", true, "", 0 },
     {"control",                      true,  OPTION_STRING, "/sim/control-mode", false, "", 0 },
     {"disable-auto-coordination",    false, OPTION_BOOL,   "/sim/auto-coordination", false, "", 0 },
     {"enable-auto-coordination",     false, OPTION_BOOL,   "/sim/auto-coordination", true, "", 0 },
@@ -1240,7 +1263,7 @@ struct OptionDesc {
     {"enable-sound",                 false, OPTION_BOOL,   "/sim/sound/pause", false, "", 0 },
     {"airport",                      true,  OPTION_STRING, "/sim/presets/airport-id", false, "", 0 },
     {"airport-id",                   true,  OPTION_STRING, "/sim/presets/airport-id", false, "", 0 },
-    {"runway",                       true,  OPTION_STRING, "/sim/presets/runway", false, "", 0 },
+    {"runway",                       true,  OPTION_FUNC,   "", false, "", fgOptRunway },
     {"vor",                          true,  OPTION_FUNC,   "", false, "", fgOptVOR },
     {"ndb",                          true,  OPTION_FUNC,   "", false, "", fgOptNDB },
     {"carrier",                      true,  OPTION_FUNC,   "", false, "", fgOptCarrier },
@@ -1329,6 +1352,7 @@ struct OptionDesc {
     {"native-fdm",                   true,  OPTION_CHANNEL, "", false, "", 0 },
     {"native-gui",                   true,  OPTION_CHANNEL, "", false, "", 0 },
     {"opengc",                       true,  OPTION_CHANNEL, "", false, "", 0 },
+    {"AV400",                        true,  OPTION_CHANNEL, "", false, "", 0 },
     {"garmin",                       true,  OPTION_CHANNEL, "", false, "", 0 },
     {"nmea",                         true,  OPTION_CHANNEL, "", false, "", 0 },
     {"generic",                      true,  OPTION_CHANNEL, "", false, "", 0 },
@@ -1356,6 +1380,7 @@ struct OptionDesc {
     {"flight-plan",                  true,  OPTION_FUNC,   "", false, "", fgOptFlightPlan },
     {"config",                       true,  OPTION_FUNC,   "", false, "", fgOptConfig },
     {"aircraft",                     true,  OPTION_STRING, "/sim/aircraft", false, "", 0 },
+    {"vehicle",                      true,  OPTION_STRING, "/sim/aircraft", false, "", 0 },
     {"failure",                      true,  OPTION_FUNC,   "", false, "", fgOptFailure },
     {"com1",                         true,  OPTION_DOUBLE, "/instrumentation/comm[0]/frequencies/selected-mhz", false, "", 0 },
     {"com2",                         true,  OPTION_DOUBLE, "/instrumentation/comm[1]/frequencies/selected-mhz", false, "", 0 },
@@ -1365,10 +1390,51 @@ struct OptionDesc {
     {"dme",                          true,  OPTION_FUNC,   "", false, "", fgOptDME },
     {"min-status",                   true,  OPTION_STRING,  "/sim/aircraft-min-status", false, "all", 0 },
     {"livery",                       true,  OPTION_FUNC,   "", false, "", fgOptLivery },
+    {"ai-scenario",                  true,  OPTION_FUNC,   "", false, "", fgOptScenario },
     {0}
 };
 
 
+// Set a property for the --prop: option. Syntax: --prop:[<type>:]<name>=<value>
+// <type> can be "double" etc. but also only the first letter "d".
+// Examples:  --prop:alpha=1  --prop:bool:beta=true  --prop:d:gamma=0.123
+static bool
+set_property(const string& arg)
+{
+    string::size_type pos = arg.find('=');
+    if (pos == arg.npos || pos == 0 || pos + 1 == arg.size())
+        return false;
+
+    string name = arg.substr(0, pos);
+    string value = arg.substr(pos + 1);
+    string type;
+    pos = name.find(':');
+
+    if (pos != name.npos && pos != 0 && pos + 1 != name.size()) {
+        type = name.substr(0, pos);
+        name = name.substr(pos + 1);
+    }
+    SGPropertyNode *n = fgGetNode(name.c_str(), true);
+
+    if (type.empty())
+        return n->setUnspecifiedValue(value.c_str());
+    else if (type == "s" || type == "string")
+        return n->setStringValue(value.c_str());
+    else if (type == "d" || type == "double")
+        return n->setDoubleValue(strtod(value.c_str(), 0));
+    else if (type == "f" || type == "float")
+        return n->setFloatValue(atof(value.c_str()));
+    else if (type == "l" || type == "long")
+        return n->setLongValue(strtol(value.c_str(), 0, 0));
+    else if (type == "i" || type == "int")
+        return n->setIntValue(atoi(value.c_str()));
+    else if (type == "b" || type == "bool")
+        return n->setBoolValue(value == "true" || atoi(value.c_str()) != 0);
+
+    return false;
+}
+
+
 // Parse a single option
 static int
 parse_option (const string& arg)
@@ -1393,15 +1459,9 @@ parse_option (const string& arg)
     } else if ( arg.find( "--show-aircraft") == 0) {
         return(FG_OPTIONS_SHOW_AIRCRAFT);
     } else if ( arg.find( "--prop:" ) == 0 ) {
-        string assign = arg.substr(7);
-	string::size_type pos = assign.find('=');
-	if ( pos == arg.npos || pos == 0 || pos + 1 == assign.size() ) {
-	    SG_LOG( SG_GENERAL, SG_ALERT, "Bad property assignment: " << arg );
+        if (!set_property(arg.substr(7))) {
+            SG_LOG( SG_GENERAL, SG_ALERT, "Bad property assignment: " << arg );
             return FG_OPTIONS_ERROR;
-        } else {
-	    string name = assign.substr(0, pos);
-	    string value = assign.substr(pos + 1);
-	    fgSetString(name.c_str(), value.c_str());
         }
     } else if ( arg.find( "--" ) == 0 ) {
         size_t pos = arg.find( '=' );
@@ -1594,7 +1654,7 @@ fgUsage (bool verbose)
 
     try {
         fgLoadProps("options.xml", &options_root);
-    } catch (const sg_exception &ex) {
+    } catch (const sg_exception &) {
         cout << "Unable to read the help file." << endl;
         cout << "Make sure the file options.xml is located in the FlightGear base directory," << endl;
         cout << "and the location of the base directory is specified by setting $FG_ROOT or" << endl;
@@ -1626,7 +1686,7 @@ fgUsage (bool verbose)
             SGPropertyNode *short_name = option[k]->getNode("short");
             SGPropertyNode *key = option[k]->getNode("key");
             SGPropertyNode *arg = option[k]->getNode("arg");
-            bool brief = option[k]->getNode("brief");
+            bool brief = option[k]->getNode("brief") != 0;
 
             if ((brief || verbose) && name) {
                 string tmp = name->getStringValue();
