@@ -19,7 +19,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: renderer.cxx,v 1.46 2006-04-01 10:50:52 frohlich Exp $
+// $Id: renderer.cxx,v 1.53.2.1 2007-05-02 18:49:34 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -37,14 +37,15 @@
 
 #include <simgear/screen/extensions.hxx>
 #include <simgear/scene/material/matlib.hxx>
+#include <simgear/scene/model/animation.hxx>
+#include <simgear/scene/model/model.hxx>
+#include <simgear/scene/model/modellib.hxx>
+#include <simgear/scene/model/placement.hxx>
+#include <simgear/scene/tgdb/pt_lights.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/timing/sg_time.hxx>
-#include <simgear/scene/model/animation.hxx>
 #include <simgear/ephemeris/ephemeris.hxx>
-#include <simgear/scene/model/placement.hxx>
 #include <simgear/math/sg_random.h>
-#include <simgear/scene/model/modellib.hxx>
-#include <simgear/scene/model/model.hxx>
 #ifdef FG_JPEG_SERVER
 #include <simgear/screen/jpgfactory.hxx>
 #endif
@@ -68,6 +69,8 @@
 #include <Scenery/tilemgr.hxx>
 #include <ATC/ATCdisplay.hxx>
 #include <GUI/new_gui.hxx>
+#include <Instrumentation/instrument_mgr.hxx>
+#include <Instrumentation/HUD/HUD.hxx>
 
 #include "splash.hxx"
 #include "renderer.hxx"
@@ -231,8 +234,12 @@ FGRenderer::update( bool refresh_camera_settings ) {
 
     bool draw_otw = fgGetBool("/sim/rendering/draw-otw");
     bool skyblend = fgGetBool("/sim/rendering/skyblend");
+    bool use_point_sprites = fgGetBool("/sim/rendering/point-sprites");
     bool enhanced_lighting = fgGetBool("/sim/rendering/enhanced-lighting");
-    bool distance_attenuation = fgGetBool("/sim/rendering/distance-attenuation");
+    bool distance_attenuation
+        = fgGetBool("/sim/rendering/distance-attenuation");
+    sgConfigureDirectionalLights( use_point_sprites, enhanced_lighting,
+                                  distance_attenuation );
     bool volumetric_clouds = sgEnviro.get_clouds_enable_state();
 #ifdef FG_ENABLE_MULTIPASS_CLOUDS
     bool multi_pass_clouds = fgGetBool("/sim/rendering/multi-pass-clouds") && 
@@ -298,10 +305,6 @@ FGRenderer::update( bool refresh_camera_settings ) {
     // now work without seg faulting the system.
 
     FGViewer *current__view = globals->get_current_view();
-
-    // calculate our current position in cartesian space
-    Point3D cntr = globals->get_scenery()->get_next_center();
-    globals->get_scenery()->set_center(cntr);
     // Force update of center dependent values ...
     current__view->set_dirty();
 
@@ -355,65 +358,17 @@ FGRenderer::update( bool refresh_camera_settings ) {
 
     // update the sky dome
     if ( skyblend ) {
-        /*
-         SG_LOG( SG_GENERAL, SG_BULK, "thesky->repaint() sky_color = "
-         << l->sky_color()[0] << " "
-         << l->sky_color()[1] << " "
-         << l->sky_color()[2] << " "
-         << l->sky_color()[3] );
-        SG_LOG( SG_GENERAL, SG_BULK, "    fog = "
-         << l->fog_color()[0] << " "
-         << l->fog_color()[1] << " "
-         << l->fog_color()[2] << " "
-         << l->fog_color()[3] ); 
-        SG_LOG( SG_GENERAL, SG_BULK,
-                "    sun_angle = " << l->sun_angle
-         << "    moon_angle = " << l->moon_angle );
-        */
-
-        static SGSkyColor scolor;
-//        FGLight *l = (FGLight *)(globals->get_subsystem("lighting"));
-
-        scolor.sky_color   = l->sky_color();
-        scolor.fog_color   = l->adj_fog_color();
-        scolor.cloud_color = l->cloud_color();
-        scolor.sun_angle   = l->get_sun_angle();
-        scolor.moon_angle  = l->get_moon_angle();
-        scolor.nplanets    = globals->get_ephem()->getNumPlanets();
-        scolor.nstars      = globals->get_ephem()->getNumStars();
-        scolor.planet_data = globals->get_ephem()->getPlanets();
-        scolor.star_data   = globals->get_ephem()->getStars();
-
-        thesky->repaint( scolor );
-
-        /*
-        SG_LOG( SG_GENERAL, SG_BULK,
-                "thesky->reposition( view_pos = " << view_pos[0] << " "
-         << view_pos[1] << " " << view_pos[2] );
-        SG_LOG( SG_GENERAL, SG_BULK,
-                "    zero_elev = " << zero_elev[0] << " "
-         << zero_elev[1] << " " << zero_elev[2]
-         << " lon = " << cur_fdm_state->get_Longitude()
-         << " lat = " << cur_fdm_state->get_Latitude() );
-        SG_LOG( SG_GENERAL, SG_BULK,
-                "    sun_rot = " << l->get_sun_rotation
-         << " gst = " << SGTime::cur_time_params->getGst() );
-        SG_LOG( SG_GENERAL, SG_BULK,
-             "    sun ra = " << globals->get_ephem()->getSunRightAscension()
-          << " sun dec = " << globals->get_ephem()->getSunDeclination()
-          << " moon ra = " << globals->get_ephem()->getMoonRightAscension()
-          << " moon dec = " << globals->get_ephem()->getMoonDeclination() );
-        */
 
         // The sun and moon distances are scaled down versions
         // of the actual distance to get both the moon and the sun
         // within the range of the far clip plane.
         // Moon distance:    384,467 kilometers
         // Sun distance: 150,000,000 kilometers
+
         double sun_horiz_eff, moon_horiz_eff;
         if (fgGetBool("/sim/rendering/horizon-effect")) {
-        sun_horiz_eff = 0.67+pow(0.5+cos(l->get_sun_angle())*2/2, 0.33)/3;
-        moon_horiz_eff = 0.67+pow(0.5+cos(l->get_moon_angle())*2/2, 0.33)/3;
+           sun_horiz_eff = 0.67+pow(0.5+cos(l->get_sun_angle())*2/2, 0.33)/3;
+           moon_horiz_eff = 0.67+pow(0.5+cos(l->get_moon_angle())*2/2, 0.33)/3;
         } else {
            sun_horiz_eff = moon_horiz_eff = 1.0;
         }
@@ -437,16 +392,68 @@ FGRenderer::update( bool refresh_camera_settings ) {
         sstate.moon_ra   = globals->get_ephem()->getMoonRightAscension();
         sstate.moon_dec  = globals->get_ephem()->getMoonDeclination();
         sstate.moon_dist = 40000.0 * moon_horiz_eff;
+        sstate.sun_angle = l->get_sun_angle();
+
+
+        /*
+         SG_LOG( SG_GENERAL, SG_BULK, "thesky->repaint() sky_color = "
+         << l->sky_color()[0] << " "
+         << l->sky_color()[1] << " "
+         << l->sky_color()[2] << " "
+         << l->sky_color()[3] );
+        SG_LOG( SG_GENERAL, SG_BULK, "    fog = "
+         << l->fog_color()[0] << " "
+         << l->fog_color()[1] << " "
+         << l->fog_color()[2] << " "
+         << l->fog_color()[3] );
+        SG_LOG( SG_GENERAL, SG_BULK,
+                "    sun_angle = " << l->sun_angle
+         << "    moon_angle = " << l->moon_angle );
+        */
+
+        static SGSkyColor scolor;
+//        FGLight *l = (FGLight *)(globals->get_subsystem("lighting"));
+
+        scolor.sky_color   = l->sky_color();
+        scolor.fog_color   = l->adj_fog_color();
+        scolor.cloud_color = l->cloud_color();
+        scolor.sun_angle   = l->get_sun_angle();
+        scolor.moon_angle  = l->get_moon_angle();
+        scolor.nplanets    = globals->get_ephem()->getNumPlanets();
+        scolor.nstars      = globals->get_ephem()->getNumStars();
+        scolor.planet_data = globals->get_ephem()->getPlanets();
+        scolor.star_data   = globals->get_ephem()->getStars();
 
         thesky->reposition( sstate, delta_time_sec );
+        thesky->repaint( scolor );
 
-        shadows->setupShadows( 
+        /*
+        SG_LOG( SG_GENERAL, SG_BULK,
+                "thesky->reposition( view_pos = " << view_pos[0] << " "
+         << view_pos[1] << " " << view_pos[2] );
+        SG_LOG( SG_GENERAL, SG_BULK,
+                "    zero_elev = " << zero_elev[0] << " "
+         << zero_elev[1] << " " << zero_elev[2]
+         << " lon = " << cur_fdm_state->get_Longitude()
+         << " lat = " << cur_fdm_state->get_Latitude() );
+        SG_LOG( SG_GENERAL, SG_BULK,
+                "    sun_rot = " << l->get_sun_rotation
+         << " gst = " << SGTime::cur_time_params->getGst() );
+        SG_LOG( SG_GENERAL, SG_BULK,
+             "    sun ra = " << globals->get_ephem()->getSunRightAscension()
+          << " sun dec = " << globals->get_ephem()->getSunDeclination()
+          << " moon ra = " << globals->get_ephem()->getMoonRightAscension()
+          << " moon dec = " << globals->get_ephem()->getMoonDeclination() );
+        */
+
+        shadows->setupShadows(
           current__view->getLongitude_deg(),
           current__view->getLatitude_deg(),
           globals->get_time_params()->getGst(),
           globals->get_ephem()->getSunRightAscension(),
           globals->get_ephem()->getSunDeclination(),
           l->get_sun_angle());
+
     }
 
     glEnable( GL_DEPTH_TEST );
@@ -581,6 +588,12 @@ FGRenderer::update( bool refresh_camera_settings ) {
     // CLO - 02/25/2005 - DO WE NEED THIS extra fgSetNearFar()?
     // fgSetNearFar( scene_nearplane, scene_farplane );
 
+    if ( use_point_sprites ) {
+        glEnable(GL_POINT_SPRITE);
+    } else {
+        glDisable(GL_POINT_SPRITE);
+    }
+
     if ( enhanced_lighting ) {
 
         // Enable states for drawing points with GL_extension
@@ -594,7 +607,7 @@ FGRenderer::update( bool refresh_camera_settings ) {
             float quadratic[3] = {1.0, 0.001, 0.0000001};
             // makes the points fade as they move away
             glPointParameterfvPtr(GL_DISTANCE_ATTENUATION_EXT, quadratic);
-            glPointParameterfPtr(GL_POINT_SIZE_MIN_EXT, 1.0); 
+            // glPointParameterfPtr(GL_POINT_SIZE_MIN_EXT, 1.0); 
         }
 
         glPointSize(4.0);
@@ -729,6 +742,10 @@ FGRenderer::update( bool refresh_camera_settings ) {
     hud_and_panel->apply();
     fgCockpitUpdate();
 
+    FGInstrumentMgr *instr = static_cast<FGInstrumentMgr *>(globals->get_subsystem("instrumentation"));
+    HUD *hud = static_cast<HUD *>(instr->get_subsystem("hud"));
+    hud->draw();
+
     // Use the hud_and_panel ssgSimpleState for rendering the ATC output
     // This only works properly if called before the panel call
     if((fgGetBool("/sim/atc/enabled")) || (fgGetBool("/sim/ai-traffic/enabled")))
@@ -753,6 +770,8 @@ FGRenderer::update( bool refresh_camera_settings ) {
     double t = globals->get_sim_time_sec();
     if (t <= 2.5)
         fgSplashUpdate((2.5 - t) / 2.5);
+    else
+        fgSplashExit();
 }
 
 
@@ -773,8 +792,8 @@ FGRenderer::resize( int width, int height ) {
 
     glViewport( 0, (GLint)(height - view_h), (GLint)(width), (GLint)(view_h) );
 
-    static int lastwidth = width;
-    static int lastheight = height;
+    static int lastwidth = 0;
+    static int lastheight = 0;
     if (width != lastwidth)
         fgSetInt("/sim/startup/xsize", lastwidth = width);
     if (height != lastheight)
@@ -913,7 +932,8 @@ void FGRenderer::setNearFar( float n, float f ) {
     fgHackFrustum();
 }
 
-bool FGRenderer::getPickInfo( sgdVec3 pt, sgdVec3 dir, unsigned x, unsigned y )
+bool FGRenderer::getPickInfo( SGVec3d& pt, SGVec3d& dir,
+                              unsigned x, unsigned y )
 {
   // Get the matrices involved in the transform from global to screen
   // coordinates.
@@ -949,11 +969,11 @@ bool FGRenderer::getPickInfo( sgdVec3 pt, sgdVec3 dir, unsigned x, unsigned y )
   FGViewer* view = globals->get_current_view();
   sgVec4 fDir;
   sgSubVec3(fDir, worldPt, view->get_view_pos());
-  sgdSetVec3(dir, fDir);
-  sgdNormalizeVec3(dir);
+  sgdSetVec3(dir.sg(), fDir);
+  sgdNormalizeVec3(dir.sg());
 
   // Copy the start point
-  sgdCopyVec3(pt, view->get_absolute_view_pos());
+  sgdCopyVec3(pt.sg(), view->get_absolute_view_pos());
 
   return true;
 }
