@@ -19,7 +19,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: splash.cxx,v 1.21 2006-02-21 01:19:20 mfranz Exp $
+// $Id: splash.cxx,v 1.26.2.5 2007-05-12 12:39:03 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -54,14 +54,9 @@
 #include "fg_os.hxx"
 #include "renderer.hxx"
 
-static const int fontsize = 19;
-static const char fontname[] = "default.txf";
 static const char *progress_text = 0;
-
-
-static SGTexture splash;
-static fntTexFont font;
-static fntRenderer info;
+static SGTexture *splash = new SGTexture;
+SGPropertyNode_ptr style = 0;
 
 
 // Initialize the splash screen
@@ -70,30 +65,12 @@ void fgSplashInit ( const char *splash_texture ) {
 
     SG_LOG( SG_GENERAL, SG_INFO, "Initializing splash screen" );
 
-
-    SGPath fontpath;
-    char* envp = ::getenv("FG_FONTS");
-    if (envp != NULL) {
-        fontpath.set(envp);
-    } else {
-        fontpath.set(globals->get_fg_root());
-        fontpath.append("Fonts");
-    }
-    SGPath path(fontpath);
-    path.append(fontname);
-
-    if (!font.load((char *)path.c_str())) {
-        SG_LOG( SG_GENERAL, SG_ALERT, "Error loading font " << path.str() );
-        return;
-    }
-
-    info.setFont(&font);
-    info.setPointSize(fontsize);
+    style = fgGetNode("/sim/gui/style[0]", true);
 
     if (!fgGetBool("/sim/startup/splash-screen"))
         return;
 
-    splash.bind();
+    splash->bind();
 
     SGPath tpath( globals->get_fg_root() );
     if (splash_texture == NULL || !strcmp(splash_texture, "")) {
@@ -108,15 +85,15 @@ void fgSplashInit ( const char *splash_texture ) {
     } else
         tpath.append( splash_texture );
 
-    splash.read_rgb_texture(tpath.c_str());
-    if (!splash.usable())
+    splash->read_rgba_texture(tpath.c_str());
+    if (!splash->usable())
     {
         // Try compressed
         SGPath fg_tpath = tpath;
         fg_tpath.concat( ".gz" );
 
-        splash.read_rgb_texture(fg_tpath.c_str());
-        if ( !splash.usable() )
+        splash->read_rgba_texture(fg_tpath.c_str());
+        if ( !splash->usable() )
         {
             SG_LOG( SG_GENERAL, SG_ALERT,
                     "Error in loading splash screen texture " << tpath.str() );
@@ -124,7 +101,14 @@ void fgSplashInit ( const char *splash_texture ) {
         }
     }
 
-    splash.select();
+    splash->select();
+}
+
+
+void fgSplashExit ()
+{
+    delete splash;
+    splash = 0;
 }
 
 
@@ -137,6 +121,8 @@ void fgSplashProgress ( const char *s )
 
 // Update the splash screen with alpha specified from 0.0 to 1.0
 void fgSplashUpdate ( float alpha ) {
+    const int EMPTYSPACE = 80;
+
     int screen_width = fgGetInt("/sim/startup/xsize", 0);
     int screen_height = fgGetInt("/sim/startup/ysize", 0);
 
@@ -144,8 +130,8 @@ void fgSplashUpdate ( float alpha ) {
         return;
 
     globals->get_renderer()->resize(screen_width, screen_height);
-    int size = screen_width < (screen_height - 5 * fontsize)
-            ? screen_width : screen_height - 5 * fontsize;
+    int size = screen_width < (screen_height - EMPTYSPACE)
+            ? screen_width : screen_height - EMPTYSPACE;
     if (size > 512)
         size = 512;
 
@@ -160,16 +146,18 @@ void fgSplashUpdate ( float alpha ) {
     glPushMatrix();
     glLoadIdentity();
     gluOrtho2D(0, screen_width, 0, screen_height);
+    glViewport(0, 0, screen_width, screen_height);
     glMatrixMode(GL_MODELVIEW);
     glPushMatrix();
     glLoadIdentity();
 
     glDisable(GL_DEPTH_TEST);
     glDisable(GL_LIGHTING);
+    glDisable(GL_CULL_FACE);
 
     // draw the background
     FGColor c(0.0, 0.0, 0.0);
-    c.merge(fgGetNode("/sim/gui/colors/splash-screen"));
+    c.merge(style->getNode("colors/splash-screen"));
     glColor4f(c.red(), c.green(), c.blue(), alpha );
     glBegin(GL_POLYGON);
     glVertex2f(0.0, 0.0);
@@ -178,10 +166,15 @@ void fgSplashUpdate ( float alpha ) {
     glVertex2f(0.0, screen_height);
     glEnd();
 
+    glEnable(GL_ALPHA_TEST);
+    glEnable(GL_BLEND);
+    glAlphaFunc(GL_GREATER, 0.1f);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
     // now draw the logo
-    if (fgGetBool("/sim/startup/splash-screen", true)) {
+    if (splash && fgGetBool("/sim/startup/splash-screen", true)) {
         glEnable(GL_TEXTURE_2D);
-        splash.bind();
+        splash->bind();
         glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
         glColor4f( 1.0, 1.0, 1.0, alpha );
@@ -191,36 +184,28 @@ void fgSplashUpdate ( float alpha ) {
         glTexCoord2f(1.0, 1.0); glVertex2f(xmax, ymax);
         glTexCoord2f(0.0, 1.0); glVertex2f(xmin, ymax);
         glEnd();
+        glDisable(GL_TEXTURE_2D);
     }
 
-    if (info.getFont() && progress_text && fgGetBool("/sim/startup/splash-progress", true)) {
-        glEnable(GL_ALPHA_TEST);
-        glEnable(GL_BLEND);
-        glAlphaFunc(GL_GREATER, 0.1f);
-        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-        glDisable(GL_CULL_FACE);
+    if (progress_text && fgGetBool("/sim/startup/splash-progress", true)) {
+        puFont *fnt = globals->get_fontcache()->get(style->getNode("fonts/splash"));
 
-        float left, right, bot, top;
-
-        info.begin();
         FGColor c(1.0, 0.9, 0.0);
-        c.merge(fgGetNode("/sim/gui/colors/splash-font"));
+        c.merge(style->getNode("colors/splash-font"));
         glColor4f(c.red(), c.green(), c.blue(), alpha);
 
-        font.getBBox(progress_text, fontsize, 0, &left, &right, &bot, &top);
-        info.start2f((screen_width - right) / 2.0, 10.0 - bot);
-        info.puts(progress_text);
+        float height = fnt->getStringHeight("/M$");
+        float descender = fnt->getStringDescender();
+        float width = fnt->getFloatStringWidth(progress_text);
+        fnt->drawString(progress_text, int((screen_width - width) / 2), int(10 + descender));
 
-        const char *s = fgGetString("/sim/startup/splash-title", "");
-        font.getBBox(s, fontsize, 0, &left, &right, &bot, &top);
-        info.start2f((screen_width - right) / 2.0, screen_height - top - bot - 10.0);
-        info.puts(s);
-        info.end();
+        const char *title = fgGetString("/sim/startup/splash-title", "");
+        width = fnt->getFloatStringWidth(title);
+        fnt->drawString(title, int((screen_width - width) / 2), int(screen_height - 10 - height));
     }
 
     glEnable(GL_DEPTH_TEST);
     glEnable(GL_LIGHTING);
-    glDisable(GL_TEXTURE_2D);
 
     glMatrixMode(GL_PROJECTION);
     glPopMatrix();
