@@ -21,7 +21,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: viewer.cxx,v 1.22 2006-02-21 01:19:20 mfranz Exp $
+// $Id: viewer.cxx,v 1.23.2.2 2007-11-05 22:12:01 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -576,9 +576,7 @@ FGViewer::recalc ()
 void
 FGViewer::recalcLookFrom ()
 {
-
   sgVec3 right, forward;
-  // sgVec3 eye_pos;
   sgVec3 position_offset; // eye position offsets (xyz)
 
   // LOOKFROM mode...
@@ -631,6 +629,12 @@ FGViewer::recalcLookFrom ()
   // add the offsets including rotations to the translation vector
   sgAddVec3( _view_pos, position_offset );
 
+  sgdVec3 offset;
+  offset[0] = position_offset[0];
+  offset[1] = position_offset[1];
+  offset[2] = position_offset[2];
+  sgdAddVec3(_absolute_view_pos, _absolute_view_pos, offset);
+
   // multiply the OFFSETS (for heading and pitch) into the VIEW
   sgPostMultMat4(VIEW, VIEW_OFFSET);
 
@@ -642,7 +646,6 @@ FGViewer::recalcLookFrom ()
 void
 FGViewer::recalcLookAt ()
 {
-
   sgVec3 right, forward;
   sgVec3 eye_pos, at_pos;
   sgVec3 position_offset; // eye position offsets (xyz)
@@ -663,7 +666,7 @@ FGViewer::recalcLookAt ()
           _target_roll_deg, _target_pitch_deg, _target_heading_deg );
   }
   // calculate the "at" target object positon relative to eye or view's tile center...
-  Point3D center = globals->get_scenery()->get_next_center();
+  SGVec3d center = globals->get_scenery()->get_center();
   sgdVec3 dVec3;
   sgdSetVec3(dVec3, center[0], center[1], center[2]);
   sgdSubVec3(dVec3, _target_location->get_absolute_view_pos(), dVec3 );
@@ -679,7 +682,7 @@ FGViewer::recalcLookAt ()
           _roll_deg, _pitch_deg, _heading_deg );
   }
   // save the eye positon...
-  sgCopyVec3(eye_pos,  _location->get_view_pos(center));
+  sgCopyVec3(eye_pos, _location->get_view_pos(center));
 
   // copy data from location class to local items...
   copyLocationData();
@@ -703,7 +706,7 @@ FGViewer::recalcLookAt ()
     _roll_offset_deg * SG_DEGREES_TO_RADIANS, forward );
 
   // add in the position offsets
-  sgSetVec3( position_offset, _y_offset_m, _x_offset_m, _z_offset_m );
+  sgSetVec3( position_offset, _x_offset_m, _y_offset_m, _z_offset_m );
   sgXformVec3( position_offset, position_offset, VIEW_UP);
 
   // apply the Orientation offsets
@@ -712,6 +715,12 @@ FGViewer::recalcLookAt ()
   // add the Position offsets from object to the eye position
   sgAddVec3( eye_pos, eye_pos, position_offset );
 
+  sgdVec3 offset;
+  offset[0] = position_offset[0];
+  offset[1] = position_offset[1];
+  offset[2] = position_offset[2];
+  sgdAddVec3(_absolute_view_pos, _absolute_view_pos, offset);
+
   // add target offsets to at_position...
   sgSetVec3(target_position_offset, _target_z_offset_m,  _target_x_offset_m,
                                     _target_y_offset_m );
@@ -730,7 +739,7 @@ FGViewer::recalcLookAt ()
 void
 FGViewer::copyLocationData()
 {
-  Point3D center = globals->get_scenery()->get_center();
+  SGVec3d center = globals->get_scenery()->get_center();
   // Get our friendly vectors from the eye location...
   sgdCopyVec3(_absolute_view_pos, _location->get_absolute_view_pos());
   sgCopyVec3(_relative_view_pos, _location->get_view_pos(center));
