@@ -32,7 +32,6 @@ extern SGShadowVolume *shadows;
 FGModelMgr::FGModelMgr ()
   : _models(fgGetNode("/models", true)),
     _listener(new Listener(this))
-
 {
   _models->addChangeListener(_listener);
 }
@@ -54,13 +53,8 @@ FGModelMgr::init ()
 {
   vector<SGPropertyNode_ptr> model_nodes = _models->getChildren("model");
 
-  for (unsigned int i = 0; i < model_nodes.size(); i++) {
-    try {
+  for (unsigned int i = 0; i < model_nodes.size(); i++)
       add_model(model_nodes[i]);
-    } catch (const sg_throwable& t) {
-      SG_LOG(SG_GENERAL, SG_ALERT, t.getFormattedMessage() << t.getOrigin());
-    }
-  }
 }
 
 void
@@ -73,15 +67,23 @@ FGModelMgr::add_model (SGPropertyNode * node)
   instance->model = model;
   instance->node = node;
   SGModelLib *model_lib = globals->get_model_lib();
-  ssgBranch *object = (ssgBranch *)model_lib->load_model(
-      globals->get_fg_root(),
-      node->getStringValue("path",
-                           "Models/Geometry/glider.ac"),
-      globals->get_props(),
-      globals->get_sim_time_sec(), /*cache_object=*/false);
+
+  const char *path = node->getStringValue("path", "Models/Geometry/glider.ac");
+  ssgBranch *object;
+
+  try {
+    object = (ssgBranch *)model_lib->load_model(
+        globals->get_fg_root(),
+        path,
+        globals->get_props(),
+        globals->get_sim_time_sec(), /*cache_object=*/false);
+  } catch (const sg_throwable& t) {
+    SG_LOG(SG_GENERAL, SG_ALERT, "Error loading " << path << ":\n  "
+        << t.getFormattedMessage() << t.getOrigin());
+    return;
+  }
 
   model->init( object );
-  shadows->addOccluder((ssgBranch *)object, SGShadowVolume::occluderTypeTileObject);
 
 				// Set position and orientation either
 				// indirectly through property refs
@@ -167,6 +169,12 @@ FGModelMgr::update (double dt)
       model->setHeadingDeg(instance->heading_deg_node->getDoubleValue());
 
     instance->model->update();
+
+    if (shadows && !instance->shadow) {
+      ssgBranch *branch = (ssgBranch *)instance->model->getSceneGraph();
+      shadows->addOccluder(branch, SGShadowVolume::occluderTypeTileObject);
+      instance->shadow = true;
+    }
   }
 }
 
@@ -210,7 +218,8 @@ FGModelMgr::Instance::Instance ()
     elev_ft_node(0),
     roll_deg_node(0),
     pitch_deg_node(0),
-    heading_deg_node(0)
+    heading_deg_node(0),
+    shadow(false)
 {
 }
 
@@ -234,11 +243,7 @@ FGModelMgr::Listener::childAdded(SGPropertyNode * parent, SGPropertyNode * child
   if (strcmp(parent->getName(), "model") || strcmp(child->getName(), "load"))
     return;
 
-  try {
-    _mgr->add_model(parent);
-  } catch (const sg_throwable& t) {
-    SG_LOG(SG_GENERAL, SG_ALERT, t.getFormattedMessage() << t.getOrigin());
-  }
+  _mgr->add_model(parent);
 }
 
 void
@@ -258,7 +263,8 @@ FGModelMgr::Listener::childRemoved(SGPropertyNode * parent, SGPropertyNode * chi
 
     _mgr->_instances.erase(it);
     ssgBranch *branch = (ssgBranch *)instance->model->getSceneGraph();
-    shadows->deleteOccluder(branch);
+    if (shadows && instance->shadow)
+        shadows->deleteOccluder(branch);
     globals->get_scenery()->get_scene_graph()->removeKid(branch);
 
     delete instance;
