@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: navdb.cxx,v 1.15 2006-03-10 03:46:50 david Exp $
+// $Id: navdb.cxx,v 1.16.2.2 2007-10-14 07:42:01 durk Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -83,8 +83,10 @@ bool fgNavDBInit( FGAirportList *airports,
 #endif
 
         FGNavRecord *r = new FGNavRecord;
+        //nav_rec_ptr r = new FGNavRecord;
         in >> (*r);
         if ( r->get_type() > 95 ) {
+            delete r;
             break;
         }
 
@@ -125,8 +127,8 @@ bool fgNavDBInit( FGAirportList *airports,
         } else if ( r->get_type() == 12 || r->get_type() == 13) {
             // DME with ILS=12; standalone DME=13
             string str1( r->get_name() );
-            unsigned int loc1= str1.find( "TACAN", 0 );
-            unsigned int loc2 = str1.find( "VORTAC", 0 );
+            string::size_type loc1= str1.find( "TACAN", 0 );
+            string::size_type loc2 = str1.find( "VORTAC", 0 );
                        
             if( loc1 != string::npos || loc2 != string::npos ){
                  //cout << " name = " << r->get_name() ;
