@@ -18,24 +18,26 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: generic.cxx,v 1.17 2006-03-09 21:41:51 mfranz Exp $
+// $Id: generic.cxx,v 1.19.2.1 2007-07-29 13:59:01 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#include <string.h>		// strstr()
-#include <stdlib.h>		// strtod(), atoi()
+#include <string.h>                // strstr()
+#include <stdlib.h>                // strtod(), atoi()
 
 #include <simgear/debug/logstream.hxx>
 #include <simgear/io/iochannel.hxx>
 #include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/misc/stdint.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
 
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/util.hxx>
 
 #include "generic.hxx"
 
@@ -60,70 +62,115 @@ FGGeneric::FGGeneric(string& config) {
          return;
     }
 
-    _out_message.clear();
     SGPropertyNode *output = root.getNode("generic/output");
     if (output)
         read_config(output, _out_message);
 
-    _in_message.clear();
     SGPropertyNode *input = root.getNode("generic/input");
     if (input)
         read_config(input, _in_message);
 }
 
 FGGeneric::~FGGeneric() {
-    _out_message.clear();
-    _in_message.clear();
 }
 
 
 // generate the message
 bool FGGeneric::gen_message() {
-
     string generic_sentence;
     char tmp[255];
+    length = 0;
 
     double val;
 
     for (unsigned int i = 0; i < _out_message.size(); i++) {
 
-        if (i > 0)
-           generic_sentence += var_separator;
+        if (i > 0 && !binary_mode)
+            generic_sentence += var_separator;
 
         switch (_out_message[i].type) {
         case FG_INT:
             val = _out_message[i].offset +
                   _out_message[i].prop->getIntValue() * _out_message[i].factor;
-            snprintf(tmp, 255, _out_message[i].format.c_str(), (int)val);
+            if (binary_mode) {
+                *((int32_t*)&buf[length]) = (int32_t)val;
+                length += sizeof(int32_t);
+            } else {
+                snprintf(tmp, 255, _out_message[i].format.c_str(), (int)val);
+            }
             break;
 
         case FG_BOOL:
-            snprintf(tmp, 255, _out_message[i].format.c_str(),
-                               _out_message[i].prop->getBoolValue());
+            if (binary_mode) {
+                *((int8_t*)&buf[length])
+                          = _out_message[i].prop->getBoolValue() ? true : false;
+                length += sizeof(int8_t);
+            } else {
+                snprintf(tmp, 255, _out_message[i].format.c_str(),
+                                   _out_message[i].prop->getBoolValue());
+            }
             break;
 
         case FG_DOUBLE:
             val = _out_message[i].offset +
                 _out_message[i].prop->getFloatValue() * _out_message[i].factor;
-            snprintf(tmp, 255, _out_message[i].format.c_str(), (float)val);
+            if (binary_mode) {
+                *((double*)&buf[length]) = val;
+                length += sizeof(double);
+            } else {
+                snprintf(tmp, 255, _out_message[i].format.c_str(), (float)val);
+            }
             break;
 
         default: // SG_STRING
-             snprintf(tmp, 255, _out_message[i].format.c_str(),
-                                _out_message[i].prop->getStringValue());
+            if (binary_mode) {
+                const char *strdata = _out_message[i].prop->getStringValue();
+                int strlength = strlen(strdata);
+
+                /* Format for strings is 
+                 * [length as int, 4 bytes][ASCII data, length bytes]
+                 */
+                *((int32_t*)&buf[length]) = strlength;
+                                length += sizeof(int32_t);
+                                strncpy(&buf[length], strdata, strlength);
+                                length += strlength; 
+                /* FIXME padding for alignment? Something like: 
+                 * length += (strlength % 4 > 0 ? sizeof(int32_t) - strlength % 4 : 0;
+                 */
+            } else {
+                snprintf(tmp, 255, _out_message[i].format.c_str(),
+                                   _out_message[i].prop->getStringValue());
+            }
         }
 
-        generic_sentence += tmp;
+        if (!binary_mode) {
+            generic_sentence += tmp;
+        }
     }
 
-    /* After each lot of variables has been added, put the line separator
-     * char/string
-     */
-    generic_sentence += line_separator;
- 
-            
-    length =  generic_sentence.length();
-    strncpy( buf, generic_sentence.c_str(), length );
+    if (!binary_mode) {
+        /* After each lot of variables has been added, put the line separator
+         * char/string
+         */
+        generic_sentence += line_separator;
+
+        length =  generic_sentence.length();
+        strncpy( buf, generic_sentence.c_str(), length );
+    } else {
+        // add the footer to the packet ("line")
+        switch (binary_footer_type) {
+            case FOOTER_LENGTH:
+                binary_footer_value = length;
+                break;
+
+            case FOOTER_MAGIC:
+                break;
+        }
+        if (binary_footer_type != FOOTER_NONE) {
+            *((int32_t*)&buf[length]) = binary_footer_value;
+            length += sizeof(int32_t);
+        }
+    }
 
     return true;
 }
@@ -133,8 +180,12 @@ bool FGGeneric::parse_message() {
     double val;
     int i = -1;
 
+    if (binary_mode)
+        SG_LOG( SG_IO, SG_ALERT,
+                "generic protocol: binary mode input is not yet implemented.");
+        
     while ((++i < (int)_in_message.size()) &&
-           p1 && strcmp(p1, line_separator.c_str())) {
+            p1 && strcmp(p1, line_separator.c_str())) {
 
         p2 = strstr(p1, var_separator.c_str());
         if (p2) {
@@ -186,6 +237,13 @@ bool FGGeneric::open() {
 
     set_enabled( true );
 
+    if ( get_direction() == SG_IO_OUT && ! preamble.empty() ) {
+        if ( ! io->write( preamble.c_str(), preamble.size() ) ) {
+            SG_LOG( SG_IO, SG_WARN, "Error writing preamble." );
+            return false;
+        }
+    }
+
     return true;
 }
 
@@ -217,6 +275,13 @@ bool FGGeneric::process() {
 bool FGGeneric::close() {
     SGIOChannel *io = get_io_channel();
 
+    if ( get_direction() == SG_IO_OUT && ! postamble.empty() ) {
+        if ( ! io->write( postamble.c_str(), postamble.size() ) ) {
+            SG_LOG( SG_IO, SG_ALERT, "Error writing postamble." );
+            return false;
+        }
+    }
+
     set_enabled( false );
 
     if ( ! io->close() ) {
@@ -230,6 +295,9 @@ bool FGGeneric::close() {
 void
 FGGeneric::read_config(SGPropertyNode *root, vector<_serial_prot> &msg)
 {
+    binary_mode = root->getBoolValue("binary_mode");
+
+    if (!binary_mode) {
         /* These variables specified in the $FG_ROOT/data/Protocol/xxx.xml
          * file for each format
          *
@@ -237,8 +305,10 @@ FGGeneric::read_config(SGPropertyNode *root, vector<_serial_prot> &msg)
          * line_sep_string = the string/charachter to place at the end of each
          *                   lot of variables
          */
-    var_sep_string = root->getStringValue("var_separator");
-    line_sep_string = root->getStringValue("line_separator");
+        preamble = fgUnescape(root->getStringValue("preamble"));
+        postamble = fgUnescape(root->getStringValue("postamble"));
+        var_sep_string = fgUnescape(root->getStringValue("var_separator"));
+        line_sep_string = fgUnescape(root->getStringValue("line_separator"));
 
         if ( var_sep_string == "newline" )
                 var_separator = '\n';
@@ -269,19 +339,34 @@ FGGeneric::read_config(SGPropertyNode *root, vector<_serial_prot> &msg)
                 line_separator = '\v';
         else
                 line_separator = line_sep_string;
-
+    } else {
+        binary_footer_type = FOOTER_NONE; // default choice
+        if ( root->hasValue("binary_footer") ) {
+            string footer_type = root->getStringValue("binary_footer");
+            if ( footer_type == "length" )
+                binary_footer_type = FOOTER_LENGTH;
+            else if ( footer_type.substr(0, 5) == "magic" ) {
+                binary_footer_type = FOOTER_MAGIC;
+                binary_footer_value = strtol(footer_type.substr(6, 
+                            footer_type.length() - 6).c_str(), (char**)0, 0);
+            } else if ( footer_type != "none" )
+                SG_LOG(SG_IO, SG_ALERT,
+                       "generic protocol: Undefined generic binary protocol"
+                                           "footer, using no footer.");
+        }
+    }
 
     vector<SGPropertyNode_ptr> chunks = root->getChildren("chunk");
     for (unsigned int i = 0; i < chunks.size(); i++) {
 
         _serial_prot chunk;
 
-     // chunk.name = chunks[i]->getStringValue("name");
-        chunk.format = chunks[i]->getStringValue("format", "%d");
+        // chunk.name = chunks[i]->getStringValue("name");
+        chunk.format = fgUnescape(chunks[i]->getStringValue("format", "%d"));
         chunk.offset = chunks[i]->getDoubleValue("offset");
         chunk.factor = chunks[i]->getDoubleValue("factor", 1.0);
 
-        string node = chunks[i]->getStringValue("node");
+        string node = chunks[i]->getStringValue("node", "/null");
         chunk.prop = fgGetNode(node.c_str(), true);
 
         string type = chunks[i]->getStringValue("type");
