@@ -22,7 +22,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: httpd.cxx,v 1.10 2006-02-21 01:19:47 mfranz Exp $
+// $Id: httpd.cxx,v 1.10.2.3 2007-11-03 17:01:03 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -31,6 +31,7 @@
 
 #include <simgear/compiler.h>
 
+#include <algorithm>		// sort()
 #include <stdlib.h>		// atoi() atof()
 
 #include STL_STRING
@@ -76,15 +77,22 @@ bool FGHttpd::process() {
 bool FGHttpd::close() {
     SG_LOG( SG_IO, SG_INFO, "closing FGHttpd" );   
 
-    // the following delete causes a seg fault, gdb is not helpful.
-    // delete server;
-
+    delete server;
     set_enabled( false );
 
     return true;
 }
 
 
+class CompareNodes {
+public:
+    bool operator() (const SGPropertyNode *a, const SGPropertyNode *b) const {
+        int r = strcmp(a->getName(), b->getName());
+        return r ? r < 0 : a->getIndex() < b->getIndex();
+    }
+};
+
+
 // Handle http GET requests
 void HttpdChannel::foundTerminator (void) {
     
@@ -195,8 +203,15 @@ void HttpdChannel::foundTerminator (void) {
             response += "\"</H3>";
             response += getTerminator();
 
-            for (int i = 0; i < node->nChildren(); i++) {
-                SGPropertyNode *child = node->getChild(i);
+
+            vector<SGPropertyNode *> children;
+            for (int i = 0; i < node->nChildren(); i++)
+                children.push_back(node->getChild(i));
+            std::sort(children.begin(), children.end(), CompareNodes());
+
+            vector<SGPropertyNode *>::iterator it, end = children.end();
+            for (it = children.begin(); it != end; ++it) {
+                SGPropertyNode *child = *it;
                 string name = child->getDisplayName(true);
                 string line = "";
                 if ( child->nChildren() > 0 ) {
