@@ -46,7 +46,7 @@ SG_USING_STD(string);
 
 
 // These constants are provided so that the ident command can list file versions.
-const char sFG_MULTIPLAY_BID[] = "$Id: multiplay.cxx,v 1.12 2006-02-21 01:19:47 mfranz Exp $";
+const char sFG_MULTIPLAY_BID[] = "$Id: multiplay.cxx,v 1.15 2006-10-10 05:17:07 frohlich Exp $";
 const char sFG_MULTIPLAY_HID[] = FG_MULTIPLAY_HID;
 
 
@@ -156,7 +156,7 @@ bool FGMultiplay::process() {
     // first the aprioriate structure for the geodetic one
     SGGeod geod = SGGeod::fromRadFt(lon, lat, ifce->get_Altitude());
     // Convert to cartesion coordinate
-    motionInfo.position = geod;
+    motionInfo.position = SGVec3d::fromGeod(geod);
     
     // The quaternion rotating from the earth centered frame to the
     // horizontal local frame
@@ -198,14 +198,68 @@ bool FGMultiplay::process() {
     // now send the properties
     PropertyMap::iterator it;
     for (it = mPropertyMap.begin(); it != mPropertyMap.end(); ++it) {
-      FGFloatPropertyData pData;
-      pData.id = it->first;
-      pData.value = it->second->getFloatValue();
+      FGPropertyData* pData = new FGPropertyData;
+      pData->id = it->first;
+      pData->type = it->second->getType();
+      
+      switch (pData->type) {
+        case SGPropertyNode::INT:        
+        case SGPropertyNode::LONG:       
+        case SGPropertyNode::BOOL:
+          pData->int_value = it->second->getIntValue();
+          break;
+        case SGPropertyNode::FLOAT:
+        case SGPropertyNode::DOUBLE:
+          pData->float_value = it->second->getFloatValue();
+          break;
+        case SGPropertyNode::STRING:
+        case SGPropertyNode::UNSPECIFIED:
+          {
+            // FIXME: We assume unspecified are strings for the moment.
+
+            const char* cstr = it->second->getStringValue();
+            int len = strlen(cstr);
+            
+            if (len > 0)
+            {            
+              pData->string_value = new char[len + 1];
+              strcpy(pData->string_value, cstr);
+            }
+            else
+            {
+              // Size 0 - ignore
+              pData->string_value = 0;            
+            }
+
+            //cout << " Sending property " << pData->id << " " << pData->type << " " <<  pData->string_value << "\n";
+            break;        
+          }
+        default:
+          // FIXME Currently default to a float. 
+          //cout << "Unknown type when iterating through props: " << pData->type << "\n";
+          pData->float_value = it->second->getFloatValue();
+          break;
+      }
+      
       motionInfo.properties.push_back(pData);
     }
 
     FGMultiplayMgr* mpmgr = globals->get_multiplayer_mgr();
     mpmgr->SendMyPosition(motionInfo);
+    
+    // Now remove the data
+    std::vector<FGPropertyData*>::const_iterator propIt;
+    std::vector<FGPropertyData*>::const_iterator propItEnd;
+    propIt = motionInfo.properties.begin();
+    propItEnd = motionInfo.properties.end();
+
+    //cout << "Deleting data\n";
+
+    while (propIt != propItEnd)
+    {
+      delete *propIt;
+      propIt++;
+    }    
   }
 
   return true;
@@ -219,13 +273,19 @@ bool FGMultiplay::process() {
 ******************************************************************/
 bool FGMultiplay::close() {
 
+  FGMultiplayMgr *mgr = globals->get_multiplayer_mgr();
+
+  if (mgr == 0) {
+    return false;
+  }
+
   if (get_direction() == SG_IO_IN) {
 
-    globals->get_multiplayer_mgr()->Close();
+    mgr->Close();
 
   } else if (get_direction() == SG_IO_OUT) {
 
-    globals->get_multiplayer_mgr()->Close();
+    mgr->Close();
 
   }
 
