@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: native_gui.cxx,v 1.19 2006-02-21 01:19:47 mfranz Exp $
+// $Id: native_gui.cxx,v 1.19.2.1 2007-03-01 17:39:49 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -309,7 +309,7 @@ void FGNetGUI2Props( FGNetGUI *net ) {
         fgSetDouble( "/instrumentation/nav[0]/frequencies/selected-mhz",
                      net->tuned_freq );
         fgSetBool( "/instrumentation/nav[0]/in-range", net->in_range );
-        fgSetDouble( "/instrumentation/dme/distance-nm", net->dist_nm );
+        fgSetDouble( "/instrumentation/dme/indicated-distance-nm", net->dist_nm );
         fgSetDouble( "/instrumentation/nav[0]/heading-needle-deflection",
                      net->course_deviation_deg );
         fgSetDouble( "/instrumentation/nav[0]/gs-needle-deflection",
