@@ -20,7 +20,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: props.cxx,v 1.18 2006-02-21 01:19:47 mfranz Exp $
+// $Id: props.cxx,v 1.18.2.2 2007-02-20 21:12:44 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -272,10 +272,10 @@ PropsChannel::foundTerminator()
 	} else if ( command == "set" ) {
 	    if ( tokens.size() >= 2 ) {
                 string value, tmp;
-                if ( tokens.size() == 3 ) {
-                    value = tokens[2];
-                } else {
-                    value = "";
+                for (unsigned int i = 2; i < tokens.size(); i++) {
+                    if (i > 2)
+                        value += " ";
+                    value += tokens[i];
                 }
                 node->getNode( tokens[1].c_str(), true )
                     ->setStringValue(value.c_str());
@@ -411,8 +411,7 @@ prompt             switch to interactive mode (default)\r\n\
 pwd                display your current path\r\n\
 quit               terminate connection\r\n\
 run <command>      run built in command\r\n\
-set <var> <val>    set <var> to a new <val>\r\n\
-show <var>         synonym for get\r\n";
+set <var> <val>    set <var> to a new <val>\r\n";
 	    push( msg );
 	}
     }
