@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: tilemgr.cxx,v 1.49 2005/10/26 09:03:49 ehofman Exp $
+// $Id: tilemgr.cxx,v 1.56.2.1 2007-06-24 08:30:56 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -40,6 +40,7 @@
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 #include <Main/viewer.hxx>
+#include <Scripting/NasalSys.hxx>
 
 #include "newcache.hxx"
 #include "scenery.hxx"
@@ -47,7 +48,7 @@
 
 #define TEST_LAST_HIT_CACHE
 
-#if defined(ENABLE_THREADS) && ENABLE_THREADS
+#if defined(ENABLE_THREADS)
 SGLockedQueue<FGTileEntry *> FGTileMgr::attach_queue;
 SGLockedQueue<FGDeferredModel *> FGTileMgr::model_queue;
 #else
@@ -92,7 +93,7 @@ int FGTileMgr::init() {
     }
 
     while ( ! model_queue.empty() ) {
-#if defined(ENABLE_THREADS) && ENABLE_THREADS
+#if defined(ENABLE_THREADS)
         FGDeferredModel* dm = model_queue.pop();
 #else
         FGDeferredModel* dm = model_queue.front();
@@ -103,8 +104,6 @@ int FGTileMgr::init() {
     loader.reinit();
 #endif
 
-    hit_list.clear();
-
     state = Inited;
 
     previous_bucket.make_bad();
@@ -164,9 +163,10 @@ void FGTileMgr::schedule_needed( double vis, const SGBucket& curr_bucket) {
         SG_LOG( SG_TERRAIN, SG_ALERT,
                 "Attempting to schedule tiles for bogus lon and lat  = ("
                 << longitude << "," << latitude << ")" );
+        return;		// FIXME
         SG_LOG( SG_TERRAIN, SG_ALERT,
                 "This is a FATAL error.  Exiting!" );
-        exit(-1);        
+        exit(-1);
     }
 
     SG_LOG( SG_TERRAIN, SG_INFO,
@@ -299,7 +299,7 @@ void FGTileMgr::update_queues()
 
             // cout << "loading next model ..." << endl;
             // load the next tile in the queue
-#if defined(ENABLE_THREADS) && ENABLE_THREADS
+#if defined(ENABLE_THREADS)
             FGDeferredModel* dm = model_queue.pop();
 #else
             FGDeferredModel* dm = model_queue.front();
@@ -316,15 +316,22 @@ void FGTileMgr::update_queues()
                         globals->get_model_lib()->load_model( ".",
                                                   dm->get_model_path(),
                                                   globals->get_props(),
-                                                  globals->get_sim_time_sec() );
+                                                  globals->get_sim_time_sec(),
+                                                  dm->get_cache_state(),
+                                                  new FGNasalModelData );
                     if ( obj_model != NULL ) {
                         dm->get_obj_trans()->addKid( obj_model );
                         shadows->addOccluder( (ssgBranch *) obj_model->getParent(0),
                             SGShadowVolume::occluderTypeTileObject,
                             (ssgBranch *) dm->get_tile()->get_terra_transform());
                     }
-                } catch (const sg_exception& exc) {
-                    SG_LOG( SG_ALL, SG_ALERT, exc.getMessage() );
+                } catch (const sg_io_exception& exc) {
+					string m(exc.getMessage());
+					m += " ";
+					m += exc.getLocation().asString();
+                    SG_LOG( SG_ALL, SG_ALERT, m );
+                } catch (const sg_exception& exc) { // XXX may be redundant
+                    SG_LOG( SG_ALL, SG_ALERT, exc.getMessage());
                 }
                 
                 dm->get_tile()->dec_pending_models();
@@ -337,7 +344,7 @@ void FGTileMgr::update_queues()
     loader.update();
 
     if ( !attach_queue.empty() ) {
-#if defined(ENABLE_THREADS) && ENABLE_THREADS
+#if defined(ENABLE_THREADS)
         FGTileEntry* e = attach_queue.pop();
 #else
         FGTileEntry* e = attach_queue.front();
@@ -489,7 +496,7 @@ int FGTileMgr::tile_filter_cb( ssgEntity *, int )
 bool FGTileMgr::scenery_available(double lat, double lon, double range_m)
 {
   // sanity check (unfortunately needed!)
-  if ( lon < -180.0 || lon > 180.0 || lat < -90.0 || lat > 90.0 )
+  if ( lon <= -180.0 || lon >= 180.0 || lat <= -90.0 || lat >= 90.0 )
     return false;
   
   SGBucket bucket(lon, lat);
