@@ -74,6 +74,59 @@ static naRef f_getType(naContext c, naRef me, int argc, naRef* args)
     return NASTR(t);
 }
 
+static naRef f_getAttribute(naContext c, naRef me, int argc, naRef* args)
+{
+    NODEARG();
+    if(naVec_size(argv) == 0) return naNum((*node)->getAttributes());
+    naRef val = naVec_get(argv, 0);
+    char *a = naStr_data(val);
+    SGPropertyNode::Attribute attr;
+    if(!a) a = "";
+    if(!strcmp(a, "children"))         return naNum((*node)->nChildren());
+    else if(!strcmp(a, "listeners"))   return naNum((*node)->nListeners());
+    else if(!strcmp(a, "references"))  return naNum(node->getNumRefs());
+    else if(!strcmp(a, "tied"))        return naNum((*node)->isTied());
+    else if(!strcmp(a, "alias"))       return naNum((*node)->isAlias());
+    else if(!strcmp(a, "read"))        attr = SGPropertyNode::READ;
+    else if(!strcmp(a, "write"))       attr = SGPropertyNode::WRITE;
+    else if(!strcmp(a, "archive"))     attr = SGPropertyNode::ARCHIVE;
+    else if(!strcmp(a, "trace-read"))  attr = SGPropertyNode::TRACE_READ;
+    else if(!strcmp(a, "trace-write")) attr = SGPropertyNode::TRACE_WRITE;
+    else if(!strcmp(a, "userarchive")) attr = SGPropertyNode::USERARCHIVE;
+    else {
+        naRuntimeError(c, "props.getAttribute() with invalid attribute");
+        return naNil();
+    }
+    return naNum((*node)->getAttribute(attr));
+}
+
+static naRef f_setAttribute(naContext c, naRef me, int argc, naRef* args)
+{
+    NODEARG();
+    naRef val = naVec_get(argv, 0);
+    if(naVec_size(argv) == 1 && naIsNum(val)) {
+        naRef ret = naNum((*node)->getAttributes());
+        (*node)->setAttributes((int)val.num);
+        return ret;
+    }
+    SGPropertyNode::Attribute attr;
+    char *a = naStr_data(val);
+    if(!a) a = "";
+    if(!strcmp(a, "read"))             attr = SGPropertyNode::READ;
+    else if(!strcmp(a, "write"))       attr = SGPropertyNode::WRITE;
+    else if(!strcmp(a, "archive"))     attr = SGPropertyNode::ARCHIVE;
+    else if(!strcmp(a, "trace-read"))  attr = SGPropertyNode::TRACE_READ;
+    else if(!strcmp(a, "trace-write")) attr = SGPropertyNode::TRACE_WRITE;
+    else if(!strcmp(a, "userarchive")) attr = SGPropertyNode::USERARCHIVE;
+    else {
+        naRuntimeError(c, "props.setAttribute() with invalid attribute");
+        return naNil();
+    }
+    naRef ret = naNum((*node)->getAttribute(attr));
+    (*node)->setAttribute(attr, naTrue(naVec_get(argv, 1)) ? true : false);
+    return ret;
+}
+
 static naRef f_getName(naContext c, naRef me, int argc, naRef* args)
 {
     NODEARG();
@@ -97,8 +150,9 @@ static naRef f_getValue(naContext c, naRef me, int argc, naRef* args)
     case SGPropertyNode::STRING:
     case SGPropertyNode::UNSPECIFIED:
         return NASTR((*node)->getStringValue());
+    default:
+        return naNil();
     }
-    return naNil();
 }
 
 static naRef f_setValue(naContext c, naRef me, int argc, naRef* args)
@@ -106,7 +160,12 @@ static naRef f_setValue(naContext c, naRef me, int argc, naRef* args)
     NODEARG();
     naRef val = naVec_get(argv, 0);
     if(naIsString(val)) (*node)->setStringValue(naStr_data(val));
-    else                (*node)->setDoubleValue(naNumValue(val).num);
+    else {
+        naRef n = naNumValue(val);
+        if(naIsNil(n))
+            naRuntimeError(c, "props.setValue() with non-number");
+        (*node)->setDoubleValue(naNumValue(val).num);
+    }
     return naNil();
 }
 
@@ -119,6 +178,8 @@ static naRef f_setIntValue(naContext c, naRef me, int argc, naRef* args)
     // Junk to pacify the gcc-2.95.3 optimizer:
     naRef tmp0 = naVec_get(argv, 0);
     naRef tmp1 = naNumValue(tmp0);
+    if(naIsNil(tmp1))
+        naRuntimeError(c, "props.setIntValue() with non-number");
     double tmp2 = tmp1.num;
     int iv = (int)tmp2;
 
@@ -137,7 +198,10 @@ static naRef f_setBoolValue(naContext c, naRef me, int argc, naRef* args)
 static naRef f_setDoubleValue(naContext c, naRef me, int argc, naRef* args)
 {
     NODEARG();
-    (*node)->setDoubleValue(naNumValue(naVec_get(argv, 0)).num);
+    naRef r = naNumValue(naVec_get(argv, 0));
+    if(naIsNil(r))
+        naRuntimeError(c, "props.setDoubleValue() with non-number");
+    (*node)->setDoubleValue(r.num);
     return naNil();
 }
 
@@ -157,10 +221,15 @@ static naRef f_getChild(naContext c, naRef me, int argc, naRef* args)
     naRef idx = naNumValue(naVec_get(argv, 1));
     bool create = naTrue(naVec_get(argv, 2));
     SGPropertyNode* n;
-    if(naIsNil(idx) || !naIsNum(idx)) {
-        n = (*node)->getChild(naStr_data(child), create);
-    } else {
-        n = (*node)->getChild(naStr_data(child), (int)idx.num, create);
+    try {
+        if(naIsNil(idx) || !naIsNum(idx)) {
+            n = (*node)->getChild(naStr_data(child), create);
+        } else {
+            n = (*node)->getChild(naStr_data(child), (int)idx.num, create);
+        }
+    } catch (const string& err) {
+        naRuntimeError(c, (char *)err.c_str());
+        return naNil();
     }
     if(!n) return naNil();
     return propNodeGhostCreate(c, n);
@@ -178,10 +247,15 @@ static naRef f_getChildren(naContext c, naRef me, int argc, naRef* args)
         // Get all children of a specified name
         naRef name = naVec_get(argv, 0);
         if(!naIsString(name)) return naNil();
-        vector<SGPropertyNode_ptr> children
-            = (*node)->getChildren(naStr_data(name));
-        for(unsigned int i=0; i<children.size(); i++)
-            naVec_append(result, propNodeGhostCreate(c, children[i]));
+        try {
+            vector<SGPropertyNode_ptr> children
+                = (*node)->getChildren(naStr_data(name));
+            for(unsigned int i=0; i<children.size(); i++)
+                naVec_append(result, propNodeGhostCreate(c, children[i]));
+        } catch (const string& err) {
+            naRuntimeError(c, (char *)err.c_str());
+            return naNil();
+        }
     }
     return result;
 }
@@ -192,7 +266,11 @@ static naRef f_removeChild(naContext c, naRef me, int argc, naRef* args)
     naRef child = naVec_get(argv, 0);
     naRef index = naVec_get(argv, 1);
     if(!naIsString(child) || !naIsNum(index)) return naNil();
-    (*node)->removeChild(naStr_data(child), (int)index.num, false);
+    try {
+        (*node)->removeChild(naStr_data(child), (int)index.num, false);
+    } catch (const string& err) {
+        naRuntimeError(c, (char *)err.c_str());
+    }
     return naNil();
 }
 
@@ -208,10 +286,15 @@ static naRef f_removeChildren(naContext c, naRef me, int argc, naRef* args)
         // Remove all children of a specified name
         naRef name = naVec_get(argv, 0);
         if(!naIsString(name)) return naNil();
-        vector<SGPropertyNode_ptr> children
-            = (*node)->removeChildren(naStr_data(name), false);
-        for(unsigned int i=0; i<children.size(); i++)
-            naVec_append(result, propNodeGhostCreate(c, children[i]));
+        try {
+            vector<SGPropertyNode_ptr> children
+                = (*node)->removeChildren(naStr_data(name), false);
+            for(unsigned int i=0; i<children.size(); i++)
+                naVec_append(result, propNodeGhostCreate(c, children[i]));
+        } catch (const string& err) {
+            naRuntimeError(c, (char *)err.c_str());
+            return naNil();
+        }
     }
     return result;
 }
@@ -222,7 +305,13 @@ static naRef f_getNode(naContext c, naRef me, int argc, naRef* args)
     naRef path = naVec_get(argv, 0);
     bool create = naTrue(naVec_get(argv, 1));
     if(!naIsString(path)) return naNil();
-    SGPropertyNode* n = (*node)->getNode(naStr_data(path), create);
+    SGPropertyNode* n;
+    try {
+        n = (*node)->getNode(naStr_data(path), create);
+    } catch (const string& err) {
+        naRuntimeError(c, (char *)err.c_str());
+        return naNil();
+    }
     return propNodeGhostCreate(c, n);
 }
 
@@ -238,9 +327,11 @@ static naRef f_globals(naContext c, naRef me, int argc, naRef* args)
 
 static struct {
     naCFunction func;
-    char* name;
+    const char* name;
 } propfuncs[] = {
     { f_getType, "_getType" },
+    { f_getAttribute, "_getAttribute" },
+    { f_setAttribute, "_setAttribute" },
     { f_getName, "_getName" },
     { f_getIndex, "_getIndex" },
     { f_getValue, "_getValue" },
