@@ -19,7 +19,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: fg_fx.cxx,v 1.19 2006-02-21 01:20:04 mfranz Exp $
+// $Id: fg_fx.cxx,v 1.22.2.2 2007-05-06 21:04:50 mfranz Exp $
 
 #ifdef _MSC_VER
 #pragma warning (disable: 4786)
@@ -74,7 +74,7 @@ FGFX::init()
    string path_str = node->getStringValue("path");
    SGPath path( globals->get_fg_root() );
    if (path_str.empty()) {
-      SG_LOG(SG_GENERAL, SG_ALERT, "Incorrect path in configuration file.");
+      SG_LOG(SG_GENERAL, SG_ALERT, "No path in /sim/sound/path");
       return;
    }
 
@@ -87,7 +87,7 @@ FGFX::init()
       readProperties(path.str(), &root);
    } catch (const sg_exception &) {
       SG_LOG(SG_GENERAL, SG_ALERT,
-       "Incorrect path specified in configuration file");
+       "Error reading file '" << path.str() << '\'');
       return;
    }
 
@@ -129,6 +129,10 @@ FGFX::update (double dt)
 {
     SGSoundMgr *smgr = globals->get_soundmgr();
 
+    if (smgr->is_working() == false) {
+        return;
+    }
+
     // command sound manger
     bool pause = _pause->getBoolValue();
     if ( pause != last_pause ) {
@@ -183,14 +187,17 @@ FGFX::update (double dt)
 void
 FGFX::play_message( SGSoundSample *_sample )
 {
-    _sample->set_volume( 1.0 );
     _samplequeue.push( _sample );
 }
 void
-FGFX::play_message( const string path, const string fname )
+FGFX::play_message( const string path, const string fname, double volume )
 {
+    if (globals->get_soundmgr()->is_working() == false) {
+        return;
+    }
     SGSoundSample *sample;
     sample = new SGSoundSample( path.c_str(), fname.c_str() );
+    sample->set_volume( volume );
     play_message( sample );
 }
 
