@@ -18,8 +18,11 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: electrical.cxx,v 1.34 2006-02-21 01:20:12 mfranz Exp $
+// $Id: electrical.cxx,v 1.35.2.1 2007-05-11 18:00:05 mfranz Exp $
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
@@ -330,30 +333,11 @@ bool FGElectricalConnector::get_state() {
 
 
 FGElectricalSystem::FGElectricalSystem ( SGPropertyNode *node ) :
-    name("electrical"),
-    num(0),
-    path(""),
+    name(node->getStringValue("name", "electrical")),
+    num(node->getIntValue("number", 0)),
+    path(node->getStringValue("path")),
     enabled(false)
 {
-    int i;
-    for ( i = 0; i < node->nChildren(); ++i ) {
-        SGPropertyNode *child = node->getChild(i);
-        string cname = child->getName();
-        string cval = child->getStringValue();
-        if ( cname == "name" ) {
-            name = cval;
-        } else if ( cname == "number" ) {
-            num = child->getIntValue();
-        } else if ( cname == "path" ) {
-            path = cval;
-        } else {
-            SG_LOG( SG_SYSTEMS, SG_WARN,
-                    "Error in electrical system config logic" );
-            if ( name.length() ) {
-                SG_LOG( SG_SYSTEMS, SG_WARN, "Section = " << name );
-            }
-        }
-    }
 }
 
 
@@ -389,8 +373,8 @@ void FGElectricalSystem::init () {
         config.append( path );
 
         // load an obsolete xml configuration
-        SG_LOG( SG_ALL, SG_ALERT,
-                "Reading xml electrical system model from "
+        SG_LOG( SG_ALL, SG_WARN,
+                "Reading deprecated xml electrical system model from\n    "
                 << config.str() );
         try {
             readProperties( config.str(), config_props );
