@@ -3,6 +3,10 @@
 //
 // This file is in the Public Domain and comes with no warranty.
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/sg_inlines.h>
@@ -41,7 +45,7 @@ FGSystemMgr::FGSystemMgr ()
                 enabled = true;
             } else {
                 SG_LOG( SG_ALL, SG_ALERT,
-                        "Detected an internal inconsistancy in the systems");
+                        "Detected an internal inconsistency in the systems");
                 SG_LOG( SG_ALL, SG_ALERT,
                         " system specification file.  See earlier errors for" );
                 SG_LOG( SG_ALL, SG_ALERT,
