@@ -18,7 +18,9 @@ Date: Fri, 24 Apr 1998 07:33:51 -0800
 #include <stdio.h>
 #include <stdlib.h>
 
-#include FG_GLUT_H
+#include <simgear/compiler.h>
+
+#include SG_GLUT_H
 
 
 void getPrints ( GLenum token, char *string )
